	org $0FF9 - 2
	dw  $0FF9
	
; $00,$71,$32,$53,$34,$4F,$2E,$67
; $38,$29,$42,$31,$41,$6F,$46,$51

; TODO (done)
; +++ finish / pack title screen (hires logo)
; +++ create docs
; +++ intro

; Plus/4

; +++ bugfix: prisoner following: when walking right, the prisoner goes too high
; +++ NTSC compatibility added (on NTSC, the playing area is narrower)
; +++ all colors fixed
; +++ removed all C64 sprite code, sprite handling completely rewritten
; +++ updated code and charset to make room for 16 extra chars (player's sprite)
; +++ new custom graphics for player, re-implemented shooting muzzle flash
; +++ new (native) custom music (old one had an "octave jump" problem)
; +++ new (native) custom sounds,including step sounds 
; +++ a new castle recreated for the side panel (using mostly existing characters)
; +++ explosion flashing color feature added back
; +++ fixed random number generator; yellow crates will (randomly) now have loot
; +++ improved game irq for very precise font swap

; C64

; +++ bugfix: fixed garbled chars on first floor (present in C64 version)
; +++ bugfix: bullet 1 moving right never stopped (causing random odd bugs)
; +++ bugfix: picking up jewels sometimes drew the empty crate in the wrong place
; +++ side panel fixed (all hires colors)
; +++ typos fixed, space from before "!" removed
; +++ improved digits 1,4,7 in the 2x2 font: proper center alignment
; +++ added item pickup sound for jewels
; +++ color improvements (extra shades) added to map, basement has darker walls
; +++ vase redrawn, the original is a weird mix of multi/hires (thanks KiCHY)
; +++ wall corner fixed on ground floor
; +++ table next to officers fixed (red color added)
; +++ "gemstones" renamed to jewels (for consistency)
; +++ apostrophe added to 2x2 font (and into scroll text)
; +++ special apostrophe (custom chars) for highscores screen
; +++ redrawn gun for enemy facing left
; +++ removed flash and other small glitches from intro screen
; +++ added credits to intro programmer screen
; +++ "right" allows intro screens to be quickly toggled
; +++ intro auto-detects which joy is used
; +++ added keyboard controls (press space from intro)
; +++ swapped colors in title bitmap to avoid ff15 change
; +++ add scratches to logo from Atari version
; +++ during explosion, disable screen update and sync to bottom of screen
; +++ prisoner and player gfx (floor color) fixed
; +++ screen transition out synced to frame (instead of using delay) - no tearing
; +++ cheat menu (with custom 1x1 font) added
;   - enemies die from one bullet
;   - respawn enemies
;   - enemy speed
;   - invulnerability
;   - unlimited ammo
;   - player speed
;   - all doors are open
;   - lifts open
;   - safe explosives
;   - lift hotkey
;   - nuke hotkey
;   - ghost mode hotkey
; +++ added red warning color for "hits" getting dangerous, ammo getting low
; +++ added extra map limits (for ghost mode)
; +++ moved all variables to zp

; Constants
PRISONER_CHR			= $D0

; Generic zeropage vectors
zp02				= $02 ; 16bit
zp04				= $04 ; 16bit
zp06				= $06 ; 16bit
zp08				= $08 ; 16bit
zp0A				= $0A ; 16bit

cheat_flags			= $10
cheat_selection			= $1F

sprite_src_ptr			= $2E
sprite_chr_ptr			= $30
sprite_col_ptr			= $32
sprite_add			= $34
sprite_passed			= $36		; 16 bit address, screen position after the player's sprite

HS_FADE_INDEX			= $3C
HS_FADE_STATE			= $3D
NAME_ENTRY_CURSOR_POS		= $3E
VAR_ENTRY_FLAG			= $3F
VAR_LETTERS_POS			= $40
VAR_LETTER_POS_X2		= $41
enter_your_name_tick		= $42
VAR_MISSION_TEMP		= $43
VAR_ELEV_CURR_FLOOR		= $44
HS_POS				= $45
VAR_PRISONER_X			= $46
VAR_PRISONER_Y			= $47
VAR_PRISONER_LO_TEMP1		= $48
VAR_PRISONER_HI_TEMP1		= $49
VAR_PRISONER_FLAGS		= $4A
VAR_PRISONER_LO_TEMP2		= $4B
VAR_PRISONER_HI_TEMP2		= $4C
VAR_PRISONER_TICK		= $4D
VAR_RESTORE_FLAG		= $4E
VAR_TEMP_KEYS			= $4F
VAR_TEMP_AMMO			= $50
VAR_TEMP_HITS			= $51
VAR_TEMP_SCORE1			= $52
VAR_TEMP_SCORE2			= $53
VAR_TEMP_SCORE3			= $54
VAR_MISSION			= $55
VAR_PRISONER_FLOOR		= $56
VAR_PLAYER_INIT_FLAG		= $57
VAR_PRISONER_UPDATE		= $58
VAR_PRISONER_IS_CLOSE_FLAG	= $59
VAR_PRISONER_STEP		= $5A
LETTERS_SCROLL_X		= $5B
reveal_1			= $5C
reveal_2			= $5D
reveal_toggle			= $5E
reveal_state			= $5F
VAR_DETONATOR_FLAGS		= $60		; 4 bytes (one per floor)
VAR_RANDOM_INDEX		= $64		; not cleared between games

SAVED_6X6			= $66

; game variables
VAR_CURRENT_FLOOR		= $A7
VAR_MAP_Y_COUNTER		= $A8
SCROLL_X			= $A9
VAR_SCREEN_SCROLL		= $AA
VAR_KEYS			= $AC		; keys
VAR_REMOVE_DOOR_INDEX_1		= $AD
VAR_REMOVE_DOOR_INDEX_2		= $AE
VAR_BULLETS_FLAG		= $AF		; low nibble: bullet 1, high nibble: bullet 2
VAR_PLAYER_DIRECTION		= $B0		; 1=up, 2=down, 3=left, 4=right
VAR_BULLET_DIRECTION_1		= $B1
VAR_BULLET_MISC_1		= $B2		; misc. bullet related variable
VAR_BULLET_DIRECTION_2		= $B3		; bullet direction
VAR_BULLET_MISC_2		= $B4		; misc. bullet related variable
VAR_FIRE_FLAG			= $B5		; 0=no fire, 1=fire button held
VAR_PLAYER_SCREEN_TEMP_X	= $B6		; player screen x pos (temp)
VAR_PLAYER_SCREEN_TEMP_Y	= $B7		; player screen y pos (temp)
VAR_DOOR_MISC_1			= $B8		; holds $44 or $4A
VAR_DOOR_MISC_2			= $B9		; holds $45 or $47
VAR_PLAYER_CONTROL_DELAY	= $BA
;
VAR_AMMO			= $BD
VAR_LIFT_PASS_MESSAGE_FLAG	= $BE
VAR_PLAYER_INACTIVE_COUNTER	= $BF
VAR_PLAYER_INACTIVE_COUNTER_2	= $C0
VAR_TIMER			= $C1
VAR_SCORE_01			= $C2
VAR_SCORE_02			= $C3
VAR_SCORE_03			= $C4
VAR_SCORE_ADD			= $C5		; hundreds added to score
VAR_HITS			= $C6
VAR_TRANSITION_COUNT		= $C7		; 
VAR_LIFT_STATE			= $C8		; 0=no lift pass, 1=have lift pass, 2=entering lift
VAR_PAUSE			= $C9		; 0=normal mode, 1=paused
VAR_PLAYER_CALC_X		= $CA
VAR_PLAYER_CALC_Y		= $CB
VAR_ENEMY_DELAY			= $CC
VAR_RESURRECT_ENEMY_COUNT	= $CD
VAR_ENEMY_INDEX			= $CE
VAR_ENEMY_COUNT			= $CF
VAR_MAP_X			= $D0
VAR_MAP_Y			= $D1
VAR_BULLET_DIRECTION_TEMP	= $D2
VAR_EXPLOSION_FLASH_INDEX_1	= $D3
VAR_EXPLOSION_FLASH_INDEX_2	= $D4
VAR_STEP_SOUND_FLAG		= $D8
VAR_PLAYER_STEP_COUNTER		= $DA
VAR_BULLET_X_1			= $DB
VAR_BULLET_Y_1			= $DC
VAR_BULLET_X_2			= $DD
VAR_BULLET_Y_2			= $DE
VAR_PLAYER_SCREEN_X		= $DF		; player screen x pos
VAR_PLAYER_SCREEN_Y		= $E0		; player screen y pos
VAR_PLAYER_SHOOTING_FLAG	= $E3
VAR_PRISONER_CNT		= $E4
VAR_PRISONER_FLAG		= $E5
VAR_FLOOR_TEMP			= $E6
VAR_DOOR_TEMP			= $E7
VAR_TEMP_FLAG			= $E8
VAR_WIN_STATE			= $E9
VAR_DETONATOR_LO		= $EA
VAR_DETONATOR_HI		= $EB
VAR_PRISONER_LO			= $ED
VAR_PRISONER_HI			= $EE
;--

KEYBOARD			= $F0		; 
ROW_LOOP			= $F1		; temp. var for screen update
SCROLL_INDEX			= $F2
SCROLL_MAX			= $F3
SCROLL_2x_FLAG			= $F4		; 0=left side of 2x2 char, 1=right side of 2x2 char.

PRINT_CHAR			= $F5
PRINT_HPOS			= $F6
PRINT_VPOS			= $F7
PRINT_COLOR1			= $F8
PRINT_COLOR2			= $F9

ptr1				= $FA 		;16 bit - generic game usage
ptr2				= $FC 		;16 bit
ptr3				= $FE 		;16 bit

COLOR_MAP			= $0800
CHAR_MAP			= COLOR_MAP + $0400

ENEMY_X				= $0200
ENEMY_Y				= $0300
ENEMY_LO			= $0400
ENEMY_HI			= $0500
ENEMY_DIRECTION_AND_FLAGS	= $0600		; $40 = shot once $20 = killed $10 = step $03 = direction
DAT_SCROLLTEXT			= $0700

	sei
	sta $FF3F
	jmp start

	; aligned to $0800	
CHARSET_2X2
	include "charset_2x2.asm"
CHARSET_GAME
	include "charset_game.asm"

	; aligned to $1000
GAME_MAP
	ds $4000

	; aligned to $2000
	include "logo.asm"
	
	align 256
TILE_CHARS
	include "tiles.asm"

map_data
	include "map_data.asm"
	
CHAR_COLORS
	include "char_colors.asm"

DAT_RANDOM
	db $5E,$8C,$A6,$D2,$43,$42,$5F,$81,$29,$62,$A4,$91,$D4,$29,$FA,$C8
	db $8B,$F0,$E5,$EC,$D4,$D4,$72,$FA,$AF,$F9,$61,$FE,$5C,$C5,$7D,$BA
	db $AE,$7F,$75,$1F,$1E,$E1,$3F,$AB,$0F,$D7,$95,$CF,$4E,$5D,$E3,$03
	db $E7,$35,$E1,$1A,$F2,$2F,$19,$69,$9C,$A0,$5C,$F6,$1C,$C1,$83,$BE
	db $63,$CD,$13,$54,$2B,$B2,$AC,$83,$F0,$57,$CC,$99,$BB,$1B,$EA,$A5
	db $D5,$F0,$FE,$4A,$F9,$F8,$B3,$EC,$7D,$F4,$9A,$7E,$FE,$FF,$29,$DF
	db $36,$FD,$0F,$7E,$53,$54,$AF,$9F,$E5,$FB,$C6,$5F,$41,$14,$AF,$22
	db $75,$B3,$1C,$5B,$B8,$8A,$0B,$4E,$35,$84,$8E,$F6,$C3,$43,$71,$97
	db $21,$49,$F6,$90,$09,$77,$7E,$97,$08,$B2,$24,$CD,$55,$56,$52,$0A
	db $2F,$81,$21,$D6,$84,$84,$D9,$C2,$6A,$01,$A5,$B0,$7D,$D7,$FA,$10
	db $DB,$B8,$6D,$48,$34,$43,$16,$AC,$2F,$F2,$1B,$97,$8D,$8D,$E9,$C6
	db $C2,$55,$E7,$A8,$31,$E4,$F0,$1D,$B8,$7B,$CA,$5C,$FC,$09,$55,$8E
	db $AE,$45,$6E,$80,$F1,$97,$39,$EB,$8A,$D1,$51,$8A,$18,$0C,$22,$11
	db $9A,$4A,$A4,$ED,$01,$F4,$16,$A4,$8B,$42,$52,$F2,$81,$9D,$60,$E8
	db $82,$D4,$D4,$F1,$6C,$CA,$30,$F3,$AC,$7D,$D7,$50,$3A,$1E,$AD,$39
	db $95,$CF,$6A,$C7,$03,$F5,$0A,$A0,$E8,$13,$F2,$99,$ED,$D8,$E0,$0E
	
	include "sounds.asm"
	include "music.asm"
	
	;
	; --------
	;
SUB_UPDATE_SCREEN
	;dec $FF19
	;
	jsr SUB_CALC_SPRITE_SCREEN_POS		; calculate screen position first
	;
	lda sprite_chr_ptr			; add 4 lines to calculate when the sprite has "passed"
	clc
	adc #4 * 40
	sta sprite_passed
	lda sprite_chr_ptr+1
	adc #$00
	sta sprite_passed+1
	;
	lda #$00				; reset the flag each time
	sta sprite_converted_flag+1
	;
ntsc3	lda #$29				; start from 2nd line
	sta update_chr_target + 1		; preloading and adding #40 is faster than table lookup each row
	sta update_col_target + 1
	lda #CHAR_MAP >> 8
	sta update_chr_target + 2
	lda #COLOR_MAP >> 8
	sta update_col_target + 2
	;
	lda ptr1
	clc
ntsc2	adc #$00
	sta update_src + 1
	lda ptr1+1
	sta update_src + 2
	;
	lda #$14				; row count
	sta ROW_LOOP
	;
sus1	;
ntsc1	ldy #$1B				; column count/loop
update_src
	db $BF,0,0				; LAX
update_chr_target
	sta $FFFF,y				; write char
	lda CHAR_COLORS,x
update_col_target
	sta $FFFF,y				; write color
	dey
	bpl update_src
	;
	lda update_src + 1			; source pointer += $80
	eor #$80
	sta update_src + 1
	bmi *+5
	inc update_src + 2
	;
	lda update_chr_target + 1
	clc
	adc #$28
	sta update_chr_target + 1
	sta update_col_target + 1
	bcc *+8
	inc update_chr_target + 2
	inc update_col_target + 2
	;
sprite_converted_flag
	lda #$00
	bne skip_sprite_conversion
	;
	lda update_chr_target + 2		; check high byte first
	cmp sprite_passed+1
	bcc skip_sprite_conversion		; smaller, we can skip
	bne do_sprite_conversion		; non-equals means larger, so we're goood
	lda update_chr_target + 1		; otherwise high byte is equal, check low byte
	cmp sprite_passed
	bcc skip_sprite_conversion
	;
do_sprite_conversion
	jsr SUB_SPRITE_CONVERSION		; it's safe to draw the sprite now
	inc sprite_converted_flag+1		; set flag
	;
skip_sprite_conversion
	dec ROW_LOOP
	bpl sus1
	;
	;inc $FF19
	;
	rts

start	;sei
	;sta $FF3F
	;
	lda $FF00
	eor $FF02
	eor $FF1D
	sta VAR_RANDOM_INDEX
	;
	ldx #$FF				; bottom of screen sync value for PAL
	lda $FF07
	and #$40
	beq pal
	;
	ldy #$80+$40
	sty hires9+1
	;
	ldy #$98+$40
	sty multi2+1
	sty multi3+1
	sty multi4+1
	sty multi5+1
	sty multi6+1
	;
	ldy #$90+$40
	sty multi7+1
	;
	ldy #$1B-6				; decrease number of columns updated
	sty ntsc1+1
	ldy #$03				; add to the source address
	sty ntsc2+1
	ldy #$29+3				; and to the target address
	sty ntsc3+1
	;
	ldx #$F7				; bottom of screen sync value for NTSC
	stx ntsc4+1
	;
pal	cpx $FF1D
	bne *-3
	ldy #$00				; screen off
	sty $FF19				; initial black color
	sty $FF06
	sty $FF15
	;
	ora #$98
	sta $FF07
	;
	lda $FF12
	and #$03				; keep lowest 2 bits (sound)	
	ora #(logo) >> 10			; $08 = $2000, $10 = $4000, etc.
	sta $FF12
	;
	;lda #$F8
	;sta $FF12				; Sound1 Freq Hi / Gfx
	;
	lda #COLOR_MAP >> 8
	sta $FF14				; Video Matrix
	;
	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
	;lda #N_77E0 & 255			; $77E0 to $FFFE (16bit value)
	;sta $FFFE				; NMI Vector Lo Byte
	;lda #N_77E0 >> 8
	;sta $FFFF				; NMI Vector Hi Byte
	;lda #$A0				; $8AA0 to $FFFC (16bit value)
	;sta $FFFC				; Reset Vector Lo Byte
	;lda #$8A
	;sta $FFFD				; Reset Vector Hi Byte
	;
	lda #$FF				; hide cursor
	sta $FF0C
	sta $FF0D
	;
	lda #player_up_1 & 255 
	sta sprite_src_ptr
	lda #player_up_1 >> 8
	sta sprite_src_ptr+1
	;
	ldx #$0F
	lda #$00
il01	sta cheat_flags,x
	dex
	bpl il01
	;
	; ---
	;
;	ldy #$00
;	sty ptr1
;	lda #TILE_CHARS >> 8
;	sta ptr1+1
;	ldx #$07
;tc02	lda (ptr1),y
;	cmp #$02
;	bcc tc01
;	cmp #$08
;	bcs tc01
;	lda #$00
;	sta (ptr1),y
;	;
;tc01	iny
;	bne tc02
;	inc ptr1+1
;	dex
;	bpl tc02
	;
	;
	;!!!
	;jmp cheat_menu
	;jmp LAB_CHECK_HIGHSCORE
	;jmp LAB_ENTER_YOUR_NAME
	jmp INTRO_COLD_START
	;
	; --------
	;	
IRQ_ENTER_YOUR_NAME
	pha
	txa
	pha
	tya
	pha
	asl $FF09				; IRQ - Acknowledge IRQ
	;
	inc enter_your_name_tick
	;
	lda enter_your_name_tick
	cmp #$01
	beq B_0881
	;
	cmp #$02
	beq B_088C
	;
	cmp #$03
	beq B_0897
	;
	; ---
	; IRQ during "enter your name"
	;
	lda #$3B				; turn on graphics
	sta $FF06
	;
	lda #$71				; white for logo (during enter your name IRQ)
	sta $FF16				; Multi Color 1
	;
	jsr SUB_NAME_ENTRY
	;
	; ---
	;
	lda NAME_ENTRY_CURSOR_POS
	asl a
	adc #$0A
	sta PRINT_HPOS
	lda #$17
	sta PRINT_VPOS
	lda CHAR_MAP + $030B
	clc
	adc #$01
	lsr a					; div / 4
	lsr a
	sta PRINT_CHAR
	jsr SUB_PRINT_CHAR			; print the char
	;
	; ---
	;
	lda #$00
	sta enter_your_name_tick
	;
	lda #$3A
	bne irq_enter_your_name_end		; always jump
	;
B_0881	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen           
	;
	lda #$1B				; switch to text screen
	sta $FF06
	;
	lda #$96
	bne irq_enter_your_name_end		; always jump
	;
B_088C	lda LETTERS_SCROLL_X			;
multi7	ora #$90
	sta $FF07				; set horiz scroll for letter selection scroller
	;
	lda #$AA
	bne irq_enter_your_name_end		; always jump
	;
B_0897	;
multi3	lda #$98				; reset horiz scroll
	sta $FF07 
	;
	lda #$D0
irq_enter_your_name_end
	sta $FF0B				; Vert raster
	;
	pla
	tay
	pla
	tax
	pla
	rti
	;
	; --------
	;
SUB_WRITE_2X2_LETTERS
	lda VAR_LETTERS_POS
	lsr a
	sta VAR_LETTER_POS_X2
	bcs B_09E0
	ldx VAR_LETTER_POS_X2
	ldy #$00
B_09B9	lda TXT_LETTERS,x
	and #$3F
	asl a					; mul * 4
	asl a
	sta CHAR_MAP + $02F8,y
	adc #$01
	sta CHAR_MAP + $0320,y
	adc #$01
	sta CHAR_MAP + $02F9,y
	adc #$01
	sta CHAR_MAP + $0321,y
	inx
	cpx #$30
	bne *+4
	ldx #$00
	;
	iny
	iny
	cpy #$28
	bne B_09B9
	rts
	;
	; --------
	;
B_09E0	ldx VAR_LETTER_POS_X2
	ldy #$00
B_09E5	lda TXT_LETTERS,x
	and #$3F
	asl a					; mul * 4
	asl a
	adc #$02
	sta CHAR_MAP + $02F8,y
	adc #$01
	sta CHAR_MAP + $0320,y
	inx
	cpx #$30
	bne *+4
	ldx #$00
	;
	lda TXT_LETTERS,x
	and #$3F
	asl a					; mul * 4
	asl a
	sta CHAR_MAP + $02F9,y
	adc #$01
	sta CHAR_MAP + $0321,y
	iny
	iny
	cpy #$28
	bne B_09E5
	rts
	; --------

SUB_JOYSTICK_CHECK
	ldx #$00
	beq check_joystick
	;
	; otherwise X=$FF, which is the value that will be returned, each bit is for a specific direction
	;
	lda #$DF
	sta $FD30
	sta $FF08
	lda $FF08
	tay
	and #$08				; check cursor up
	bne kc1
	;
	txa
	and #$FE				; up = $01
	tax
	;
kc1	tya
	and #$01				; check cursor down
	bne kc2
	;
	txa
	and #$FD				; down = $02
	tax
	;
kc2	lda #$BF
	sta $FD30
	sta $FF08
	lda $FF08
	tay
	and #$01				; check cursor left
	bne kc3
	;
	txa
	and #$FB				; left = $04
	tax
	;
kc3	tya
	and #$08				; check cursor right
	bne kc4
	;
	txa
	and #$F7				; right = $08
	tax
	;
kc4	lda #$7F
	sta $FD30
	sta $FF08
	lda $FF08
	and #$10				; check space (fire)
	bne kc5
	;
	txa
	and #$EF				; fire = $10
	tax
	;
kc5	txa
	rts
	;
check_joystick
	lda #$FF				; no keyboard
	sta $FD30
joy_selector
	lda #$FD				; Query joystick
	sta $FF08
	lda $FF08
	eor #$FF
	pha
	and #$0F				; keep directions (lower 4 bits)
	sta j_temp + 1
	pla
joy_fire_bit
	and #$80
	beq j_temp				; no fire
	lda #$10				; handle fire separately (use $10 when fire is pressed)
j_temp	ora #$00
	eor #$FF
	rts

SUB_NAME_ENTRY
	jsr SUB_JOYSTICK_CHECK			; high score name entry
	tax					; save joy state
	and #$04
	beq ENTRY_CURSOR_LEFT
	;
	txa
	and #$08
	beq ENTRY_CURSOR_RIGHT
	;
	txa
	and #$10
	beq ENTRY_CURSOR_SELECT
	;
	lda #$00
	sta VAR_ENTRY_FLAG
sne_ret	rts
	; --------
ENTRY_CURSOR_LEFT
	lda LETTERS_SCROLL_X
	clc
	adc #$04
	sta LETTERS_SCROLL_X
	cmp #$08
	bne sne_ret
	lda #$00
	sta LETTERS_SCROLL_X
	ldx VAR_LETTERS_POS
	dex
	cpx #$FF
	bne *+4
	ldx #$5F
	stx VAR_LETTERS_POS
	jmp SUB_WRITE_2X2_LETTERS
	;
ENTRY_CURSOR_RIGHT
	lda LETTERS_SCROLL_X
	sec
	sbc #$04
	sta LETTERS_SCROLL_X
	bcs sne_ret
	lda #$04
	sta LETTERS_SCROLL_X
	ldx VAR_LETTERS_POS
	inx
	cpx #$60
	bne *+4
	ldx #$00
	stx VAR_LETTERS_POS
	jmp SUB_WRITE_2X2_LETTERS
	;
ENTRY_CURSOR_SELECT
	lda VAR_ENTRY_FLAG			;
	bne sne_ret
	lda CHAR_MAP + $030B
	clc
	adc #$01
	lsr a					; div / 4
	lsr a
	cmp #$1F
	beq entry_backspace
	;
	ldx NAME_ENTRY_CURSOR_POS		; at end?
	cpx #$0A
	beq ecs_ret
	inx
	stx NAME_ENTRY_CURSOR_POS
	lda #$01
	sta VAR_ENTRY_FLAG
ecs_ret	rts
	; --------
entry_backspace	
	lda NAME_ENTRY_CURSOR_POS
	asl a
	adc #$0A
	sta PRINT_HPOS
	lda #$17
	sta PRINT_VPOS
	lda #$2E
	sta PRINT_CHAR				;
	jsr SUB_PRINT_CHAR			;
	lda NAME_ENTRY_CURSOR_POS
	sec
	sbc #$01
	bcc eb1
	sta NAME_ENTRY_CURSOR_POS
eb1	lda #$01
	sta VAR_ENTRY_FLAG
	rts
	;
	;
	;
print	macro addr, xcoord, ycoord
	ldx #xcoord
	ldy #ycoord
	jsr SUB_SET_CURSOR_POS
	;
	ldx #addr & 255
	ldy #addr >> 8
	jsr SUB_PRINT
	endm

SUB_SET_CURSOR_POS				; x,y = pos
	txa
	clc
	adc screen_pos_lo,y
	sta ptr2
	sta ptr3
	lda screen_pos_hi,y
	adc #$00
	sta ptr2+1
	and #$FB
	sta ptr3+1
	rts
	;
SUB_PRINT
	stx print_src_ptr+1
	sty print_src_ptr+2
	;
	ldx #$00
print_loop
print_src_ptr
	lda $FFFF,x
	beq print_done
	jsr SUB_PRINT_CHR_CONT
	inx
	bne print_loop
print_done
	rts

SUB_PRINT_CHR_CONT
	;inc $FF19
	ldy #$00
	asl a
	asl a
	sta (ptr2),y
	iny
	ora #$02
	sta (ptr2),y
	ldy #$28
	eor #$03
	sta (ptr2),y
	iny
	ora #$02
	sta (ptr2),y
	;
	lda PRINT_COLOR2
	sta (ptr3),y				; lower two colors
	dey
	sta (ptr3),y
	;
	ldy #$00
	lda PRINT_COLOR1
	sta (ptr3),y				; upper two colors
	iny
	sta (ptr3),y
	;
	lda ptr2				; move destination pointers right (next char)
	clc
	adc #$02
	sta ptr2
	sta ptr3
	bcc *+6
	inc ptr2+1
	inc ptr3+1
	;
	;dec $FF19
	rts
	;
SUB_PRINT_CHAR
	sty sav_y+1
	;
	ldy PRINT_VPOS
	lda screen_pos_lo,y
	clc
	adc PRINT_HPOS
	sta ptr2
	sta ptr3
	lda screen_pos_hi,y
	adc #$00
	sta ptr2+1
	and #$FB
	sta ptr3+1
	;
	; ---
	;
	ldy #$00
ppc1	lda PRINT_CHAR
	asl a
	asl a
	sta (ptr2),y
	iny
	;
	ora #$02
	sta (ptr2),y
	ldy #$28
	;
	eor #$03
	sta (ptr2),y
	iny
	;
	ora #$02
	sta (ptr2),y
	;
	lda PRINT_COLOR2			; lower two colors
	sta (ptr3),y
	dey
	sta (ptr3),y
	;
	ldy #$00
	lda PRINT_COLOR1
	sta (ptr3),y				; upper two colors
	iny
	sta (ptr3),y
	;
	lda PRINT_HPOS				; x+2
	clc
	adc #$02
	cmp #$28				; =40?
	bcc ppc2
	inc PRINT_VPOS				; y+2
	inc PRINT_VPOS 
	lda #$00				; x=0
	;
ppc2	sta PRINT_HPOS
	;
sav_y	ldy #$00
	;
	rts
	;
	; ---
	;
LAB_CHECK_HIGHSCORE
	sei
	;
	lda #$0B				; screen off
	sta $FF06
	;
	ldx #11
cchk1	lda cheat_flags,x
	bne skip_checking_highscore
	dex
	bpl cchk1
	;
	lda #$0D
	sta HS_POS
	;
cchk2	ldy #$00
cchk3	tya
	clc
	adc HS_POS
	tax
	lda TXT_HIGHSCORES,x
	cmp HS_SCORE,y
	beq cchk4
	bpl cchk5
	bmi LAB_ENTER_YOUR_NAME
cchk4	iny
	cpy #$06				; length of score (6 characters)
	bne cchk3
cchk5	lda HS_POS
	clc
	adc #20					; length of one line
	sta HS_POS
	cmp #20*10				; 10 lines
	bcc cchk2
	;
skip_checking_highscore
	jmp INTRO_COLD_START
	;
	; --- made the high score	
	;
LAB_ENTER_YOUR_NAME
	lda HS_POS
	sec
	sbc #$0D
	sta HS_POS
	;
	jsr prepare_logo
	;
	lda #$00
	sta LETTERS_SCROLL_X
	;
	lda #$4F
	sta VAR_LETTERS_POS
	;
	ldx #$00
B_0BFC	lda #$80
	sta CHAR_MAP + $0168,x
	sta CHAR_MAP + $0268,x
	dex
	bne B_0BFC
	ldx #$68
B_0C09	lda #$80
	sta CHAR_MAP + $0300,x
	inx
	cpx #$E8
	bne B_0C09				; Loop until X is zero
	;
	ldx #$27
B_0C15	lda #$00
	sta COLOR_MAP + $0140,x
	lda #$32				; red colors for scrolling letters
	sta COLOR_MAP + $02F8,x
	lda #$42
	sta COLOR_MAP + $0320,x
	dex
	bpl B_0C15
	;
	jsr SUB_WRITE_2X2_LETTERS		;
	;
	lda #$36				; blues
	sta PRINT_COLOR1
	lda #$56
	sta PRINT_COLOR2
	;
	print TXT_WELL_DONE, 11, 9
	;
	lda #$33				; cyan
	sta PRINT_COLOR1
	lda #$53
	sta PRINT_COLOR2
	;
	print TXT_REAL_HERO, 1, 12
	;
	lda #$45				; greens
	sta PRINT_COLOR1
	lda #$65
	sta PRINT_COLOR2
	;
	print TXT_ENTER_YOUR_NAME, 5, 16
	;
	lda #$13				; $1513 to $0A80 (16bit value)
	sta PRINT_HPOS
	lda #$15				; $0F15 to $0A81 (16bit value)
	sta PRINT_VPOS
	;
	lda #$51;0F				; grays
	sta PRINT_COLOR1
	lda #$71;01
	sta PRINT_COLOR2
	;
	lda #$1E
	sta PRINT_CHAR				;
	jsr SUB_PRINT_CHAR			;
	;
	lda #$80
	sta CHAR_MAP + $0384
	sta CHAR_MAP + $0383
	;
	lda #$34				; purples
	sta PRINT_COLOR1
	lda #$44
	sta PRINT_COLOR2
	;
	print TXT_DOTS, 0, $17
	;
	lda #$00
	sta NAME_ENTRY_CURSOR_POS
	;
	sei
	;
	lda #IRQ_ENTER_YOUR_NAME & 255
	sta $FFFE				; IRQ Vector Lo Byte
	lda #IRQ_ENTER_YOUR_NAME >> 8
	sta $FFFF				; IRQ Vector Hi Byte
	;
	lda #$D0
	sta $FF0B				; Vert raster
	;
	lda #$00				; internal raster counter
	sta enter_your_name_tick
	cli
	;
B_0CDC	lda NAME_ENTRY_CURSOR_POS
	cmp #$0A
	bne B_0CDC
	;
	ldx #$00
	ldy #$00
B_0CE7	lda CHAR_MAP + $03A3,x
	lsr a					; div / 4
	lsr a
	sta HS_NAME,y				;
	inx
	inx
	iny
	cpy #$0A
	bne B_0CE7
	;
	ldx #(20*10)-1
B_0CF8	lda TXT_HIGHSCORES-20,x			; move scores down
	sta TXT_HIGHSCORES,x
	dex
	cpx HS_POS
	bne B_0CF8
	;
V_0D04	ldx HS_POS
	ldy #$00
B_0D09	lda #$20
	sta TXT_HIGHSCORES,x
	inx
	iny
	cpy #20
	bne B_0D09
	;
	ldx HS_POS
	ldy #$00
B_0D19	lda HS_NAME,y
	sta TXT_HIGHSCORES+1,x			; store name in highscores
	inx
	iny
	cpy #10
	bne B_0D19
	ldx HS_POS
	ldy #$00
B_0D2A	lda HS_SCORE,y
	sta TXT_HIGHSCORES+$D,x			; store score in highscores
	inx
	iny
	cpy #$06
	bne B_0D2A
	;
	; ---
	;
	sei
	lda #$0B
	sta $FF06				; screen off
	;
	ldx #$FF
dly21	ldy #$FF
dly22	lda #$01				; delay
dly23	sec
	sbc #$01
	bne dly23
	dey
	bne dly22
	dex
	bne dly21
	;
	jmp INTRO_COLD_START
	;
	; --------
	;
IRQ_HIGHSCORES
	pha
	txa
	pha
	tya
	pha
	asl $FF09				; IRQ - Acknowledge IRQ
	;
	lda HS_FADE_STATE
	cmp #$01
	beq fade_state_1
	cmp #$02
	beq fade_state_2
	cmp #$03
	beq fade_state_3
	cmp #$04
	beq fade_state_1
	;
	jsr SUB_FADE_HS_SCREEN
	;
	ldx HS_FADE_INDEX			; fade IN
	inx
	cpx #$0A
	bne fs1
	lda #$03
	sta HS_FADE_STATE
	ldx #$09
fs1	stx HS_FADE_INDEX
fade_state_1
	jmp fade_state_3
	;
fade_state_2	
	jsr SUB_FADE_HS_SCREEN
	;
	ldx HS_FADE_INDEX			; fade OUT
	dex
	bpl fs2
	ldx #$00
	lda #$04
	sta HS_FADE_STATE
fs2	stx HS_FADE_INDEX
	;
fade_state_3
	jsr MUSIC_PLAYER
	jmp IRQ_END
	;
SUB_FADE_HS_SCREEN
	;dec $FF19
	ldy HS_FADE_INDEX
	;
	lda HS_FADE_COLORS_1,y			; preload 4 fade values
	sta fade_color_1+1			; changed to self-modifying code (shorter, faster)
	;
	lda HS_FADE_COLORS_2,y
	sta fade_color_2+1
	;
	lda HS_FADE_COLORS_3,y
	sta fade_color_3+1
	;
	lda HS_FADE_COLORS_4,y
	sta fade_color_4+1
	;
	ldx #$23				; column loop (shortened by 4 columns)
	;
fade_loop	
fade_color_1
	lda #$00
	sta COLOR_MAP + 1*40 + 2,x
	;
fade_color_2
	lda #$00
	sta COLOR_MAP + 2*40 + 2,x
	;
fade_color_3
	lda #$00
	sta COLOR_MAP +  4*40 + 2,x
	sta COLOR_MAP +  6*40 + 2,x
	sta COLOR_MAP +  8*40 + 2,x
	sta COLOR_MAP + 10*40 + 2,x
	sta COLOR_MAP + 12*40 + 2,x
	sta COLOR_MAP + 14*40 + 2,x
	sta COLOR_MAP + 16*40 + 2,x
	sta COLOR_MAP + 18*40 + 2,x
	sta COLOR_MAP + 20*40 + 2,x
	sta COLOR_MAP + 22*40 + 2,x
	;
fade_color_4
	lda #$00
	sta COLOR_MAP +  5*40 + 2,x
	sta COLOR_MAP +  7*40 + 2,x
	sta COLOR_MAP +  9*40 + 2,x
	sta COLOR_MAP + 11*40 + 2,x
	sta COLOR_MAP + 13*40 + 2,x
	sta COLOR_MAP + 15*40 + 2,x
	sta COLOR_MAP + 17*40 + 2,x
	sta COLOR_MAP + 19*40 + 2,x
	sta COLOR_MAP + 21*40 + 2,x
	sta COLOR_MAP + 23*40 + 2,x
	dex
	bpl fade_loop
	;inc $FF19
	rts
	;
	; --------
	;
prepare_logo
	ldx #$00
pl1	lda #$11				; both grays
	sta CHAR_MAP + $0000,x
	lda #$42				; shades of gray
	sta COLOR_MAP + $0000,x
	inx
	bne pl1
	;
	ldx #$17
pl2	lda #$11
	sta CHAR_MAP + $0100,x
	lda #$42
	sta COLOR_MAP + $0100,x
	dex
	bpl pl2
	rts
	;
	; --------
	;
IRQ_INTRO
	pha
	txa
	pha
	tya
	pha
	;
	asl $FF09				; IRQ - Acknowledge IRQ
	;
	lda $FF0B
	cmp #$CC
	bne irq_intro_upper
	;
	lda #$3B				; turn on graphics mode for the logo
	sta $FF06
	lda #$71				; white for the logo (during intro IRQ)
	sta $FF16				; Multi Color 1
	;
	jsr MUSIC_PLAYER			; music
	;
	lda #$3A				; 
	bne irq_intro_end			; always jump

irq_intro_upper
	lda #$31				; darker grays for the game area
	sta $FF16
	lda #$41
	sta $FF17
	;
	lda #$1B				; switch back to
	sta $FF06				; character mode
	;
	lda #$CC
irq_intro_end
	sta $FF0B				; Vert raster
	;
	pla
	tay
	pla
	tax
	pla
	rti
	;
	;
	;
SUB_DRAW_INTRO_FLOOR_TILES
	lda #$E7
	sta ptr1
	sta ptr2
	lda #(CHAR_MAP + $0100) >> 8
	sta ptr1+1
	lda #(COLOR_MAP + $0100) >> 8
	sta ptr2+1
	;
	ldx #$00
sdift1	ldy #$03
sdift2	lda #$E8				; floor char
	sta (ptr1),y
	lda CHAR_COLORS				; tile color for intro screen
	sta (ptr2),y
	dey
	bpl sdift2
	;
	lda ptr1
	clc
	adc #$28
	sta ptr1
	sta ptr2
	bcc *+6
	inc ptr1+1
	inc ptr2+1
	;
	inx
	cpx #$0D
	bne sdift1
	;
	rts
	;
	;
	;
INTRO_COLD_START
	jsr MUSIC_PLAYER_INIT
	;jmp INTRO_SCREEN_2			; !!!
	;jmp LAB_ENTER_YOUR_NAME
	;
	;
	;
INTRO_SCREEN_1
	ldx #$01				; intro screen 1
	stx HS_FADE_STATE			; set up fade variables
	dex
	stx HS_FADE_INDEX
	;
	lda #$0B
	sta $FF06				; screen off
	;
	sei
	;
	lda #IRQ_HIGHSCORES & 255
	sta $FFFE				; IRQ Vector Lo Byte
	lda #IRQ_HIGHSCORES >> 8
	sta $FFFF				; IRQ Vector Hi Byte
	lda #$D0
	sta $FF0B				; Vert raster
	;
	cli
	;
	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
	ldx #$00
	lda #$80
cls00	sta CHAR_MAP + $0000,x			; clear screen
	sta CHAR_MAP + $0100,x
	sta CHAR_MAP + $0200,x
	sta CHAR_MAP + $0300,x
	dex
	bne cls00
	;
	ldx #$01
	stx PRINT_VPOS
	dex
	stx PRINT_HPOS
	stx PRINT_COLOR1
	stx PRINT_COLOR2
	;
is10	lda TXT_TODAYS_REAL_HEROES,x		; print top line
	beq is13
	sta PRINT_CHAR				;
	jsr SUB_PRINT_CHAR			;
	inx
	bne is10
	;
is13	ldx #$1F
is11	lda y_apostrophe,x			; copy custom "A'Y" characters into charset
	sta CHARSET_2X2 + $0740,x
	dex
	bpl is11
	;
	ldx #$17
	lda #$00
is12	sta CHARSET_2X2 + $75 * 8,x		; clear two 3-character areas in the 2x2 font which are used by the cheat menu
	sta CHARSET_2X2 + $81 * 8,x
	dex
	bpl is12
	;
	ldx #$E8				; put chars on screen (overwrite part of "A" and "Y")
	stx CHAR_MAP + 49
	inx
	stx CHAR_MAP + 50
	inx
	stx CHAR_MAP + 51
	inx
	stx CHAR_MAP + 90
	lda #$80
	sta CHAR_MAP + 91 
	;
	ldx #$00				; $0300 to $0A80 (16bit value)
	stx PRINT_HPOS
	lda #$04				; $0003 to $0A81 (16bit value)
	sta PRINT_VPOS
	;
is14	lda TXT_HIGHSCORES,x
	sta PRINT_CHAR				; display high scores
	jsr SUB_PRINT_CHAR			;
	inx
	cpx #20*10				; 10 lines of 20 characters
	bne is14
	;
	lda #$1B				; screen on
	sta $FF06
	;
	lda #$00
	sta HS_FADE_STATE
	;   
	jsr INTRO_PAUSE				; new code: use the same pause as all other screens
	;
	lda #$02				; start fade out
	sta HS_FADE_STATE
	;
	lda #$04				; wait for fade out to complete
fsw	cmp HS_FADE_STATE
	bne fsw 
	;
	; --------
	;
INTRO_SCREEN_2					; intro screen 2
	sei
	;
	lda #$0B				; screen off
	sta $FF06
	;
	ldx #$00
	lda #$80
cls08	sta CHAR_MAP + $0000,x			; clear screen
	sta CHAR_MAP + $0100,x
	sta CHAR_MAP + $0200,x
	sta CHAR_MAP + $0300,x
	dex
	bne cls08
	;
	ldx #$C0
cm0A	lda charset_1x1 + $30 - 1,x		; copy the 1x1 set to the end of the 2x2 charset
	sta CHARSET_2X2 + $740 - 1,x
	dex
	bne cm0A
	;
	lda #$31				; *** fixed
	sta $FF16				; Multi Color 1
	;
	lda #$41				; *** fixed
	sta $FF17				; Multi Color 2
	;
	jsr prepare_logo
	;
multi2	lda #$98				; turn on multicolor
	sta $FF07
	;
	lda #IRQ_INTRO & 255			; $0F95 to $0314 (16bit value)
	sta $FFFE				; IRQ Vector Lo Byte
	lda #IRQ_INTRO >> 8
	sta $FFFF				; IRQ Vector Hi Byte
	;
	lda #$CC
	sta $FF0B				; Vert raster
	;
	cli
	;
	; ---
	;
	lda #$36				; upper color
	sta PRINT_COLOR1
	lda #$56				; lower color
	sta PRINT_COLOR2
	;
	print TXT_PROGRAMMING, 6, 9
	print TXT_ANDY, 6, 11
	;
	lda #$45				; upper color
	sta PRINT_COLOR1
	lda #$65				; lower color
	sta PRINT_COLOR2
	;
	print TXT_GRAPHICS, 9, 14
	print TXT_ROB, 7, 16
	;
	ldx #$1B
crc1	lda TXT_CREDITS_1,x
	sta CHAR_MAP + 40 * 20 + 6,x
	lda #$67
	sta COLOR_MAP + 40 * 20 + 6,x 
	dex
	bpl crc1
	;
	ldx #$1D
crc2	lda TXT_CREDITS_2,x
	sta CHAR_MAP + 40 * 22 + 5,x
	lda #$52
	sta COLOR_MAP + 40 * 22 + 5,x 
	dex
	bpl crc2
	;
	ldx #$17
crc3	lda TXT_CREDITS_3,x
	sta CHAR_MAP + 40 * 24 + 8,x
	lda #$31
	sta COLOR_MAP + 40 * 24 + 8,x 
	dex
	bpl crc3
	;
	jsr INTRO_PAUSE
	;
	; --- INTRO SCREEN 3
	;
	ldx #$3F
cp04	lda CHARSET_GAME,x
	sta CHARSET_2X2 + $740,x
	dex
	bpl cp04
	;
	ldx #$1F
cp05	lda CHARSET_GAME + $00A0,x
	sta CHARSET_2X2 + $780,x
	lda CHARSET_GAME + $00C0,x
	sta CHARSET_2X2 + $7A0,x
	lda CHARSET_GAME + $0140,x
	sta CHARSET_2X2 + $7C0,x
	lda CHARSET_GAME + $0330,x
	sta CHARSET_2X2 + $7E0,x
	dex
	bpl cp05
	;
	ldx #$00
	lda #$80
cls09	sta CHAR_MAP + $0168,x			; clear screen (below logo)
	sta CHAR_MAP + $0200,x
	sta CHAR_MAP + $0300,x
	dex
	bne cls09
	;
	lda $FF1D				; sync to bottom of frame
	cmp #$CC
	bcc *-5	
	;
	jsr SUB_DRAW_INTRO_FLOOR_TILES
	;
	ldx #$F0
	stx CHAR_MAP + $0210
	inx
	stx CHAR_MAP + $0211
	inx
	stx CHAR_MAP + $0238
	inx
	stx CHAR_MAP + $0239
	;
	lda CHAR_COLORS + $14			; item color
	sta COLOR_MAP + $0210
	sta COLOR_MAP + $0211
	sta COLOR_MAP + $0238
	sta COLOR_MAP + $0239
	;
	ldx #$F4
	stx CHAR_MAP + $0288
	inx
	stx CHAR_MAP + $02B0
	inx
	stx CHAR_MAP + $02B1
	inx
	stx CHAR_MAP + $0289
	;
	lda CHAR_COLORS + $18
	sta COLOR_MAP + $0288
	lda CHAR_COLORS + $1B
	sta COLOR_MAP + $0289
	lda CHAR_COLORS + $19
	sta COLOR_MAP + $02B0
	lda CHAR_COLORS + $1A
	sta COLOR_MAP + $02B1
	;
	ldx #$F8
	stx CHAR_MAP + $0300
	inx
	stx CHAR_MAP + $0301
	inx
	stx CHAR_MAP + $0328
	inx
	stx CHAR_MAP + $0329
	;
	lda CHAR_COLORS + $28
	sta COLOR_MAP + $0300
	sta COLOR_MAP + $0301
	sta COLOR_MAP + $0328
	sta COLOR_MAP + $0329
	;
	ldx #$FC
	stx CHAR_MAP + $03A0
	inx
	stx CHAR_MAP + $03A1
	inx
	stx CHAR_MAP + $0378
	inx
	stx CHAR_MAP + $0379
	;
	lda CHAR_COLORS + $66
	sta COLOR_MAP + $0378
	sta COLOR_MAP + $0379
	sta COLOR_MAP + $03A0
	sta COLOR_MAP + $03A1
	;
	lda #$34				; purple
	sta PRINT_COLOR1
	lda #$54
	sta PRINT_COLOR2
	;
	print TXT_VALUABLES, $0B, $09 
	;
	lda #$33				; cyan
	sta PRINT_COLOR1
	lda #$53
	sta PRINT_COLOR2
	;
	print TXT_PAINTING, $0E, $0D
	print TXT_VASE, $0E, $10
	print TXT_PENDANT, $0E, $13
	print TXT_JEWELS, $0E, $16
	;
	jsr INTRO_PAUSE
	;
	; --- INTRO SCREEN 4
	;
	ldx #$1F
cp01	lda CHARSET_GAME + $00E0,x
	sta CHARSET_2X2 + $780,x
	lda CHARSET_GAME + $0100,x
	sta CHARSET_2X2 + $7E0,x
	dex
	bpl cp01
	;
	ldx #$0F
cp02	lda CHARSET_GAME + $0080,x
	sta CHARSET_2X2 + $7A0,x
	lda CHARSET_GAME + $0090,x
	sta CHARSET_2X2 + $7C0,x
	dex
	bpl cp02
	;
	ldx #$07
cp03	lda CHARSET_GAME,x
	sta CHARSET_2X2 + $7B0,x
	sta CHARSET_2X2 + $7B8,x
	sta CHARSET_2X2 + $7D0,x
	sta CHARSET_2X2 + $7D8,x
	dex
	bpl cp03
	;
	jsr SUB_DRAW_INTRO_FLOOR_TILES
	;
	; ---
	;
	ldx #$F0				; draw cold food
	stx CHAR_MAP + $0210
	inx
	stx CHAR_MAP + $0211
	inx
	stx CHAR_MAP + $0238
	inx
	stx CHAR_MAP + $0239
	;
	lda CHAR_COLORS + $1C
	sta COLOR_MAP + $0210
	sta COLOR_MAP + $0211
	lda CHAR_COLORS + $1E			; fixed colors
	sta COLOR_MAP + $0238
	lda CHAR_COLORS + $1F
	sta COLOR_MAP + $0239
	;
	; ---
	;
	ldx #$F4				; draw key
	stx CHAR_MAP + $0288
	inx
	stx CHAR_MAP + $0289
	;
	lda CHAR_COLORS + $10
	sta COLOR_MAP + $0288
	sta COLOR_MAP + $0289
	;
	; ---
	;
	ldx #$F8				; draw ammo
	stx CHAR_MAP + $0300
	inx
	stx CHAR_MAP + $0301
	;
	lda CHAR_COLORS + $12
	sta COLOR_MAP + $0300
	sta COLOR_MAP + $0301
	;
	; ---
	;
	ldx #$FC				; draw first aid
	stx CHAR_MAP + $0378
	inx
	stx CHAR_MAP + $0379
	inx
	stx CHAR_MAP + $03A1
	inx
	stx CHAR_MAP + $03A0
	;
	lda CHAR_COLORS + $20
	sta COLOR_MAP + $0378
	sta COLOR_MAP + $0379
	sta COLOR_MAP + $03A0
	sta COLOR_MAP + $03A1
	;
	; ---
	;
	ldx #$EA				; draw shadow for first aid
	stx CHAR_MAP + $03C9
	inx
	stx CHAR_MAP + $0377
	inx
	stx CHAR_MAP + $039F
	inx
	stx CHAR_MAP + $03C8
	ldx #$EF
	stx CHAR_MAP + $03C7
	;
	; ---
	;
	lda #$34				; purple
	sta PRINT_COLOR1
	lda #$54
	sta PRINT_COLOR2
	;
	print TXT_SUPPLIES, $0A, $09  
	;
	lda #$33				; cyan
	sta PRINT_COLOR1
	lda #$53
	sta PRINT_COLOR2
	;
	print TXT_COLD_FOOD, $0E, $0D
	print TXT_DOOR_KEY, $0E, $10
	print TXT_AMMUNITION, $0E, $13
	print TXT_FIRST_AID, $0E, $16
	;
	jsr INTRO_PAUSE
	;
	jmp INTRO_SCREEN_1
	;
use_joy_1
	ldx #$00
	stx SUB_JOYSTICK_CHECK+1
	ldx #$FB
	stx joy_selector+1
	ldx #$40
	stx joy_fire_bit+1
	rts
use_joy_2
	ldx #$00
	stx SUB_JOYSTICK_CHECK+1
	ldx #$FD
	stx joy_selector+1
	ldx #$80
	stx joy_fire_bit+1
	rts
	;
	; --- pause between screens
	;     check joy 1, joy 2 and keyboard, and set controls
	;     exit on fire
	;     end pause on right
	;     go to cheats menu on left
	;
INTRO_PAUSE
	ldx #$F0
ipau1	ldy #$FF
ipau2	lda #$03
ipau3	sec					; pause
	sbc #$01
	bne ipau3
	;
	lda #$FF				; no keyboard
	sta $FD30
	lda #$FB				; check joy 1
	sta $FF08
	lda $FF08
	and #$4C
	cmp #$4C
	beq check_joy2
	;
	jsr use_joy_1				; --- use joy 1
	jmp handle_joy	
	;
check_joy2
	lda #$FF				; no keyboard
	sta $FD30
	lda #$FD				; check joy 2
	sta $FF08
	lda $FF08
	and #$8C
	cmp #$8C
	beq check_keys_1
	;
	jsr use_joy_2				; --- use joy 2
	jmp handle_joy
	;
check_keys_1
	lda #$7F
	sta $FD30
	sta $FF08
	lda $FF08
	and #$10
	bne check_keys_2
	;
	ldx #$FF
	stx SUB_JOYSTICK_CHECK+1		; use keyboard
	bne LAB_INTRO_OVER			; always jump
	;
check_keys_2
	lda #$BF
	sta $FD30
	sta $FF08
	lda $FF08
	and #$09
	cmp #$09
	beq nothing_pressed
	;
	ldx #$FF
	stx SUB_JOYSTICK_CHECK+1		; use keyboard
	;
	and #$01
	beq go_to_cheat_menu			; left: cheat menu
	bne ipaux				; right: exit from pause
	;
nothing_pressed
	;	
	dey
	bne ipau2
	dex
	bne ipau1
	rts
	;
ipaux	jsr SUB_JOYSTICK_CHECK			; wait for release
	and #$08
	beq ipaux
	rts
	;
go_to_cheat_menu
	jmp cheat_menu 
	;
handle_joy
	tax
	and #$04
	beq go_to_cheat_menu			; left -> cheats
	txa
	and #$08
	beq ipaux				; right -> exit from pause
 	; otherwise fire is pressed, exit from intro
	;
	; --- intro over
	;
LAB_INTRO_OVER
	sei
	;
	lda #$00
	sta $FF11				; volume off
	;
	jsr CLEAR_SCREEN
	;
	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
	lda #IRQ_NULL & 255			; $77E8 to $0314 (16bit value)
	sta $FFFE				; IRQ Vector Lo Byte
	lda #IRQ_NULL >> 8
	sta $FFFF				; IRQ Vector Hi Byte
	;
	lda #$1B
	sta $FF06				; screen on
	;
wait04	jsr SUB_JOYSTICK_CHECK			; wait for fire
	and #$10
	beq wait04
	;
	cli
	;
	jmp LAB_GAME_START
	;
	; --- Screen off and clear screen
	;
CLEAR_SCREEN
	lda #$0B
	sta $FF06				; screen off
	;
	lda #$00
	tax
cls03	sta COLOR_MAP + $0000,x			; clear screen
	sta COLOR_MAP + $0100,x
	sta COLOR_MAP + $0200,x
	sta COLOR_MAP + $0300,x
	;
	sta CHAR_MAP + $0000,x
	sta CHAR_MAP + $0100,x
	sta CHAR_MAP + $0200,x
	sta CHAR_MAP + $0300,x
	dex
	bne cls03
	;
	rts
	;
	; --------
	;
LAB_ELEVATOR_SCREEN
	jsr CLEAR_SCREEN
	;
	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
multi4	lda #$98
	sta $FF07				; Horiz Scroll
	;
	lda #$2E				; blue
	sta $FF16				; Multi Color 1
	lda #$46				; blue
	sta $FF17				; Multi Color 2
	;
	lda #$23				; cyans
	sta PRINT_COLOR1
	lda #$53
	sta PRINT_COLOR2
	;
	print TXT_SELECT_FLOOR, 8, 4
	;
	lda #$67				; yellows
	sta PRINT_COLOR1
	lda #$77
	sta PRINT_COLOR2
	;
	print TXT_FL0, 6, 8
	print TXT_FL1, 6, 10
	print TXT_FL2, 6, 12
	print TXT_FL3, 6, 14
	;
	lda #$71+8				; color for selected button
	sta PRINT_COLOR1
	sta PRINT_COLOR2
	;
	ldx #$03				; loop 4 times
	;
print_buttons
	lda #$20
	sta PRINT_HPOS
	txa
	asl a
	ora #$08
	sta PRINT_VPOS
	;
	jsr SUB_PRINT_EMPTY_RECT
	;
	dex
	bpl print_buttons
	;
	; ---
	;
	ldx #$1C
les01	lda #$AC				; draw upper and lower borders
	sta CHAR_MAP + $011E,x
	lda #$AE
	sta CHAR_MAP + $0286,x
	lda #$16
	sta COLOR_MAP + $011E,x
	sta COLOR_MAP + $0286,x
	dex
	bpl les01
	;
	lda #$45				; init screen pointers
	sta ptr2
	sta ptr3
	lda #(CHAR_MAP + $100) >> 8
	sta ptr2+1
	lda #(COLOR_MAP + $100) >> 8
	sta ptr3+1
	;
	ldx #$00
les02	ldy #$00
	lda #$AF				; left border side character
	sta (ptr2),y
	ldy #$1D
	lda #$AD				; right border side character
	sta (ptr2),y
	lda #$16
	sta (ptr3),y
	ldy #$00
	sta (ptr3),y
	lda ptr2
	clc
	adc #$28
	sta ptr2
	sta ptr3
	bcc les03
	inc ptr2+1
	inc ptr3+1
les03	inx
	cpx #$08
	bne les02
	;
	ldx #$A8				; corners
	stx CHAR_MAP + $011D
	inx
	stx CHAR_MAP + $0285
	inx
	stx CHAR_MAP + $013A
	inx
	stx CHAR_MAP + $02A2
	lda #$16
	sta COLOR_MAP + $011D
	sta COLOR_MAP + $0285
	sta COLOR_MAP + $013A
	sta COLOR_MAP + $02A2
	;
	lda #$1B
	sta $FF06				; screen on
	;
	lda VAR_CURRENT_FLOOR
	sta VAR_ELEV_CURR_FLOOR
	asl a
	clc
	adc #$08
	sta PRINT_VPOS
	lda #$20				; position
	sta PRINT_HPOS
	;
	jsr SUB_PRINT_SELECTED_RECT
	;
elevator_loop
	jsr SUB_JOYSTICK_CHECK
	tax					; save joy state
	and #$10				; fire pressed?
	bne elev_no_fire
	lda VAR_ELEV_CURR_FLOOR			; done, return with the selection
	rts
	; --------
elev_no_fire
	txa
	and #$01				; up?
	beq elev_up
	;
	txa
	and #$02				; down?
	beq elev_dn
	jmp elevator_loop
	;
elevator_cont
	ldx #$FF
dly31	ldy #$80
dly32	dey					; Delay
	bne dly32
	dex
	bne dly31
	;
	lda #$C0
	cmp $FF1D				; sync to bottom of frame
	bne *-3
	;
	jmp elevator_loop
	;
elev_up	lda VAR_ELEV_CURR_FLOOR
	asl a
	clc
	adc #$08
	sta PRINT_VPOS
	lda #$20				; print position
	sta PRINT_HPOS
	;
	jsr SUB_PRINT_EMPTY_RECT
	;
	dec VAR_ELEV_CURR_FLOOR
	lda VAR_ELEV_CURR_FLOOR
	bpl *+4
	lda #$03
	sta VAR_ELEV_CURR_FLOOR
	asl a
	clc
	adc #$08
	sta PRINT_VPOS
	lda #$20				; $201C to $0A7F (16bit value)
	sta PRINT_HPOS
	;
	jsr SUB_PRINT_SELECTED_RECT
	;
	jmp elevator_cont
	;
elev_dn	lda VAR_ELEV_CURR_FLOOR
	asl a
	clc
	adc #$08
	sta PRINT_VPOS
	lda #$20				; $201B to $0A7F (16bit value)
	sta PRINT_HPOS
	;
	jsr SUB_PRINT_EMPTY_RECT
	;
	inc VAR_ELEV_CURR_FLOOR
	lda VAR_ELEV_CURR_FLOOR
	cmp #$04
	bne *+4
	lda #$00
	sta VAR_ELEV_CURR_FLOOR
	asl a
	clc
	adc #$08
	sta PRINT_VPOS
	lda #$20				; $201C to $0A7F (16bit value)
	sta PRINT_HPOS
	;
	jsr SUB_PRINT_SELECTED_RECT
	;
	jmp elevator_cont
	;
SUB_PRINT_EMPTY_RECT
	lda #$1B				; "NOT SELECTED RECTANGLE" index
	db $2C
SUB_PRINT_SELECTED_RECT
	lda #$1C				; "SELECTED RECTANGLE" index
	sta PRINT_CHAR
	jmp SUB_PRINT_CHAR
	;
	;
	;
SUB_SELECT_MISSION
	lda #$01
	sta VAR_MISSION_TEMP
	;
	jsr CLEAR_SCREEN
	;
	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
	lda #$41;0C				; grays
	sta PRINT_COLOR1
	lda #$51;0F
	sta PRINT_COLOR2
	;
	print TXT_SELECT_MISSION, $06, $06
	;
	lda #$41;0C				; darker grays
	sta PRINT_COLOR1
	lda #$31;0B
	sta PRINT_COLOR2
	;
	jsr print_mission_number
	;
	lda #$1B
	sta $FF06				; screen on
	;
select_mission_loop
	jsr SUB_JOYSTICK_CHECK
	tax					; save joy state
	and #$10
	bne sml_up
	;
	jsr SUB_DISPLAY_CURRENT_MISSION		; delay
	;
	lda VAR_MISSION_TEMP			; return with selection
	rts
	; --------
sml_up	txa
	and #$01
	bne sml_dn
	;
	ldx VAR_MISSION_TEMP			; increase
	inx
	cpx #$05				; >4?
	bne *+4
	ldx #$01				; overflow to 1
	stx VAR_MISSION_TEMP
	bne sml_1				; always jump
	;
sml_dn	txa
	and #$02
	bne select_mission_loop
	;	
	ldx VAR_MISSION_TEMP			; decrease
	dex
	bne *+4
	ldx #$04				; overflow to 4
	stx VAR_MISSION_TEMP
	;
sml_1	jsr print_mission_number
	;
	ldx #$FF
dly10	ldy #$E0
dly11	dey					; Delay
	bne dly11
	dex
	bne dly10
	;
	beq select_mission_loop			; always jump
	;
	; ---
	;
print_mission_number
	lda #$13
	sta PRINT_HPOS
	lda #$0A
	sta PRINT_VPOS
	;
	lda #$05
	sec
	sbc VAR_MISSION_TEMP
	clc
	adc #$30
	sta PRINT_CHAR
	;
	jmp SUB_PRINT_CHAR
	;
	;
	;
SUB_DISPLAY_CURRENT_MISSION
	jsr CLEAR_SCREEN
	;
	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
	lda #$41				; grays
	sta PRINT_COLOR1
	lda #$51
	sta PRINT_COLOR2
	;
	print TXT_MISSION, $0B, $06
	;
	print TXT_UNDERLINE, $0B, $08
	;
	lda #$1B
	sta PRINT_HPOS
	lda #$06
	sta PRINT_VPOS
	;
	lda #$05
	sec
	sbc VAR_MISSION_TEMP
	clc
	adc #$30
	sta PRINT_CHAR				;
	jsr SUB_PRINT_CHAR			;
	;
	lda VAR_MISSION_TEMP
	cmp #$04
	beq print_mission_1
	;
	print TXT_MISSION_2_3_4, $01, $0C
	;
	jmp display_current_mission_delay
	;
print_mission_1
	print TXT_MISSION_1, $02, $0C
	;
display_current_mission_delay
	lda #$1B
	sta $FF06				; Screen on
	;
	ldx #$FF				; delay before level!
dly01	ldy #$FF
dly02	lda #$04
dly03	sec
	sbc #$01
	bne dly03
	dey
	bne dly02
	dex
	bne dly01
	;
	rts
	;
	; --------
	;
LAB_ENDSEQUENCE
	jsr CLEAR_SCREEN
	;
	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
	lda #$41				; grays
	sta PRINT_COLOR1
	lda #$51
	sta PRINT_COLOR2
	;
	print TXT_WIN_1, $0A, $08
	print TXT_WIN_2, $02, $0C
	print TXT_WIN_3, $0C, $0F
	;
	lda #$1B
	sta $FF06				; screen on
	;
	ldx #$FF
dly04	ldy #$FF
dly05	lda #$20
dly06	sec
	sbc #$01
	bne dly06
	dey
	bne dly05
	dex
	bne dly04
	;
	rts
	;
	;
	;
IRQ_NULL
	pha
	txa
	pha
	tya
	pha	
	asl $FF09				; IRQ
IRQ_END	pla
	tay
	pla
	tax
	pla
	rti
	;
	; --- game code
	;
LAB_GAME_START
	lda #$01				; *** initial floor! (default: $01; ground floor)
	sta VAR_CURRENT_FLOOR
	;
	ldx #$00
	txa
cs2	sta CHARSET_2X2 + $740,x		; clear the extra chars used by the intro in the 2x2 font
	inx
	cpx #$C0
	bne cs2
	;
	sta VAR_RESTORE_FLAG
	sta VAR_PLAYER_INIT_FLAG
	sta VAR_DETONATOR_FLAGS
	sta VAR_DETONATOR_FLAGS + 1
	sta VAR_DETONATOR_FLAGS + 2
	sta VAR_DETONATOR_FLAGS + 3
	sta VAR_PRISONER_IS_CLOSE_FLAG
	;
	tax
	lda #$30
cs1	sta HS_SCORE,x				; clear score
	inx
	cpx #$06
	bne cs1
	;
	; ***
	;
LAB_NEW_MISSION
	sei
	lda #IRQ_NULL & 255			; clear the IRQ
	sta $FFFE				; IRQ Vector Lo Byte
	lda #IRQ_NULL >> 8
	sta $FFFF				; IRQ Vector Hi Byte
	cli
	;
	lda VAR_RESTORE_FLAG
	bne B_4995
	;
	lda #$00
	sta VAR_PRISONER_UPDATE
	;
	jsr SUB_SELECT_MISSION
	sta VAR_MISSION
	;
LAB_NEW_MISSION_A
	cmp #$01				; place prisoner based on mission
	beq B_497D
	cmp #$02
	beq B_4985
	cmp #$03
	beq B_498D
	lda #$FF
	sta VAR_PRISONER_FLOOR			; mission 1: no prisoner
	jmp L_49E6
	;
B_497D	lda #$00
	sta VAR_PRISONER_FLOOR			; in basement
	jmp L_49E6
	;
B_4985	lda #$02
	sta VAR_PRISONER_FLOOR			; first floor
	jmp L_49E6
	;
B_498D	lda #$03
	sta VAR_PRISONER_FLOOR			; second floor
	jmp L_49E6
	;
B_4995	lda VAR_CURRENT_FLOOR
	sta VAR_FLOOR_TEMP
	;
	jsr LAB_ELEVATOR_SCREEN
	sta VAR_CURRENT_FLOOR
	;
	lda VAR_PRISONER_FLOOR
	cmp #$FF
	beq B_49C2
	cmp VAR_FLOOR_TEMP
	bne B_49C2
	;
	lda #$00
	sta VAR_PRISONER_UPDATE
	lda VAR_TEMP_FLAG
	bne B_49C2
	lda #$01
	sta VAR_PRISONER_UPDATE
	lda VAR_CURRENT_FLOOR
	sta VAR_PRISONER_FLOOR
	;
B_49C2	lda VAR_KEYS
	sta VAR_TEMP_KEYS			; save values
	lda VAR_AMMO
	sta VAR_TEMP_AMMO
	lda VAR_HITS
	sta VAR_TEMP_HITS
	lda VAR_SCORE_01
	sta VAR_TEMP_SCORE1
	lda VAR_SCORE_02
	sta VAR_TEMP_SCORE2
	lda VAR_SCORE_03
	sta VAR_TEMP_SCORE3
L_49E6	jmp L_404F
	;
L_49E9	lda #$01
	sta VAR_PAUSE
	;
wait03	lda SCROLL_INDEX			; wait for scroll to finish
	cmp SCROLL_MAX
	bne wait03
	;
	lda #$00
	sta VAR_PRISONER_IS_CLOSE_FLAG
	sta VAR_TEMP_KEYS
	sta VAR_PRISONER_UPDATE
	lda #$99
	sta VAR_TEMP_AMMO
	lda #$00
	sta VAR_TEMP_HITS
	lda VAR_SCORE_01
	sta VAR_TEMP_SCORE1
	lda VAR_SCORE_02
	sta VAR_TEMP_SCORE2
	lda VAR_SCORE_03
	sta VAR_TEMP_SCORE3
	;
	jsr SUB_PAUSE
	;
	jsr SUB_SCREEN_TRANSITION_OUT
	;
	sei
	lda #IRQ_NULL & 255
	sta $FFFE
	lda #IRQ_NULL >> 8
	sta $FFFF
	cli
	;
	inc VAR_MISSION				; go to next mission
	;
	lda VAR_MISSION
	sta VAR_MISSION_TEMP
	;
	jsr SUB_DISPLAY_CURRENT_MISSION
	;
	lda #$01
	sta VAR_PLAYER_INIT_FLAG		; flag that player is at entrance
	;
	lda VAR_MISSION
	jmp LAB_NEW_MISSION_A
	;
	; ---
	;
L_404F	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
	; ---
	ldx #(VAR_PRISONER_HI - VAR_MAP_Y_COUNTER)
	lda #$00				; last var - first var
cl01	sta VAR_MAP_Y_COUNTER,x			; clear all game variables
	dex
	bpl cl01				; Loop until X is zero
	; ---
	;
	lda #$99				; initial ammo
	sta VAR_AMMO
	;
	lda VAR_RESTORE_FLAG			; restore data flag
	beq B_408C
	;
	lda VAR_TEMP_KEYS			; restore values
	sta VAR_KEYS
	lda VAR_TEMP_AMMO
	sta VAR_AMMO
	lda VAR_TEMP_HITS
	sta VAR_HITS
	lda VAR_TEMP_SCORE1
	sta VAR_SCORE_01
	lda VAR_TEMP_SCORE2
	sta VAR_SCORE_02
	lda VAR_TEMP_SCORE3
	sta VAR_SCORE_03
	;
B_408C	lda #$53				; default brick color
	ldx VAR_CURRENT_FLOOR
	bne *+4
	lda #$43				; darker brick color for basement only
	sta CHAR_COLORS+$41
	sta CHAR_COLORS+$42
	;
	jsr BUILD_MAP				; build current map
	;
	jsr SUB_REMOVE_DETONATOR_IF_NEEDED
	;
	sei
	;
	lda #$32				; rasterlines (starting values)
	sta reveal_1
	lda #$FA				; 
	sta reveal_2
	ldx #$01				; reset toggle
	stx reveal_toggle
	dex
	stx reveal_state
	;
	lda #IRQ_REVEAL_SCREEN & 255		; set up the IRQ
	sta $FFFE				; IRQ Vector Lo Byte
	lda #IRQ_REVEAL_SCREEN >> 8
	sta $FFFF				; IRQ Vector Hi Byte
	;
	lda #$1B
	sta $FF06				; screen on
	;
	cli
	;
	lda #$03
wait_for_reveal_1
	cmp reveal_state
	bne wait_for_reveal_1
	;
	lda #$00
	sta zp06
	sta SCROLL_INDEX
	sta SCROLL_MAX
	;
	lda #GAME_MAP >> 8
	sta zp06+1
	;
	lda #CHARSET_GAME >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
multi5	lda #$98
	sta $FF07				; Horiz Scroll
	;
	lda #$31				; *** fixed
	sta $FF16				; Multi Color 1
	;
	lda #$41				; *** fixed
	sta $FF17				; Multi Color 2
	;
	jsr SUB_SET_PLAYER_STARTING_POS
	;
	lda #$14				; prisoner x/y
	sta VAR_PRISONER_X
	lda #$72
	sta VAR_PRISONER_Y
	;
	lda #(GAME_MAP + 114 * $80 + 20) & 255
	sta VAR_PRISONER_LO_TEMP1
	sta VAR_PRISONER_LO_TEMP2
	lda #(GAME_MAP + 114 * $80 + 20) >> 8
	sta VAR_PRISONER_HI_TEMP1
	sta VAR_PRISONER_HI_TEMP2
	;
	lda #$00
	sta VAR_PRISONER_FLAGS
	;
	lda VAR_PRISONER_UPDATE
	bne B_4188
	;
	lda VAR_PRISONER_LO
	sta VAR_PRISONER_LO_TEMP1
	sta VAR_PRISONER_LO_TEMP2
	lda VAR_PRISONER_HI
	sta VAR_PRISONER_HI_TEMP1
	sta VAR_PRISONER_HI_TEMP2
	;
	lda VAR_PRISONER_HI			; calc prisoner x/y from address
	sec
	sbc #(GAME_MAP >> 8)			; this was $8F instead of $90 in the Plus/4 version (bug)
	sta ptr2+1
	lda VAR_PRISONER_LO
	sta ptr2
	and #$7F
	sta VAR_PRISONER_X
	asl ptr2
	rol ptr2+1
	lda ptr2+1
	sta VAR_PRISONER_Y
	;
	lda #$00
	sta VAR_PRISONER_UPDATE
	;
B_4188	lda VAR_PRISONER_LO
	sta ptr2
	lda VAR_PRISONER_HI
	sta ptr2+1
	;
	ldy #$03
B_4194	lda #$00
	sta (ptr2),y
	inc ptr2+1
	sta (ptr2),y
	tya
	ora #$80
	tay
	lda #$00
	sta (ptr2),y
	dec ptr2+1
	sta (ptr2),y
	tya
	and #$7F
	tay
	dey
	bpl B_4194
	;
	lda VAR_PRISONER_FLOOR
	cmp #$FF
	beq B_41C1
	;
	cmp VAR_CURRENT_FLOOR			; is prisoner on current floor?
	bne B_41C1
	;
	jsr L_6BD7				; deal with prisoner
	;
	jmp L_41C6
	;
B_41C1	lda #$01
	sta VAR_PRISONER_FLAG
	;
L_41C6	ldx #$00
cls07	lda #$00				; clear screen
	sta COLOR_MAP + $0000,x
	sta COLOR_MAP + $0100,x
	sta COLOR_MAP + $0200,x
	sta COLOR_MAP + $0300,x
	lda #$40				; space char
	sta CHAR_MAP + $0000,x
	sta CHAR_MAP + $0100,x
	sta CHAR_MAP + $0200,x
	sta CHAR_MAP + $0300,x
	dex
	bne cls07
	;
	lda #$01
	sta VAR_PLAYER_DIRECTION
	;
	jsr SUB_INIT_PANEL
	;
	jsr SUB_UPDATE_PANEL
	;
	lda VAR_RESTORE_FLAG
	bne B_4234
	;
	lda #$00				; sound init
	sta VAR_SOUND_REQUESTED
	sta VAR_SOUND_CURRENT
	sta VAR_SOUND_TICK
	sta $FF11
	;
	lda #TXT_WELCOME & 255			; Welcome scroll
	sta ptr2
	lda #TXT_WELCOME >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
B_4234	jsr SUB_UPDATE_SCREEN			; during prep
	;
	lda #$01
	sta reveal_state
	;
	; ---
	;
	lda #$02
wait_for_reveal_2
	cmp reveal_state
	bne wait_for_reveal_2
	;
	jsr SAVE_6X6_PLAYER
	;
	lda #$01
	sta VAR_RESTORE_FLAG
	;
	jsr SUB_SHOW_CURRENT_FLOOR
	;
	; ---
	;
	sei
	lda #IRQ_GAME_SCROLL & 255		; set up game irq
	sta $FFFE				; IRQ Vector Lo Byte
	lda #IRQ_GAME_SCROLL >> 8		;
	sta $FFFF				; IRQ Vector Hi Byte
	lda #$B3
	sta $FF0B				; Vert raster
	cli
	;
	lda #$CB				; sync to bottom of screen
	cmp $FF1D
	bcs *-3
	;
	ldx #$1B
	stx $FF06				; screen on
	;
	cmp $FF1D				; wait another frame
	bcc *-3
	cmp $FF1D
	bcs *-3
	;
	ldx #$27				; set up scroll colors (had to be delayed to avoid flash)
fl1	lda #$38				; scroll color (top)
	sta COLOR_MAP + 23 * 40,x
	lda #$68				; scroll color (bottom)
	sta COLOR_MAP + 24 * 40,x
	lda #$80				; fill lower 3 lines with space in 2x2 font
	sta CHAR_MAP + 22 * 40,x
	sta CHAR_MAP + 23 * 40,x
	sta CHAR_MAP + 24 * 40,x
	dex
	bpl fl1
	;
	; ---
	;
MAIN_GAME_LOOP
	lda #$A3
	cmp $FF1D
	bne *-3
	;
	lda VAR_EXPLOSION_FLASH_INDEX_1
	cmp #$FF
	beq game_over
	;
	lda VAR_LIFT_STATE
	cmp #$02
	beq B_48D4
	;
	lda VAR_WIN_STATE
	cmp #$01
	beq B_4893
	cmp #$02
	beq B_4896
	;
	lda KEYBOARD				; check if pause is requested
	cmp #$80				; (run/stop, changed from space)
	beq do_pause
	;
cheat10	bit lift_hotkey
cheat11	bit nuke_hotkey
cheat12	bit ghost_mode_hotkey
	;
	jmp MAIN_GAME_LOOP
	;
B_4893	jmp L_49E9
	;
B_4896	jmp LAB_WIN_GAME
	;
do_pause
	lda #$01
	sta VAR_PAUSE
	;
B_489E	lda KEYBOARD
	bne B_489E
	;
L_48A2	jsr SUB_JOYSTICK_CHECK
	and #$1F
	eor #$1F
	bne L_48C8
	;
	lda KEYBOARD
	beq B_48B2
	jmp L_48C8
	;
B_48B2	lda SCROLL_INDEX
	cmp SCROLL_MAX
	bne L_48A2
	;
	lda #TXT_PAUSING & 255	
	sta ptr2
	lda #TXT_PAUSING >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	jmp L_48A2
	;
L_48C8	lda KEYBOARD
	bne L_48C8
	lda #$00
	sta VAR_PAUSE				; clear pause
	;
	jmp MAIN_GAME_LOOP
	;
B_48D4	lda #$01
	sta VAR_PAUSE
	;
wait01	lda SCROLL_INDEX			; wait for scroll to finish
	cmp SCROLL_MAX
	bne wait01
	;
	jsr SUB_SCREEN_TRANSITION_OUT
	;
	jmp LAB_NEW_MISSION
	;
	; --- game over
	;
game_over
	jsr stop_game	
	;
	sei
	;
	jsr CLEAR_SCREEN
	;
	lda #CHARSET_2X2 >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
	lda #$26				; blues
	sta PRINT_COLOR1
	lda #$46
	sta PRINT_COLOR2
	;   
	print TXT_DEAD, 8, 10
	;
	lda #$1B
	sta $FF06				; screen on
	;
	jsr SUB_PAUSE
	jsr SUB_PAUSE
	;
	jmp LAB_CHECK_HIGHSCORE
	;
	; ---
	;
stop_game
	lda #$01				; enter pause mode
	sta VAR_PAUSE
	;
wait02	lda SCROLL_INDEX			; wait for scroll to finish
	cmp SCROLL_MAX
	bne wait02
	;
	jsr SUB_PAUSE
	;
	lda #$CC				; sync out of frame
	cmp $FF1D
	bcc *-3
	;
	sei
	lda #IRQ_NULL & 255			; clear the IRQ
	sta $FFFE
	lda #IRQ_NULL >> 8
	sta $FFFF
	cli
	;
	lda #$00				; stop all sounds
	sta $FF11
	;
	ldx #40*3-1
cls10	sta COLOR_MAP + 22 * 40,x		; clear scroll
	dex
	bpl cls10
	;
	ldx #$00
B_4915	lda CHAR_MAP + $01D7,x
	and #$0F
	ora #$30
	sta HS_SCORE,x				; copy score from screen
	inx
	cpx #$06				; length of score (6 chars)
	bne B_4915
	;
	jmp SUB_SCREEN_TRANSITION_OUT
	;
	; -------------------------------------
	;
S_426F	lda VAR_SCREEN_SCROLL
	and #$07
	beq B_427B
	;
	ldx #$00
	stx GAME_TICK+1
	;
B_427B	cmp #$01
	bne B_4282
	jmp L_432E
	;
B_4282	cmp #$02
	bne B_4289
	jmp L_43D6
	;
B_4289	cmp #$03
	bne B_4290
	jmp L_4480
	;
B_4290	cmp #$04
	bne B_4297
	jmp L_4528
	;
B_4297	jsr SAVE_6X6_PLAYER
	;
	lda VAR_PLAYER_SHOOTING_FLAG
	beq B_42AC
	jsr SUB_JOYSTICK_CHECK
	and #$10
	bne B_42A7
	rts
	;
	; --------
	;
B_42A7	lda #$00
	sta VAR_PLAYER_SHOOTING_FLAG
B_42AC	lda VAR_PLAYER_CONTROL_DELAY
cheat06	cmp #$03				; player speed!
	beq ctrl0
	inc VAR_PLAYER_CONTROL_DELAY
	rts
	; --------
ctrl0	lda #$00
	sta VAR_PLAYER_CONTROL_DELAY
	;
	jsr SUB_JOYSTICK_CHECK
	tax					; save joy state
	and #$01				; check up
	beq LAB_PLAYER_MOVE_UP
	;
	txa
	and #$02				; check down
	bne ctrl1
	jmp LAB_PLAYER_MOVE_DOWN
	;
ctrl1	txa
	and #$04				; check left
	bne ctrl2
	jmp LAB_PLAYER_MOVE_LEFT
	;
ctrl2	txa
	and #$08				; check right
	bne ctrl3
	jmp LAB_PLAYER_MOVE_RIGHT
	;
ctrl3	rts
	;
	; --------
	;
LAB_PLAYER_MOVE_UP
	lda #$01				; facing up
	sta VAR_PLAYER_DIRECTION
	;
	lda SAVED_6X6+1				; can we move up?
gm01	cmp #$2D				; any character below $2D is "free space"
	bcs cant_move_up			; these values will be changed for "ghost mode"
	;
	lda SAVED_6X6+2
gm02	cmp #$2D
	bcs cant_move_up
	;
	lda SAVED_6X6+3
gm03	cmp #$2D
	bcs cant_move_up
	;
	lda SAVED_6X6+4
gm04	cmp #$2D
	bcs cant_move_up
	;
	lda VAR_PLAYER_SCREEN_Y			; at top of the screen?
	cmp #$62
	beq B_4324
	cmp #$42				; new code: can't move higher than this
	beq cant_move_up
	;
	lda VAR_PLAYER_SCREEN_Y			; move player up (without scrolling)
	sec
	sbc #$08
	sta VAR_PLAYER_SCREEN_Y
	;
	lda #$00
	sta VAR_LIFT_PASS_MESSAGE_FLAG
	inc VAR_PLAYER_STEP_COUNTER
	;lda VAR_PLAYER_STEP_COUNTER
	;and #$01
	;sta VAR_STEP_SOUND_FLAG
	;
cant_move_up
	rts
	; --------
B_4324	lda VAR_SCREEN_SCROLL
	and #$08
	ora #$A1
	sta VAR_SCREEN_SCROLL
	;
L_432E	lda ptr3+1
	bne B_433E
	;
V_4332	lda VAR_PLAYER_SCREEN_Y
	sec
	sbc #$08
	sta VAR_PLAYER_SCREEN_Y
	;
	jmp L_4381
	;
B_433E	dec ptr3+1
	;
	lda ptr1
	sec
	sbc #$80
	sta ptr1
	bcs B_434B
	dec ptr1+1
B_434B	lda VAR_PLAYER_SCREEN_Y
	clc
	adc #$08
	sta VAR_PLAYER_SCREEN_Y
	;
	lda VAR_BULLETS_FLAG
	and #$01
	beq B_4364
	lda VAR_BULLET_Y_1
	clc
	adc #$08
	sta VAR_BULLET_Y_1
	;
B_4364	lda VAR_BULLETS_FLAG
	and #$10
	beq B_4374
	lda VAR_BULLET_Y_2
	clc
	adc #$08
	sta VAR_BULLET_Y_2
	;
B_4374	lda VAR_SCREEN_SCROLL
	sec
	sbc #$10
	sta VAR_SCREEN_SCROLL
	and #$F0
	bne B_4389
L_4381	lda VAR_SCREEN_SCROLL
	and #$08
	sta VAR_SCREEN_SCROLL
B_4389	rts
	;
	; --------
	;
LAB_PLAYER_MOVE_DOWN
	lda #$02				; facing down
	sta VAR_PLAYER_DIRECTION
	;
	lda SAVED_6X6+31			; can we move down?
gm11	cmp #$2D
	bcs cant_move_down
	;
	lda SAVED_6X6+32
gm12	cmp #$2D
	bcs cant_move_down
	;
	lda SAVED_6X6+33
gm13	cmp #$2D
	bcs cant_move_down
	;
	lda SAVED_6X6+34
gm14	cmp #$2D
	bcs cant_move_down
	;
	lda VAR_PLAYER_SCREEN_Y			; at bottom of screen?
	cmp #$B2
	beq B_43CC
	cmp #$C2				; new code: don't move lower than this
	beq cant_move_down
	;
	lda VAR_PLAYER_SCREEN_Y			; move player down (without scrolling)
	clc
	adc #$08
	sta VAR_PLAYER_SCREEN_Y
	;
	lda #$00
	sta VAR_LIFT_PASS_MESSAGE_FLAG
	inc VAR_PLAYER_STEP_COUNTER
	;lda VAR_PLAYER_STEP_COUNTER
	;and #$01
	;sta VAR_STEP_SOUND_FLAG
	;
cant_move_down
	rts
	; --------
B_43CC	lda VAR_SCREEN_SCROLL
	and #$08
	ora #$A2
	sta VAR_SCREEN_SCROLL
	;
L_43D6	lda ptr3+1
	cmp #$6B
	bne B_43E8
	lda VAR_PLAYER_SCREEN_Y
	clc
	adc #$08
	sta VAR_PLAYER_SCREEN_Y
	jmp L_442B
	;
B_43E8	inc ptr3+1
	lda ptr1
	clc
	adc #$80
	sta ptr1
	bcc B_43F5
	inc ptr1+1
B_43F5	lda VAR_PLAYER_SCREEN_Y
	sec
	sbc #$08
	sta VAR_PLAYER_SCREEN_Y
	;
	lda VAR_BULLETS_FLAG
	and #$01
	beq B_440E
	lda VAR_BULLET_Y_1
	sec
	sbc #$08
	sta VAR_BULLET_Y_1
	;
B_440E	lda VAR_BULLETS_FLAG
	and #$10
	beq B_441E
	lda VAR_BULLET_Y_2
	sec
	sbc #$08
	sta VAR_BULLET_Y_2
	;
B_441E	lda VAR_SCREEN_SCROLL
	sec
	sbc #$10
	sta VAR_SCREEN_SCROLL
	and #$F0
	bne B_4433
L_442B	lda VAR_SCREEN_SCROLL
	and #$08
	sta VAR_SCREEN_SCROLL
B_4433	rts
	;
	; --------
	;
LAB_PLAYER_MOVE_LEFT
	lda #$03				; facing left
	sta VAR_PLAYER_DIRECTION
	;
	lda SAVED_6X6+6
gm21	cmp #$2D
	bcs cant_move_left
	;
	lda SAVED_6X6+12
gm22	cmp #$2D
	bcs cant_move_left
	;
	lda SAVED_6X6+18
gm23	cmp #$2D
	bcs cant_move_left
	;
	lda SAVED_6X6+24
gm24	cmp #$2D
	bcs cant_move_left
	;
	lda VAR_PLAYER_SCREEN_X
	cmp #$48
	beq B_4476
	cmp #$20				; new code: don't allow less than $20
	beq cant_move_left
	;
	lda VAR_PLAYER_SCREEN_X
	sec
	sbc #$08
	sta VAR_PLAYER_SCREEN_X
	;
	lda #$00
	sta VAR_LIFT_PASS_MESSAGE_FLAG
	inc VAR_PLAYER_STEP_COUNTER
	;lda VAR_PLAYER_STEP_COUNTER
	;and #$01
	;sta VAR_STEP_SOUND_FLAG
	;
cant_move_left
	rts
	; --------
B_4476	lda VAR_SCREEN_SCROLL
	and #$08
	ora #$D3
	sta VAR_SCREEN_SCROLL
L_4480	lda ptr3
	bne B_4490
	lda VAR_PLAYER_SCREEN_X
	sec
	sbc #$08
	sta VAR_PLAYER_SCREEN_X
	jmp L_44D3
	;
B_4490	dec ptr3
	lda ptr1
	sec
	sbc #$01
	sta ptr1
	bcs B_449D
	dec ptr1+1
B_449D	lda VAR_PLAYER_SCREEN_X
	clc
	adc #$08
	sta VAR_PLAYER_SCREEN_X
	;
	lda VAR_BULLETS_FLAG
	and #$01
	beq B_44B6
	lda VAR_BULLET_X_1
	clc
	adc #$08
	sta VAR_BULLET_X_1
	;
B_44B6	lda VAR_BULLETS_FLAG
	and #$10
	beq B_44C6
	lda VAR_BULLET_X_2
	clc
	adc #$08
	sta VAR_BULLET_X_2
	;
B_44C6	lda VAR_SCREEN_SCROLL
	sec
	sbc #$10
	sta VAR_SCREEN_SCROLL
	and #$F0
	bne B_44DB
L_44D3	lda VAR_SCREEN_SCROLL
	and #$08
	sta VAR_SCREEN_SCROLL
B_44DB	rts
	;
	; --------
	;
LAB_PLAYER_MOVE_RIGHT
	lda #$04				; facing right
	sta VAR_PLAYER_DIRECTION
	;
	lda SAVED_6X6+11
gm31	cmp #$2D
	bcs cant_move_right
	;
	lda SAVED_6X6+17
gm32	cmp #$2D
	bcs cant_move_right
	;
	lda SAVED_6X6+23
gm33	cmp #$2D
	bcs cant_move_right
	;
	lda SAVED_6X6+29
gm34	cmp #$2D
	bcs cant_move_right
	;
	lda VAR_PLAYER_SCREEN_X
	cmp #$B0
	beq B_451E
	cmp #$D8				; new code: never allow higher than $D8 (map edge)
	beq cant_move_right
	;
	lda VAR_PLAYER_SCREEN_X			; move player right
	clc
	adc #$08
	sta VAR_PLAYER_SCREEN_X
	;
	lda #$00
	sta VAR_LIFT_PASS_MESSAGE_FLAG
	inc VAR_PLAYER_STEP_COUNTER
	;lda VAR_PLAYER_STEP_COUNTER
	;and #$01
	;sta VAR_STEP_SOUND_FLAG
	;
cant_move_right
	rts
	; --------
B_451E	lda VAR_SCREEN_SCROLL
	and #$08
	ora #$D4
	sta VAR_SCREEN_SCROLL
L_4528	lda ptr3
	cmp #$64
	bne B_453A
	lda VAR_PLAYER_SCREEN_X
	clc
	adc #$08
	sta VAR_PLAYER_SCREEN_X
	jmp L_457D
	;
B_453A	inc ptr3
	lda ptr1
	clc
	adc #$01
	sta ptr1
	bcc B_4547
	inc ptr1+1
B_4547	lda VAR_PLAYER_SCREEN_X
	sec
	sbc #$08
	sta VAR_PLAYER_SCREEN_X
	;
	lda VAR_BULLETS_FLAG
	and #$01
	beq B_4560
	lda VAR_BULLET_X_1
	sec
	sbc #$08
	sta VAR_BULLET_X_1
	;
B_4560	lda VAR_BULLETS_FLAG
	and #$10
	beq B_4570
	lda VAR_BULLET_X_2
	sec
	sbc #$08
	sta VAR_BULLET_X_2
	;
B_4570	lda VAR_SCREEN_SCROLL
	sec
	sbc #$10
	sta VAR_SCREEN_SCROLL
	and #$F0
	bne B_4585
V_457D
L_457D	lda VAR_SCREEN_SCROLL
	and #$08
	sta VAR_SCREEN_SCROLL
B_4585	rts
	;
	; --------
	;
SUB_MOVE_BULLETS
	lda VAR_BULLETS_FLAG
	and #$02
	beq mb_1
	;
	lda VAR_BULLETS_FLAG			; clear lower nibble
	and #$F0
	sta VAR_BULLETS_FLAG
	;
	lda #$00				; clear bullet 1
	sta VAR_BULLET_X_1
	sta VAR_BULLET_Y_1
	sta VAR_BULLET_MISC_1
	;
	beq mb_2				; always jump
	;
mb_1	lda VAR_BULLETS_FLAG
	and #$20
	beq mb_2
	;
	lda VAR_BULLETS_FLAG			; clear upper nibble
	and #$0F
	sta VAR_BULLETS_FLAG
	;
	lda #$00				; clear bullet 2
	sta VAR_BULLET_X_2
	sta VAR_BULLET_Y_2
	sta VAR_BULLET_MISC_2
	;
mb_2	jsr SUB_MOVE_BULLET_1
	jsr SUB_MOVE_BULLET_2
	;
	jsr SUB_JOYSTICK_CHECK
	and #$10
	beq mb_3
	;
	lda #$00
	sta VAR_FIRE_FLAG
	;
	rts
	;
mb_3	lda VAR_FIRE_FLAG
	beq mb_4
	;
	rts
	;
mb_4	inc VAR_FIRE_FLAG
	;
	lda VAR_BULLETS_FLAG
	and #$0F
	bne B_463F
	;
	lda VAR_BULLETS_FLAG
	and #$F0
	beq mb_5
	;
	lda VAR_BULLET_MISC_2
	sec
	sbc #$08
	bcs mb_5
	;
ret1	rts
	; --------
mb_5	lda VAR_AMMO				; does the player have ammo left?
	bne have_ammo_1
	;
LAB_OUT_OF_AMMO
	lda SCROLL_INDEX
	cmp SCROLL_MAX
	bne ret1
	;
	lda #TXT_OUT_OF_AMMO & 255
	sta ptr2
	lda #TXT_OUT_OF_AMMO >> 8
	sta ptr2+1
	jmp SUB_SET_SCROLLTEXT
	;
	; --------
	;
have_ammo_1
	jsr SUB_SHOT_TAKEN			; bullet 1 fired
	;
	ldx VAR_PLAYER_DIRECTION
	stx VAR_BULLET_DIRECTION_1
	dex
	;
	lda VAR_PLAYER_SCREEN_X
	clc
	adc bullet_table_1,x
	sta VAR_BULLET_X_1
	;
	lda VAR_PLAYER_SCREEN_Y
	clc
	adc bullet_table_2,x
	sta VAR_BULLET_Y_1
	;
	lda VAR_BULLETS_FLAG			; flag bullet 1
	eor #$01
	sta VAR_BULLETS_FLAG
	;
	lda #$00
	sta VAR_BULLET_MISC_1
	rts
	; --------
B_463F	lda VAR_BULLETS_FLAG
	and #$F0
	bne ret2
	;
	lda VAR_BULLET_MISC_1
	sec
	sbc #$08
	bcs ret2
	;
	lda VAR_AMMO				; does the player have ammo left?
	bne have_ammo_2
	;
	jmp LAB_OUT_OF_AMMO
	;
have_ammo_2
	jsr SUB_SHOT_TAKEN			; bullet 2 fired
	;
	ldx VAR_PLAYER_DIRECTION
	stx VAR_BULLET_DIRECTION_2
	dex
	;
	lda VAR_PLAYER_SCREEN_X
	clc
	adc bullet_table_1,x
	sta VAR_BULLET_X_2
	;
	lda VAR_PLAYER_SCREEN_Y
	clc
	adc bullet_table_2,x
	sta VAR_BULLET_Y_2
	;
	lda VAR_BULLETS_FLAG			; flag bullet 2
	eor #$10
	sta VAR_BULLETS_FLAG
	;
	lda #$00
	sta VAR_BULLET_MISC_2
	;
ret2	rts
	;
	; --- new code: take care of everything when shot is taken
	;
SUB_SHOT_TAKEN
	sed
	sec
cheat05	sbc #$01				; *** no ammo loss cheat
	sta VAR_AMMO
	cld
	;
	ldx VAR_PLAYER_DIRECTION		; update player sprite
	lda player_flash_lo,x
	sta sprite_src_ptr
	lda player_flash_hi,x
	sta sprite_src_ptr+1
	;
	lda #$01				; set flag
	sta VAR_PLAYER_SHOOTING_FLAG
	;
	lda #SND_SHOOT
	sta VAR_SOUND_REQUESTED			; play sound
	;
	rts
	;
	; --------
	;
SUB_MOVE_BULLET_1
	lda VAR_BULLETS_FLAG			; check flag to see if bullet 1 is active
	and #$01
	bne B_4702
	rts
	; --------
B_4702	inc VAR_BULLET_MISC_1
	;
	lda VAR_BULLET_DIRECTION_1
	cmp #$01
	bne B_4724
	;
	lda VAR_BULLET_Y_1			; bullet 1 up
	sec
	sbc #$08
	sta VAR_BULLET_Y_1
	cmp #$3A
	bne B_4721
	;
	lda VAR_BULLETS_FLAG
	eor #$02
	sta VAR_BULLETS_FLAG
	;
B_4721	rts
	;
B_4724	cmp #$02
	bne B_4740
	;
	lda VAR_BULLET_Y_1			; bullet 1 down
	clc
	adc #$08
	sta VAR_BULLET_Y_1
	cmp #$E2
	bne B_473D
	;
	lda VAR_BULLETS_FLAG
	eor #$02
	sta VAR_BULLETS_FLAG
	;
B_473D	rts
	;
B_4740	cmp #$03
	bne B_475C
	;
	lda VAR_BULLET_X_1			; bullet 1 left
	sec
	sbc #$08
	sta VAR_BULLET_X_1
	cmp #$20
	bne B_4759
	;
	lda VAR_BULLETS_FLAG
	eor #$02
	sta VAR_BULLETS_FLAG
	;
B_4759	rts
	;
B_475C	lda VAR_BULLET_X_1			; bullet 1 right
	clc
	adc #$08
	sta VAR_BULLET_X_1
	cmp #$F0;#$F2				; BUGFIX! boundary check corrected
	bne B_4771
	;
	lda VAR_BULLETS_FLAG
	eor #$02
	sta VAR_BULLETS_FLAG
	;
B_4771	rts
	;
	; --------
	;
SUB_MOVE_BULLET_2
	inc VAR_BULLET_MISC_2
	;
	lda VAR_BULLETS_FLAG			; check flag to see if bullet 2 is active
	and #$10
	bne B_4780
	rts
	; --------
B_4780	lda VAR_BULLET_DIRECTION_2
	cmp #$01
	bne B_479F
	;
	lda VAR_BULLET_Y_2			; bullet 2 up
	sec
	sbc #$08
	sta VAR_BULLET_Y_2
	cmp #$3A
	bne B_479C
	;
	lda VAR_BULLETS_FLAG
	eor #$20
	sta VAR_BULLETS_FLAG
	;
B_479C	rts
	;
B_479F	cmp #$02
	bne B_47BB
	;
	lda VAR_BULLET_Y_2			; bullet 2 down
	clc
	adc #$08
	sta VAR_BULLET_Y_2
	cmp #$E2
	bne B_47B8
	;
	lda VAR_BULLETS_FLAG
	eor #$20
	sta VAR_BULLETS_FLAG
	;
B_47B8	rts
	;
B_47BB	cmp #$03
	bne B_47D7
	;
	lda VAR_BULLET_X_2			; bullet 2 left
	sec
	sbc #$08
	sta VAR_BULLET_X_2
	cmp #$20
	bne B_47D4
	;
	lda VAR_BULLETS_FLAG
	eor #$20
	sta VAR_BULLETS_FLAG
	;
B_47D4	rts
	;
B_47D7	lda VAR_BULLET_X_2			; bullet 2 right
	clc
	adc #$08
	sta VAR_BULLET_X_2
	cmp #$F0				; correct limit
	bne B_47EC
	;
	lda VAR_BULLETS_FLAG
	eor #$20
	sta VAR_BULLETS_FLAG
	;
B_47EC	rts
	;
	; ---
	;
LAB_WIN_GAME
	jsr stop_game
	;
	jsr LAB_ENDSEQUENCE
	;
	jmp LAB_CHECK_HIGHSCORE
	;
IRQ_GAME_SCROLL
	pha
	;
	lda #CHARSET_2X2 >> 8			; change to 2x2 set
	sta $FF13
	;
	txa
	pha
	tya
	pha
	;
	; ---
	;
	lda SCROLL_X				; set horiz. scroll
hires9	ora #$80
	sta $FF07
	;
	jsr LAB_PLAY_SOUND			; the only place it's called from (once per frame)
	;
	; --- step
	;
	ldy #$00				; y will be volume to be set
	ldx VAR_STEP_SOUND_FLAG			; x will be flag
	lda VAR_SOUND_CURRENT			; is there a current sound?
	bne step_x				; yes - do nothing, change nothing
	lda VAR_PLAYER_STEP_COUNTER
	and #$01
	beq step_clear_flag			; every other step (when there's no sound), clear the flag
	;
	cpx #$02				; when the flag is on 2, we're done, clear sound
	beq step_q
	;
	cpx #$01
	beq step_sound_off			; when flag is on 1, clear sound (and allow flag to become 2)
	;
	lda VAR_PLAYER_STEP_COUNTER		; otherwise, play sound (and flag will become 1)
	and #$02
	asl a
	asl a
	ora #$C0				; sound changes with every step
	sta $FF0F
	lda #$03
	sta $FF10
	ldy #$42				; very quiet noise
	;
step_sound_off
	inx
	db $2C
	;
step_clear_flag
	ldx #$00
	stx VAR_STEP_SOUND_FLAG
	;
step_q	sty $FF11
step_x	;
	; ---
	;
	lda VAR_PAUSE
	bne B_4B83
	;
	; ---
	;
B_4B79	lda VAR_SCREEN_SCROLL
	and #$08
	bne B_4B83
	;
	jsr S_426F
	;
B_4B83	lda VAR_PLAYER_SHOOTING_FLAG		; check shoot flag
	beq player_not_shooting
	cmp #$05
	beq player_remove_flash
	;
	inc VAR_PLAYER_SHOOTING_FLAG
	bne gs_cont				; always jump
	;
player_remove_flash
	ldx VAR_PLAYER_DIRECTION
	lda player_shoot_lo,x
	sta sprite_src_ptr
	lda player_shoot_hi,x
	sta sprite_src_ptr+1
	bne gs_cont				; always jump
	;
player_not_shooting
	ldx VAR_PLAYER_DIRECTION
	lda VAR_PLAYER_STEP_COUNTER
	clc
	adc #$01
	and #$02				; every 2nd step, change sprites
	beq ps2
	;
	inx					; add 4 to index
	inx
	inx
	inx
	;
ps2	lda player_sprite_lo,x
	sta sprite_src_ptr
	lda player_sprite_hi,x
	sta sprite_src_ptr+1 
	;
gs_cont	lda #IRQ_GAME_MAIN & 255
	sta $FFFE
	lda #IRQ_GAME_MAIN >> 8
	sta $FFFF
	lda #$CF
	sta $FF0B
	;
	asl $FF09				; IRQ - Acknowledge IRQ
	;
	pla
	tay
	pla
	tax
	pla
	rti
	
IRQ_GAME_MAIN
	pha
	txa
	pha
	tya
	pha
	asl $FF09	
	;
	jsr game_code_top
	;
	lda #IRQ_GAME_SCROLL & 255
	sta $FFFE
	lda #IRQ_GAME_SCROLL >> 8
	sta $FFFF
	lda #$B3
	sta $FF0B
	;
	pla
	tay
	pla
	tax
	pla
	rti

scroll_2_chars	macro addr
	lda addr+1
	sta addr
	lda addr+41
	sta addr+40
	endm

game_code_top
	lda #CHARSET_GAME >> 8			; Set charset
	sta $FF13				; Char. Gen
	;
multi6	lda #$98
	sta $FF07				; Horiz Scroll
	;
	jsr SUB_UPDATE_SCORE
	jsr SUB_UPDATE_PANEL
	jsr SUB_PAUSE_CHECK			; keyboard check
	;
	jsr SUB_SCREEN_COLOR_FLASH
	;
	lda VAR_PAUSE
	bne GAME_SLICE_COMMON
	;
	lda VAR_SCREEN_SCROLL
	and #$07
	bne GAME_SLICE_1
	;
	inc GAME_TICK+1
GAME_TICK
	lda #$00
	cmp #$01
	beq GAME_SLICE_1
	cmp #$02
	beq GAME_SLICE_2
	;
	; GAME SLICE 3
	lda #$00				; reset tick to zero
	sta GAME_TICK+1
	;
	lda #$03
	sta move_prisoner_flag+1
	;
	jsr S_5B1B
	jsr S_520C
	jsr SUB_MOVE_BULLETS
	jmp GAME_SLICE_COMMON
	;
GAME_SLICE_2
	jsr S_4D34
	jsr S_520C
	jsr SUB_MOVE_BULLETS
	jsr SUB_CHECK_INACTIVE
	;
	lda #$01
	sta move_prisoner_flag+1
	;
	jsr S_5B1B
	jsr SUB_JEWELS_CHECK
	jsr SUB_CASTLE_EXIT_CHECK
	jmp GAME_SLICE_COMMON
	;
GAME_SLICE_1
	jsr SUB_UPDATE_SCREEN
	;
GAME_SLICE_COMMON
	lda SCROLL_X
	sec
	sbc #$04				; scroll speed
	sta SCROLL_X
	bcc sps4				; overflow? (next char)
	rts
	;
sps4	adc #$08				; yes
	sta SCROLL_X
	;
	;dec $FF19
	;
	scroll_2_chars CHAR_MAP + 23*40 + 0	; unrolled loop, nearly twice as fast
	scroll_2_chars CHAR_MAP + 23*40 + 1
	scroll_2_chars CHAR_MAP + 23*40 + 2
	scroll_2_chars CHAR_MAP + 23*40 + 3
	scroll_2_chars CHAR_MAP + 23*40 + 4
	scroll_2_chars CHAR_MAP + 23*40 + 5
	scroll_2_chars CHAR_MAP + 23*40 + 6
	scroll_2_chars CHAR_MAP + 23*40 + 7
	scroll_2_chars CHAR_MAP + 23*40 + 8
	scroll_2_chars CHAR_MAP + 23*40 + 9
	scroll_2_chars CHAR_MAP + 23*40 + 10
	scroll_2_chars CHAR_MAP + 23*40 + 11
	scroll_2_chars CHAR_MAP + 23*40 + 12
	scroll_2_chars CHAR_MAP + 23*40 + 13
	scroll_2_chars CHAR_MAP + 23*40 + 14
	scroll_2_chars CHAR_MAP + 23*40 + 15
	scroll_2_chars CHAR_MAP + 23*40 + 16
	scroll_2_chars CHAR_MAP + 23*40 + 17
	scroll_2_chars CHAR_MAP + 23*40 + 18
	scroll_2_chars CHAR_MAP + 23*40 + 19
	scroll_2_chars CHAR_MAP + 23*40 + 20
	scroll_2_chars CHAR_MAP + 23*40 + 21
	scroll_2_chars CHAR_MAP + 23*40 + 22
	scroll_2_chars CHAR_MAP + 23*40 + 23
	scroll_2_chars CHAR_MAP + 23*40 + 24
	scroll_2_chars CHAR_MAP + 23*40 + 25
	scroll_2_chars CHAR_MAP + 23*40 + 26
	scroll_2_chars CHAR_MAP + 23*40 + 27
	scroll_2_chars CHAR_MAP + 23*40 + 28
	scroll_2_chars CHAR_MAP + 23*40 + 29
	scroll_2_chars CHAR_MAP + 23*40 + 30
	scroll_2_chars CHAR_MAP + 23*40 + 31
	scroll_2_chars CHAR_MAP + 23*40 + 32
	scroll_2_chars CHAR_MAP + 23*40 + 33
	scroll_2_chars CHAR_MAP + 23*40 + 34
	scroll_2_chars CHAR_MAP + 23*40 + 35
	scroll_2_chars CHAR_MAP + 23*40 + 36
	scroll_2_chars CHAR_MAP + 23*40 + 37
	scroll_2_chars CHAR_MAP + 23*40 + 38
	;
;	ldx #$D8
;sps0	lda CHAR_MAP + $02C1,x			; scroll chars on screen by copying
;	sta CHAR_MAP + $02C0,x
;	lda CHAR_MAP + $02E9,x
;	sta CHAR_MAP + $02E8,x
;	inx
;	bne sps0				; Loop until X is zero
	;
	;inc $FF19
	;
;S_594B	;
	;
	lda SCROLL_INDEX			; any text left?
	cmp SCROLL_MAX
	bne sps1
	;
	lda #$80				; blank char
	bne scroll_put_2			; always jump
	;
;	sta CHAR_MAP + $03BF
;	sta CHAR_MAP + $03E7
;	rts
;;$595C  --------
sps1	ldx SCROLL_INDEX
	lda DAT_SCROLLTEXT,x
	asl a					; mul * 4
	asl a
	tax
	ldy #$01
	lda SCROLL_2x_FLAG
	beq scroll_put
	;
	inc SCROLL_INDEX
	ldy #$00
	;
	cpx #($1D*4)				; exceptions for apostrophe
	beq skip_2nd_col
	cpx #($2F*4) 				; and narrow "I"
	bne dont_skip_2nd_col
	;
skip_2nd_col
	sty SCROLL_2x_FLAG			; skip the 2nd column, narrow letters
	beq sps1				; always jump
	;
dont_skip_2nd_col
	inx
	inx
	;
scroll_put
	sty SCROLL_2x_FLAG
	txa
scroll_put_2
	sta CHAR_MAP + $03BF			; upper row
	clc
	adc #$01
	sta CHAR_MAP + $03E7			; lower row
	;
spsx	rts
	;
	; --------
	;
IRQ_REVEAL_SCREEN
	pha
	txa
	pha
	tya
	pha
	asl $FF09				; IRQ - Acknowledge IRQ
	;
	lda reveal_toggle
	eor #$01 
	sta reveal_toggle
	beq B_4C35
	;
	jsr S_4C44
	;
	lda reveal_1
	jmp set_reveal_rasterline
	;
B_4C35	lda #$1B				; screen on
	sta $FF06
	;
	lda reveal_2
set_reveal_rasterline
	sec
	sbc #$30
	sta $FF0B				; Vert raster
	;
	pla
	tay
	pla
	tax
	pla
	rti
	; --------
S_4C44	lda #$7B				; *must* be $7B (not $0B)
	sta $FF06				; screen off
	;
	lda reveal_state
	beq B_4C67
	cmp #$01
	beq B_4C87
	rts
	; --------
B_4C67	lda reveal_1
	cmp #$92
	beq B_4C81
	;
	lda reveal_1
	clc
	adc #$04
	sta reveal_1
	;
	lda reveal_2
	sec
	sbc #$04
	sta reveal_2
	;
	rts
	; --------
B_4C81	lda #$03
	sta reveal_state
	rts
	; --------
B_4C87	lda reveal_1
	cmp #$32
	beq B_4CA1
	;
	lda reveal_1
	sec
	sbc #$04
	sta reveal_1
	;
	lda reveal_2
	clc
	adc #$04
	sta reveal_2
	;
	rts
	; --------
B_4CA1	lda #$02
	sta reveal_state
	rts
	;
	; --------
	;
S_4D34	ldy #$00
B_4D36	ldx inner_check_table,y
	lda SAVED_6X6,x
	and #$38
	bne B_4D57
	iny
	cpy #$10
	bne B_4D36
	;
	jsr RESTORE_6X6
	;
	jmp L_503A
	;
B_4D57	ldy #$00
B_4D59	lda SAVED_6X6,y
	cmp #$10
	beq B_4DBF
	cmp #$14
	bne B_4D67
	jmp L_4EAE
	;
B_4D67	cmp #$18
	bne B_4D6E
	jmp L_4ED8
	;
B_4D6E	cmp #$1C
	bne B_4D75
	jmp L_4F02
	;
B_4D75	cmp #$28
	bne B_4D7C
	jmp L_4F32
	;
B_4D7C	cmp #$12
	bne B_4D97
	cpy #$00
	beq B_4D97
	cpy #$01
	beq B_4D97
	cpy #$02
	beq B_4D97
	cpy #$03
	beq B_4D97
	cpy #$04
	beq B_4D97
	jmp LAB_AMMO_PICKED_UP
	;
B_4D97	cmp #$20
	bne B_4D9E
	jmp L_4F8A
	;
B_4D9E	cmp #$08
	bne B_4DA5
	jmp L_4FFC
	;
B_4DA5	iny
	cpy #$05
	beq B_4DA5
	cpy #$0B
	beq B_4DA5
	cpy #$11
	beq B_4DA5
	cpy #$17
	beq B_4DA5
	cpy #$1D
	beq B_4DA5
	cpy #$1E
	bne B_4D59
	rts
	; --------
B_4DBF	lda VAR_KEYS				; picked up a key
	cmp #$99				; new code: stop at 99
	beq dont_overflow_keys
	sed
	clc
	adc #$01
	sta VAR_KEYS
	cld
	;  
dont_overflow_keys
	lda #SND_ITEM_PICKUP
	sta VAR_SOUND_REQUESTED
	;
	lda #TXT_FOUND & 255			; "found"
	sta ptr2
	lda #TXT_FOUND >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #TXT_A_DOOR_KEY & 255
	sta ptr2
	lda #TXT_A_DOOR_KEY >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #$00
	sta SAVED_6X6,y
	iny
	sta SAVED_6X6,y
	jmp RESTORE_6X6
	;
L_4DF1	tya
	tax
	lda VAR_BULLET_DIRECTION_TEMP
	cmp #$01
	bne B_4E0E
	txa
	sec
	sbc #$06
	tax
	lda SAVED_6X6,x
	bne RESTORE_6X6
	inx
	lda SAVED_6X6,x
	bne RESTORE_6X6
	dex
	jmp L_4E5D
	;
B_4E0E	cmp #$02
	bne B_4E26
	txa
	clc
	adc #$06
V_4E16	tax
	lda SAVED_6X6,x
	bne RESTORE_6X6
	inx
	lda SAVED_6X6,x
	bne RESTORE_6X6
	dex
	jmp L_4E5D
	;
B_4E26	cmp #$03
	bne B_4E55
	lda zp02
	sta ptr2
	lda zp02+1
	sta ptr2+1
	ldy #$00
	lda ptr2
	sec
	sbc #$02
	sta ptr2
	bcs B_4E3F
	dec ptr2+1
B_4E3F	lda (ptr2),y
	bne RESTORE_6X6
	lda #$10
	sta (ptr2),y
	lda #$11
	sta SAVED_6X6,x
	inx
	lda #$00
	sta SAVED_6X6,x
	jmp RESTORE_6X6
	;
B_4E55	inx
	inx
	lda SAVED_6X6,x
	bne RESTORE_6X6
	dex
L_4E5D	lda #$00
	sta SAVED_6X6,y
	iny
	sta SAVED_6X6,y
	lda #$10
	sta SAVED_6X6,x
	inx
	lda #$11
	sta SAVED_6X6,x
	;
RESTORE_6X6
	lda zp06				; new code: ALWAYS
	pha					; save pointer (zp06) to stack
	lda zp06+1				; (since the loop below will modify it)
	pha
	;
	ldx #$00
restore_6x6_loop
	ldy #$00
	lda SAVED_6X6,x				; restore ("place") the saved 6x6 chars
	sta (zp06),y				; back onto the map
	iny
	lda SAVED_6X6+1,x
	sta (zp06),y
	iny
	lda SAVED_6X6+2,x
	sta (zp06),y
	iny
	lda SAVED_6X6+3,x
	sta (zp06),y
	iny
	lda SAVED_6X6+4,x
	sta (zp06),y
	iny
	lda SAVED_6X6+5,x
	sta (zp06),y
	;
	lda zp06
	clc
	adc #$80
	sta zp06
	bcc *+4
	inc zp06+1
	;
	inx
	inx
	inx
	inx
	inx
	inx
	cpx #36
	bne restore_6x6_loop
	;
	pla
	sta zp06+1
	pla
	sta zp06
	;
	rts
	; --------
L_4EAE	
	lda #TXT_FOUND & 255			; "found"
	sta ptr2
	lda #TXT_FOUND >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #SND_ITEM_PICKUP
	sta VAR_SOUND_REQUESTED
	;
	lda VAR_SCORE_ADD
	clc
	adc #25					; score += 2500
	sta VAR_SCORE_ADD
	;
L_4EC7	jsr SUB_REMOVE_2X2_FROM_6X6
	;
	lda #TXT_A_PAINTING & 255
	sta ptr2
	lda #TXT_A_PAINTING >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	jmp RESTORE_6X6
	;
L_4ED8	
	lda #TXT_FOUND & 255			; "found"
	sta ptr2
	lda #TXT_FOUND >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #SND_ITEM_PICKUP
	sta VAR_SOUND_REQUESTED
	;
	lda VAR_SCORE_ADD
	clc
	adc #15					; score += 1500
	sta VAR_SCORE_ADD
	;
L_4EF1	jsr SUB_REMOVE_2X2_FROM_6X6
	;
	lda #TXT_A_VASE & 255			; $5624 to $FC (16bit value)
	sta ptr2
	lda #TXT_A_VASE >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	jmp RESTORE_6X6
	;
L_4F02	lda #TXT_FOUND & 255			; "found"
	sta ptr2
	lda #TXT_FOUND >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #SND_HEAL
	sta VAR_SOUND_REQUESTED
	;
	sed
	lda VAR_HITS
	sec
	sbc #$10				; cold food reduces hits by 10
	bcs B_4F1D
	lda #$00
B_4F1D	sta VAR_HITS
	cld
	;
L_4F21	jsr SUB_REMOVE_2X2_FROM_6X6
	;
	lda #TXT_SOME_COLD_FOOD & 255
	sta ptr2
	lda #TXT_SOME_COLD_FOOD >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	jmp RESTORE_6X6
	;
L_4F32	lda #TXT_FOUND & 255
	sta ptr2
	lda #TXT_FOUND >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #SND_ITEM_PICKUP
	sta VAR_SOUND_REQUESTED
	;
	lda VAR_SCORE_ADD
	clc
	adc #40					; score += 4000
	sta VAR_SCORE_ADD
	;
L_4F4B	jsr SUB_REMOVE_2X2_FROM_6X6
	;
	lda #TXT_A_PENDANT & 255
	sta ptr2
	lda #TXT_A_PENDANT >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	jmp RESTORE_6X6
	;
LAB_AMMO_PICKED_UP
	lda #$00
	sta SAVED_6X6,y				; remove ammo from 6X6
	iny
	sta SAVED_6X6,y
	;
	sed
	lda VAR_AMMO
	clc
	adc #$10				; ammo += 10
	bcc B_4F83
	;
	lda SCROLL_INDEX
	cmp SCROLL_MAX
	bne B_4F81
	;
	lda #TXT_AMMO_WASTED & 255		; can't pick up any more
	sta ptr2
	lda #TXT_AMMO_WASTED >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
B_4F81	lda #$99
B_4F83	sta VAR_AMMO
	cld
	jmp RESTORE_6X6
	;
L_4F8A	lda #TXT_FOUND & 255			; "found"
	sta ptr2
	lda #TXT_FOUND >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #SND_HEAL
	sta VAR_SOUND_REQUESTED
	;
	lda #$00				; first aid reduces hits to zero
	sta VAR_HITS
	;
L_4F9F	tya
	pha
	lda zp06
	sta ptr2
	lda zp06+1
	sta ptr2+1
L_4FA9	tya
	sec
	sbc #$06
	bcc B_4FBE
	tay
	lda ptr2
	clc
	adc #$80
	sta ptr2
	bcc B_4FBB
	inc ptr2+1
B_4FBB	jmp L_4FA9
	;
B_4FBE	tya
	clc
	adc ptr2
	sta ptr2
	bcc B_4FC8
	dec ptr2+1
B_4FC8	lda ptr2
	sec
	sbc #$01
	sta ptr2
	bcs B_4FD3
	dec ptr2+1
B_4FD3	pla
	tay
	;
	jsr SUB_REMOVE_2X2_FROM_6X6
	;
	jsr RESTORE_6X6
	;
	ldy #$00
	tya
	sta (ptr2),y
	ldy #$80
	sta (ptr2),y
	inc ptr2+1
	ldy #$00
	sta (ptr2),y
	iny
	sta (ptr2),y
	iny
	sta (ptr2),y
	;
	lda #TXT_FIRST_AID_PERIOD & 255
	sta ptr2
	lda #TXT_FIRST_AID_PERIOD >> 8
	sta ptr2+1
	jmp SUB_SET_SCROLLTEXT
	;
	; --------
	;
L_4FFC	lda #TXT_FOUND & 255			; "found"
	sta ptr2
	lda #TXT_FOUND >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #SND_ITEM_PICKUP
	sta VAR_SOUND_REQUESTED
	;
	lda #$00
	sta SAVED_6X6,y
	;
	lda #TXT_LIFT_PASS & 255
	sta ptr2
	lda #TXT_LIFT_PASS >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #$01				; flag that player has lift pass
	sta VAR_LIFT_STATE
	;
	jmp RESTORE_6X6
	;
SUB_REMOVE_2X2_FROM_6X6
	lda #$00				; place $00 (space) characters
	sta SAVED_6X6,y				; in a 2x2 area indexed by "Y"
	iny					; (remove picked up item)
	sta SAVED_6X6,y
	iny
	iny
	iny
	iny
	iny
	sta SAVED_6X6,y
	iny
	sta SAVED_6X6,y
	rts
	; --------
L_503A	lda SAVED_6X6+32
	cmp #$54				; check lift chain
	beq try_to_enter_lift
	cmp #$0B				; check lift body - should not be necessary, since player can never walk into it
	beq try_to_enter_lift			; but changed to the new character ($0B)
	bne check_door_vertical
	;
try_to_enter_lift
	lda VAR_PLAYER_DIRECTION		; new code added: check that player is facing down
	cmp #$02				; this was necessary so that the "lifts need pass" cheat doesn't cause an infinite loop
	bne dont_enter_lift
	;
	lda VAR_LIFT_STATE			; have lift pass?
cheat08	beq B_505E				; cheat (bit $00 will remove this check)
	;
	lda #$02				; entering lift
	sta VAR_LIFT_STATE
	;
	lda #TXT_ENTERING_LIFT & 255
	sta ptr2
	lda #TXT_ENTERING_LIFT >> 8
	sta ptr2+1
	jmp SUB_SET_SCROLLTEXT
	; --------
B_505E	lda SCROLL_INDEX
	cmp SCROLL_MAX
	beq B_506A
	sta VAR_LIFT_PASS_MESSAGE_FLAG
	;
dont_enter_lift
	rts
	; --------
B_506A	lda VAR_LIFT_PASS_MESSAGE_FLAG
	beq B_5070
	rts
	; --------
B_5070	lda VAR_PLAYER_CONTROL_DELAY
	bne B_5076
	rts
	; --------
B_5076	inc VAR_LIFT_PASS_MESSAGE_FLAG
	;
	lda #TXT_NEED_LIFT_PASS & 255
	sta ptr2
	lda #TXT_NEED_LIFT_PASS >> 8
	sta ptr2+1
	jmp SUB_SET_SCROLLTEXT
	;
	; --------
	;
check_door_vertical
	nop					; will be disabled in ghost mode
	;
	lda #$44				; door char
	sta VAR_DOOR_MISC_1
	;
	lda SAVED_6X6+17
	cmp #$43				; vertical metal door?
	bne B_509E
	;
	lda #$06
	sta VAR_REMOVE_DOOR_INDEX_1
	lda #$86
	sta VAR_REMOVE_DOOR_INDEX_2
	;
	jmp vertical_door_found
	;
B_509E	lda SAVED_6X6+12
	cmp #$44				; vertical metal door?
	bne B_50B2
	;
	lda #$00
	sta VAR_REMOVE_DOOR_INDEX_1
	lda #$80
	sta VAR_REMOVE_DOOR_INDEX_2
	;
	jmp vertical_door_found
	;
B_50B2	jmp check_door_horizontal
	;
vertical_door_found	
cheat07a
	lda VAR_KEYS				; have key?
	bne B_50E1
	;
L_50BA	lda SCROLL_INDEX
	cmp SCROLL_MAX
	beq B_50C6
	sta VAR_LIFT_PASS_MESSAGE_FLAG
	rts
	; --------
B_50C6	lda VAR_LIFT_PASS_MESSAGE_FLAG
	beq B_50CC
	rts
	; --------
B_50CC	lda VAR_PLAYER_CONTROL_DELAY
	bne B_50D2
	rts
	; --------
B_50D2	inc VAR_LIFT_PASS_MESSAGE_FLAG
	;
	lda #TXT_DOOR_LOCKED & 255
	sta ptr2
	lda #TXT_DOOR_LOCKED >> 8
	sta ptr2+1
	jmp SUB_SET_SCROLLTEXT
	; --------
B_50E1	sed
	lda VAR_KEYS
	beq zero_keys_2
	sec
	sbc #$01				; key removed
	sta VAR_KEYS
zero_keys_2
	cld
	;
	lda #SND_DOOR_OPEN
	sta VAR_SOUND_REQUESTED
	;
L_50F1	ldy VAR_REMOVE_DOOR_INDEX_1
L_50F4	lda (zp06),y
	cmp VAR_DOOR_MISC_1
	bne B_5109
	;
	lda zp06
	sec
	sbc #$80
	sta zp06
	bcs B_5106
	dec zp06+1
B_5106	jmp L_50F4
	;
B_5109	dec zp06
	bne B_510F
	dec zp06+1
B_510F	ldy VAR_REMOVE_DOOR_INDEX_2
	lda #$05
	sta (zp06),y
	iny
	sta (zp06),y
	lda #$00
	inc zp06+1
	ldy VAR_REMOVE_DOOR_INDEX_1
	sta (zp06),y
	iny
	sta (zp06),y
	ldy VAR_REMOVE_DOOR_INDEX_2
	sta (zp06),y
	iny
	sta (zp06),y
	inc zp06+1
	ldy VAR_REMOVE_DOOR_INDEX_1
	sta (zp06),y
	iny
	sta (zp06),y
	ldy VAR_REMOVE_DOOR_INDEX_2
	sta (zp06),y
	iny
	sta (zp06),y
	rts
	; --------
check_door_horizontal
	lda #$45				; door char
	sta VAR_DOOR_MISC_2
	;
	lda SAVED_6X6+32
	cmp #$45				; horizontal door?
	bne B_515E
	;
	inc zp06+1
	inc zp06+1
	lda zp06
	clc
	adc #$80
	sta zp06
	bcc B_515B
	inc zp06+1
B_515B	jmp horizontal_door_found
	;
B_515E	lda SAVED_6X6+2
	cmp #$46				; horizontal door?
	bne B_5173
	lda zp06
	sec
	sbc #$80
	sta zp06
	bcs B_5170
	dec zp06+1
B_5170	jmp horizontal_door_found
	;
B_5173	rts
	; --------
horizontal_door_found
cheat07b
	lda VAR_KEYS				; have key?
	bne B_517C
	;
	jmp L_50BA
	;
B_517C	sed
	lda VAR_KEYS
	beq zero_keys_1
	sec
	sbc #$01				; key removed
	sta VAR_KEYS
zero_keys_1
	cld
	;
	lda #SND_DOOR_OPEN
	sta VAR_SOUND_REQUESTED
	;
L_518C	ldy #$00
L_518E	lda (zp06),y
	cmp VAR_DOOR_MISC_2
	bne B_519E
	dec zp06
	bne B_519B
	dec zp06+1
B_519B	jmp L_518E
	;
B_519E	ldy #$01				; erase door
	lda #$00
	sta (zp06),y
	iny
	sta (zp06),y
	iny
	sta (zp06),y
	iny
	sta (zp06),y
	iny
	lda #$04
	sta (zp06),y
	ldy #$81
	lda #$00
	sta (zp06),y
	iny
	sta (zp06),y
	iny
	sta (zp06),y
	iny
	sta (zp06),y
	iny
	lda #$04
	sta (zp06),y
	rts
	; --------
CALC_PLAYER_POS
	lda #$00				; player position calculation
	sta zp02				; store on zp02 (16 bit)
	sta zp02+1
	;
	lda VAR_PLAYER_SCREEN_TEMP_Y
	sec
	sbc #$3A
	sta zp02
	clc
	asl zp02
	rol zp02+1
	asl zp02
	rol zp02+1
	asl zp02
	rol zp02+1
	asl zp02
	rol zp02+1
	lda VAR_PLAYER_SCREEN_TEMP_X
	sec
	sbc #$20
	clc
	lsr a					; div / 8
	lsr a
	lsr a
	clc
	adc zp02
	sta zp02
	bcc B_51F9
	inc zp02+1
B_51F9	lda zp02
	clc
	adc ptr1
	sta zp02
	bcc B_5204
	inc zp02+1
B_5204	lda zp02+1
	clc
	adc ptr1+1
	sta zp02+1
	rts
	; --------
S_520C	lda VAR_BULLETS_FLAG
	and #$01
	bne B_5216
	jmp L_5251
	;
B_5216	lda VAR_BULLET_X_1
	sta VAR_PLAYER_SCREEN_TEMP_X
	lda VAR_BULLET_Y_1
	sta VAR_PLAYER_SCREEN_TEMP_Y
	;
	jsr CALC_PLAYER_POS
	;
	ldy #$00
	lda (zp02),y
	cmp #$3E				; char code #$3E
	beq L_5251
	cmp #$3F				; char code #$3F
	beq L_5251
	;
	and #$F8
	beq L_5251
	;
	lda VAR_BULLETS_FLAG
	and #$F0
	sta VAR_BULLETS_FLAG
	;
	lda #$00
	sta VAR_BULLET_X_1
	sta VAR_BULLET_Y_1
	sta VAR_BULLET_MISC_1
	lda VAR_BULLET_DIRECTION_1
	sta VAR_BULLET_DIRECTION_TEMP
	;
	jsr S_5295
	;
L_5251	lda VAR_BULLETS_FLAG
	and #$10
	bne B_5259
	rts
	; --------
B_5259	lda VAR_BULLET_X_2
	sta VAR_PLAYER_SCREEN_TEMP_X
	lda VAR_BULLET_Y_2
	sta VAR_PLAYER_SCREEN_TEMP_Y
	;
	jsr CALC_PLAYER_POS
	;
	ldy #$00
	lda (zp02),y
	cmp #$3E
	beq B_5294
	cmp #$3F
	beq B_5294
	and #$F8
	beq B_5294
	;
	lda VAR_BULLETS_FLAG
	and #$0F
	sta VAR_BULLETS_FLAG
	;
	lda #$00
	sta VAR_BULLET_X_2
	sta VAR_BULLET_Y_2
	sta VAR_BULLET_MISC_2
	lda VAR_BULLET_DIRECTION_2
	sta VAR_BULLET_DIRECTION_TEMP
	;
	jsr S_5295
	;
B_5294	rts
	; --------
S_5295	lda (zp02),y
	sec
	sbc #$76
	bcc B_52A3
	sbc #$4B
	bcs B_52A3
	;jmp L_6040
	jmp L_6EA9
	;
B_52A3	lda (zp02),y
	cmp #PRISONER_CHR			; check for prisoner chars
	bcc B_52B0
	cmp #(PRISONER_CHR + $10)
	bcs B_52B0
	;
	jmp L_6E20
	;
B_52B0	cmp #$30				; check for officer sitting down
	bcc B_52BB
	cmp #$3F
	bcs B_52BB
	;
	jmp LAB_OFFICER_SHOT
	;
B_52BB	cmp #$C0				; check for officer facing up
	bcc B_52C6
	cmp #$CC
	bcs B_52C6
	jmp LAB_OFFICER_SHOT
	;
B_52C6	cmp #$FA				; purple chair (blocks shot only)
	bcc B_52D1
	cmp #$FC
	bcs B_52D1
	;
	jmp LAB_OFFICER_SHOT
	;
B_52D1	cmp #$5E
	bcc B_52DC
	cmp #$6E
	bcs B_52DC
	;
	jmp L_5522
	;
B_52DC	cmp #$49
	bne B_52E3
	jmp L_54AF
	;
B_52E3	cmp #$4A
	bne B_52EA
	jmp L_54CE
	;
B_52EA	cmp #$47				; "G"
	bne B_52F1
	jmp L_54ED
	;
B_52F1	cmp #$48				; "H"
	bne B_52F8
	jmp L_5502
	;
B_52F8	cmp #$43				; check for metal door
	beq B_530C
	cmp #$44				; "D"
	beq B_530C
	cmp #$45				; "E"
	beq B_530C
	cmp #$46				; "F"
	beq B_530C
	jmp L_5312
	;
B_530C	ldx #SND_METAL_DOOR
	stx VAR_SOUND_REQUESTED
	rts
	; --------
L_5312	cmp #$40
	bcc B_531D				; < $40
	cmp #$FC
	bcs B_531D
	jmp L_5429
	;
B_531D	jsr SAVE_6X6
	;
	ldx #$00
B_5322	lda V_536E,x
	tay
	lda SAVED_6X6,y
	cmp #$10
	bne B_5330
	jmp L_539E
	;
B_5330	cmp #$12				; ammo character
	bne B_5337
	jmp L_53B9
	;
B_5337	cmp #$14				; painting
	bne B_533E
	jmp L_53CA
	;
B_533E	cmp #$18				; vase
	bne B_5345
	jmp L_5403
	;
B_5345	cmp #$1C				; cold food
	bne B_534C
	jmp L_53DD
	;
B_534C	cmp #$28				; pendant
	bne B_5353
	jmp L_53F0
	;
B_5353	cmp #$20				; first aid
	bne B_535A
	jmp L_5416
	;
B_535A	cmp #$08				; lift pass character
	bne B_5361
	jmp L_549B
	;
B_5361	cmp #$FC				; $FC-$FF: detonator characters
	bne B_5368
	jmp L_5380
	;
B_5368	inx
	cpx #$09
	bne B_5322
	rts
	; --------
V_536E	db $07,$06,$08
	db $00,$01,$02
	db $0C,$0D,$0E
	;
	db $07,$06,$08
	db $0C,$0D,$0E
	db $00,$01,$02
	;
L_5380	lda #TXT_DETONATOR_ACTIVATED & 255
	sta ptr2
	lda #TXT_DETONATOR_ACTIVATED >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	ldx VAR_CURRENT_FLOOR
	lda #$01
	sta VAR_DETONATOR_FLAGS,x		; mark detonator as activated
	;
	lda #SND_ITEM_PICKUP
	sta VAR_SOUND_REQUESTED
	;
	jsr SUB_REMOVE_2X2_FROM_6X6
	jmp RESTORE_6X6
	;
L_539E	lda SCROLL_INDEX
	cmp SCROLL_MAX
	bne B_53B1
	;
	lda #TXT_WASTING_AMMO & 255
	sta ptr2
	lda #TXT_WASTING_AMMO >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
B_53B1	lda #SND_AMMO_ALERT
	sta VAR_SOUND_REQUESTED
	;
	jmp L_4DF1
	;
L_53B9	lda #$00
	sta SAVED_6X6,y
	iny
	sta SAVED_6X6,y
	;
	lda #SND_DESTROYED
	sta VAR_SOUND_REQUESTED
	;
	jmp RESTORE_6X6
	;----
shot_scroll_sound
	lda #SND_DESTROYED
	sta VAR_SOUND_REQUESTED
	;
	lda #TXT_YOU_HAVE_SHOT & 255
	sta ptr2
	lda #TXT_YOU_HAVE_SHOT >> 8
	sta ptr2+1
	jmp SUB_SET_SCROLLTEXT

L_53CA	jsr shot_scroll_sound
	;
	jmp L_4EC7
	;
L_53DD	jsr shot_scroll_sound
	;
	jmp L_4F21
	;
L_53F0	jsr shot_scroll_sound
	;
	jmp L_4F4B
	;
L_5403	jsr shot_scroll_sound
	;
	jmp L_4EF1
	;
L_5416	jsr shot_scroll_sound
	;
	jmp L_4F9F
	;
L_5429	pha
	jsr SAVE_6X6
	pla
	cmp #$4C				; "L"
	bne B_5437
	ldy #$07
	jmp L_5477
	;
B_5437	cmp #$4D				; "M"
	bne B_5440
	ldy #$06
	jmp L_5477
	;
B_5440	cmp #$4E				; "N"
	bne B_5449
	ldy #$01
	jmp L_5477
	;
B_5449	cmp #$4F				; "O"
	bne B_5452
	ldy #$00
	jmp L_5477
	;
B_5452	cmp #$50				; "P"
	bne B_545B
	ldy #$07
	jmp L_5495
	;
B_545B	cmp #$51				; "Q"
	bne B_5464
	ldy #$06
	jmp L_5495
	;
B_5464	cmp #$52				; "R"
	bne B_546D
	ldy #$01
	jmp L_5495
	;
B_546D	cmp #$53				; "S"
	bne B_5476
	ldy #$00
	jmp L_5495
	;
B_5476	rts
	; --------
L_5477	lda #$50				; open barrel chars
	sta SAVED_6X6,y
	iny
	lda #$51				; place open barrel into 6x6
	sta SAVED_6X6,y
	iny
	iny
	iny
	iny
	iny
	lda #$52				; bottom parts
	sta SAVED_6X6,y
	iny
	lda #$53				; 
	sta SAVED_6X6,y
	jmp RESTORE_6X6
	;
L_5495	jsr SUB_REMOVE_2X2_FROM_6X6
	jmp RESTORE_6X6
	;
L_549B	lda SCROLL_INDEX
	cmp SCROLL_MAX
	beq B_54A4
	rts
	; --------
B_54A4	
	lda #TXT_SHOT_LIFT_PASS & 255
	sta ptr2
	lda #TXT_SHOT_LIFT_PASS >> 8
	sta ptr2+1
	jmp SUB_SET_SCROLLTEXT
	;
L_54AF	lda #$01
	sta VAR_REMOVE_DOOR_INDEX_1
	lda #$81
	sta VAR_REMOVE_DOOR_INDEX_2
	;
	lda #$4A
	sta VAR_DOOR_MISC_1
	;
	lda zp02
	sta zp06
	lda zp02+1
	sta zp06+1
	;
	lda #SND_DESTROYED
	sta VAR_SOUND_REQUESTED
	;
	jmp L_50F1
	;
L_54CE	lda #$00
	sta VAR_REMOVE_DOOR_INDEX_1
	lda #$80
	sta VAR_REMOVE_DOOR_INDEX_2
	;
	lda #$4A
	sta VAR_DOOR_MISC_1
	lda zp02
	sta zp06
	lda zp02+1
	sta zp06+1
	;
	lda #SND_DESTROYED
	sta VAR_SOUND_REQUESTED
	;
	jmp L_50F1
	;
L_54ED	lda #$47
	sta VAR_DOOR_MISC_2
	lda zp02
	sta zp06
	lda zp02+1
	sta zp06+1
	;
	lda #SND_DESTROYED
	sta VAR_SOUND_REQUESTED
	;
	jmp L_518C
	;
L_5502	lda #$47
	sta VAR_DOOR_MISC_2
	lda zp02
	sta zp06
	lda zp02+1
	sta zp06+1
	lda zp06
	sec
	sbc #$80
	sta zp06
	bcs B_551A
	dec zp06+1
	;
B_551A	lda #SND_DESTROYED
	sta VAR_SOUND_REQUESTED
	;
	jmp L_518C
	;
L_5522	cmp #$5E
	beq B_5561
	cmp #$5F
	beq B_556F
	cmp #$60
	beq B_557A
	cmp #$61
	beq B_5553
	cmp #$66
	beq B_5561
	cmp #$67
	beq B_556F
	cmp #$68
	beq B_557A
	cmp #$69
	beq B_5553
	cmp #$6A
	beq B_5561
	cmp #$6B
	beq B_556F
	cmp #$6C
	beq B_557A
	cmp #$6D
	beq B_5553
	rts
	; --------
B_5553	lda zp02
	sec
	sbc #$01
	sta zp02
	bcs B_555E
	dec zp02+1
B_555E	jmp B_557A
	;
B_5561	lda zp02
	sec
	sbc #$80
	sta zp02
	bcs B_556C
	dec zp02+1
B_556C	jmp B_557A
	;
B_556F	lda zp02
	sec
	sbc #$81
	sta zp02
	bcs B_557A
	dec zp02+1
B_557A	lda (zp02),y
	cmp #$66				; check for jewels
	bcc B_5587
	cmp #$6A
	bcs B_5587
	jmp LAB_SHOT_JEWELS
	;
B_5587	cmp #$6A				; check for explosives crate (chars $6A-$6D)
	bcc B_5592
	cmp #$6E
	bcs B_5592
cheat09	jmp LAB_SHOT_EXPLOSIVES			; gets overwritten by cheat
	;
B_5592	jsr SUB_GET_RANDOM
	;lda #$FF				; for testing only: all crates have something
	cmp #$C0				; 75% chance of empty
	bcc crate_is_empty
	jsr SUB_GET_RANDOM
	;lda #$FF				; for testing only: always find jewels
	cmp #$80				; 50% chance of jewels/explosives
	bcc found_explosives
	;
	ldx #$66				; found jewels
	jmp LAB_DRAW_CRATE_LOOT
	;
found_explosives
	ldx #$6A				; found explosives
	jmp LAB_DRAW_CRATE_LOOT
	;
crate_is_empty
	lda #TXT_EMPTY & 255
	sta ptr2
	lda #TXT_EMPTY >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
LAB_DRAW_EMPTY_CRATE
	ldx #$62
	;
LAB_DRAW_CRATE_LOOT
	ldy #$80
	txa
	sta (zp02),y
	ldy #$81
	inx
	txa
	sta (zp02),y
	ldy #$00
	inx
	txa
	sta (zp02),y
	ldy #$01
	inx
	txa
	sta (zp02),y
	rts
	; --------
LAB_SHOT_EXPLOSIVES	
	lda #TXT_FOOL_EXPLOSIVES & 255		; "fool! explosives"
	sta ptr2
	lda #TXT_FOOL_EXPLOSIVES >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	jsr LAB_DRAW_EMPTY_CRATE
	;
	lda #SND_EXPLOSION
	sta VAR_SOUND_REQUESTED
	;
	lda #$01
	sta VAR_EXPLOSION_FLASH_INDEX_1
	sta VAR_PAUSE
	;
	rts
	;
LAB_SHOT_JEWELS	
	lda #TXT_YOU_HAVE_SHOT & 255
	sta ptr2
	lda #TXT_YOU_HAVE_SHOT >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #TXT_SOME_JEWELS & 255
	sta ptr2
	lda #TXT_SOME_JEWELS >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	jmp LAB_DRAW_EMPTY_CRATE
	;
	;
	;
SUB_SET_SCROLLTEXT
	tya
	pha
	ldx SCROLL_MAX
	ldy #$00
L_592F	lda (ptr2),y
	cmp #$40
	beq B_593F
	and #$3F
	sta DAT_SCROLLTEXT,x
	inx
	iny
	jmp L_592F
	;
B_593F	lda #$20
	sta DAT_SCROLLTEXT,x
	inx
	stx SCROLL_MAX
	pla
	tay
	rts
	;
	; ---
	;
SUB_UPDATE_PANEL
	lda VAR_PLAYER_SCREEN_X
	sec
	sbc #$20
	lsr a					; div / 8
	lsr a
	lsr a
	clc
	adc ptr3
	sta VAR_PLAYER_CALC_X
	;
	lda VAR_PLAYER_SCREEN_Y
	sec
	sbc #$3A
	lsr a					; div / 8
	lsr a
	lsr a
	clc
	adc ptr3+1
	sta VAR_PLAYER_CALC_Y  
	;
	; ---
	;
	lda VAR_KEYS
	jsr SUB_SEPARATE_NIBBLES
	sty CHAR_MAP + $00C4			; write number of keys on screen
	stx CHAR_MAP + $00C5
	;
	; --
	;
	lda VAR_PAUSE
	bne skip_update_1
	;
	ldx #$67
	lda VAR_AMMO
	cmp #$10
	bcs *+4
	ldx #$42
	stx COLOR_MAP + $0114
	stx COLOR_MAP + $0115
	;
	jsr SUB_SEPARATE_NIBBLES		; write ammo
	sty CHAR_MAP + $0114
	stx CHAR_MAP + $0115
	;
	; ---
	;
	ldx #$67
	lda VAR_HITS
	cmp #$40
	bcc *+4
	ldx #$42
	stx COLOR_MAP + $0164
	stx COLOR_MAP + $0165
	;
	jsr SUB_SEPARATE_NIBBLES
	sty CHAR_MAP + $0164
	stx CHAR_MAP + $0165
	;
	; ---
	;
skip_update_1
	lda VAR_SCORE_01
	jsr SUB_SEPARATE_NIBBLES
	sty CHAR_MAP + $01D7
	stx CHAR_MAP + $01D8
	;
	lda VAR_SCORE_02
	jsr SUB_SEPARATE_NIBBLES
	sty CHAR_MAP + $01D9
	stx CHAR_MAP + $01DA
	;
	lda VAR_SCORE_03
	jsr SUB_SEPARATE_NIBBLES
	sty CHAR_MAP + $01DB
	stx CHAR_MAP + $01DC
	;
	; ---
	;
	;lda VAR_PLAYER_STEP_COUNTER		; extra debug code
	;jsr SUB_SEPARATE_NIBBLES
	;sty CHAR_MAP + $0000 + 13 * 40 + 30
	;stx CHAR_MAP + $0000 + 13 * 40 + 31
	;
	;lda VAR_DOOR_MISC_2
	;jsr SUB_SEPARATE_NIBBLES
	;sty CHAR_MAP + $0000 + 14 * 40 + 30
	;stx CHAR_MAP + $0000 + 14 * 40 + 31
	;
	;lda VAR_TIMER
	;jsr SUB_SEPARATE_NIBBLES
	;sty CHAR_MAP + $0000 + 15 * 40 + 30
	;stx CHAR_MAP + $0000 + 15 * 40 + 31
	;
	rts
	; --------
	;
	; A = byte to separate
	; return: X/Y = separated tens/ones, converted to printable digits
	;
SUB_SEPARATE_NIBBLES
	tay
	and #$0F
	ora #$F0				; faster than clc/adc
	tax					; return low nibble in X
	tya
	lsr a					; orignal AND was unnecessary, lower 4 bits are removed by LSR A below
	lsr a
	lsr a
	lsr a
	ora #$F0				; faster than clc/adc
	tay					; return high nibble in Y
	rts
	;
	; --------
	;
SUB_CHECK_INACTIVE
	inc VAR_TIMER
	lda VAR_TIMER
	cmp #$80
	beq B_5A3C
	rts
	; --------
B_5A3C	lda #$00
	sta VAR_TIMER
	;
	jsr SUB_JOYSTICK_CHECK			; moving in any direction?
	and #$0F
	cmp #$0F
	bne L_5A6D
	;
	lda SCROLL_INDEX
	cmp SCROLL_MAX
	beq B_5A55
	;
	jmp L_5A6D
	;
B_5A55	inc VAR_PLAYER_INACTIVE_COUNTER
	lda VAR_PLAYER_INACTIVE_COUNTER
	cmp #$0E
	beq B_5A62
	jmp L_5A72
	;
B_5A62	
	lda #TXT_COME_ON & 255			; eventually tell the player to move
	sta ptr2
	lda #TXT_COME_ON >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
L_5A6D	lda #$00
	sta VAR_PLAYER_INACTIVE_COUNTER
	;
L_5A72	jsr SUB_JOYSTICK_CHECK			; pressing fire?
	and #$10
	beq L_5A9A
	;
	lda SCROLL_INDEX
	cmp SCROLL_MAX
	beq B_5A84
	jmp L_5A9A
	;
B_5A84	inc VAR_PLAYER_INACTIVE_COUNTER_2
	lda VAR_PLAYER_INACTIVE_COUNTER_2
	cmp #$07
	beq B_5A8F
	rts
	; --------
B_5A8F	
	lda #TXT_LETS_GO & 255			; eventually tell the player to shoot
	sta ptr2
	lda #TXT_LETS_GO >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
L_5A9A	lda #$00
	sta VAR_PLAYER_INACTIVE_COUNTER_2
	rts
	; --------
SUB_UPDATE_SCORE
	lda VAR_SCORE_ADD
	bne B_5AA6
	rts
	; --------
B_5AA6	dec VAR_SCORE_ADD
	sed
	lda VAR_SCORE_02
	clc
	adc #$01
	sta VAR_SCORE_02
	lda VAR_SCORE_01
	adc #$00
	sta VAR_SCORE_01
	cld
	rts
	; --------
floor_text_lo
	db TXT_BASEMENT & 255
	db TXT_GROUND_FLOOR & 255
	db TXT_FIRST_FLOOR & 255
	db TXT_SECOND_FLOOR & 255
floor_text_hi	
	db TXT_BASEMENT >> 8
	db TXT_GROUND_FLOOR >> 8
	db TXT_FIRST_FLOOR >> 8
	db TXT_SECOND_FLOOR >> 8

SUB_SHOW_CURRENT_FLOOR
	lda #TXT_YOU_ARE & 255			; "YOU ARE"
	sta ptr2
	lda #TXT_YOU_ARE >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	ldx VAR_CURRENT_FLOOR			; new code; use lookup table
	lda floor_text_lo,x
	sta ptr2
	lda floor_text_hi,x
	sta ptr2+1
	jmp SUB_SET_SCROLLTEXT
	;
VAR_PLAYER_SAVED_CHARS
	db $00,$00,$00,$00
	db $00,$00,$00,$00
	db $00,$00,$00,$00
	db $00,$00,$00,$00
	;
S_5B1B	lda VAR_PLAYER_SCREEN_X
	sta VAR_PLAYER_SCREEN_TEMP_X
	lda VAR_PLAYER_SCREEN_Y
	sta VAR_PLAYER_SCREEN_TEMP_Y
	;
	jsr CALC_PLAYER_POS
	;
	ldy #$00
B_5B2C	lda (zp02),y				; save bytes under player's position...
	sta VAR_PLAYER_SAVED_CHARS,y
	lda #$55				; and write $55
	sta (zp02),y
	inc zp02+1
	lda (zp02),y
	sta VAR_PLAYER_SAVED_CHARS + 8,y
	lda #$55				; $55
	sta (zp02),y
	tya
	ora #$80
	tay
	lda (zp02),y
	pha
	lda #$55				; $55
	sta (zp02),y
	dec zp02+1
	lda (zp02),y
	pha
	lda #$55				; $55
	sta (zp02),y
	tya
	and #$7F
	tay
	pla
	sta VAR_PLAYER_SAVED_CHARS + 4,y
	pla
	sta VAR_PLAYER_SAVED_CHARS + 12,y
	iny
	cpy #$04
	bne B_5B2C
	;
	lda zp02				; save zp02 to stack
	pha
	lda zp02+1
	pha
	;
move_prisoner_flag
	lda #$00
	cmp #$01
	beq skip_prisoner
	;
	jsr SUB_MOVE_PRISONER
	;
cheat03a
skip_prisoner
	jsr SUB_MOVE_ENEMIES
	;
	pla					; restore zp02 from stack
	sta zp02+1
	pla
	sta zp02
	;
	ldy #$00
B_5B86	lda VAR_PLAYER_SAVED_CHARS,y		; restore
	sta (zp02),y
	inc zp02+1
	lda VAR_PLAYER_SAVED_CHARS + 8,y
	sta (zp02),y
	lda VAR_PLAYER_SAVED_CHARS + 4,y
	pha
	lda VAR_PLAYER_SAVED_CHARS + 12,y
	pha
	tya
	ora #$80
	tay
	pla
	sta (zp02),y
	dec zp02+1
	pla
	sta (zp02),y
	tya
	and #$7F
	tay
	iny
	cpy #$04
	bne B_5B86
	;
	rts
	; --------
SUB_SCREEN_COLOR_FLASH
	lda VAR_EXPLOSION_FLASH_INDEX_1
	;lda #$01				; for testing
	cmp #$FF
	beq B_5BD6
	;
	cmp #$00
	bne B_5BDD
	;
	lda VAR_HITS
	cmp #$50				; maximum allowed hits
	bcs B_5BD7
	;
B_5BD6	rts
	; --------
B_5BD7	lda #$FF
	sta VAR_EXPLOSION_FLASH_INDEX_1
	rts
	; --------
B_5BDD	ldx VAR_EXPLOSION_FLASH_INDEX_1
	dex
	;
explosion_sync
	lda $FF1D
ntsc4	cmp #$FF
	bcc explosion_sync
	;
	lda EXPLOSION_FLASH_COLORS,x
	sta $FF15
	sta $FF19
	cpx #$10
	beq B_5BF2
	inc VAR_EXPLOSION_FLASH_INDEX_1
	rts
	; --------
B_5BF2	inc VAR_EXPLOSION_FLASH_INDEX_2
	lda VAR_EXPLOSION_FLASH_INDEX_2
	cmp #$05
	beq B_5BD7
	lda #$01
	sta VAR_EXPLOSION_FLASH_INDEX_1
	rts
	;
	; --------
	;
EXPLOSION_FLASH_COLORS
	db $32,$67,$32,$67,$32,0,$31,$41,$51,$71,$51,$41,$31,$31,$32,$32,$00
	;
SAVE_6X6_PLAYER
	lda VAR_PLAYER_SCREEN_X
	sta VAR_PLAYER_SCREEN_TEMP_X
	lda VAR_PLAYER_SCREEN_Y
	sta VAR_PLAYER_SCREEN_TEMP_Y
	;
	jsr CALC_PLAYER_POS
	;
SAVE_6X6
	lda zp02+1				; x-=1, y-=1
	sta zp06+1				; (move one step up/left)
	lda zp02
	sec
	sbc #$81
	sta zp06
	bcs *+4
	dec zp06+1
	;
	lda zp06				; save current zp06 on stack
	pha					; (since the loop below will change it)
	lda zp06+1
	pha
	;
	ldx #$00
	ldy #$00
SAVE_6X6_LOOP
	lda (zp06),y				; save the 6x6 area under the player
	sta SAVED_6X6,x
	inc zp06+1
	lda (zp06),y
	sta SAVED_6X6+12,x
	inc zp06+1
	lda (zp06),y
	sta SAVED_6X6+24,x
	;
	tya
	ora #$80
	tay
	;
	lda (zp06),y
	sta SAVED_6X6+30,x
	dec zp06+1
	lda (zp06),y
	sta SAVED_6X6+18,x
	dec zp06+1
	lda (zp06),y
	sta SAVED_6X6+6,x
	;
	tya
	and #$7F
	tay
	;
	iny
	inx
	cpx #$06
	bne SAVE_6X6_LOOP
	;
	pla					; restore zp06 from stack
	sta zp06+1
	pla
	sta zp06
	;
	rts
	;
	; --------
	;
TAB_JEWEL_POS_MAP
	; indexes within the 6x6 area
	db 1,2,3,4				; top (going right)
	db 11,17,23,29				; right (going down)
	db 34,33,32,31				; bottom (going left)
	db 24,18,12,06				; left (going up)
	;
SUB_JEWELS_CHECK
	jsr SUB_JOYSTICK_CHECK
	tax					; save joy state
	and #$01
	bne jc1
	;
	lda SAVED_6X6+2
	cmp #$66				; bottom chars
	beq jewels_touched
	cmp #$67
	beq jewels_touched
	;
	lda SAVED_6X6+3
	cmp #$66
	beq jewels_touched
	cmp #$67
	beq jewels_touched
	;
	rts
	; --------
jc1	txa
	and #$02
	bne jc2
	;
	lda SAVED_6X6+32
	cmp #$68				; top chars
	beq jewels_touched
	cmp #$69
	beq jewels_touched
	;
	lda SAVED_6X6+33
	cmp #$68
	beq jewels_touched
	cmp #$69
	beq jewels_touched
	rts
	; --------
jc2	txa
	and #$04
	bne jc3
	;
	lda SAVED_6X6+12
	cmp #$69				; right side chars
	beq jewels_touched
	cmp #$67
	beq jewels_touched
	;
	lda SAVED_6X6+18
	cmp #$69
	beq jewels_touched
	cmp #$67
	beq jewels_touched
	rts
	; --------
jc3	txa
	and #$08
	bne jc4
	;
	lda SAVED_6X6+17
	cmp #$68				; left side chars
	beq jewels_touched
	cmp #$66
	beq jewels_touched
	;
	lda SAVED_6X6+23
	cmp #$68
	beq jewels_touched
	cmp #$66
	beq jewels_touched
jc4	rts
	; --------
jewels_touched
	lda zp06
	sta zp02
	lda zp06+1
	sta zp02+1
	;
	; scan the 6x6 area
	;
	ldx #$0F
jt_loop	lda TAB_JEWEL_POS_MAP,x			; check for jewels
	tay
	lda SAVED_6X6,y
	cmp #$68				; jewels chars; $68: top/left
	beq jt_calc_pos
	cmp #$69				; top/right
	beq jt_top_right
	cmp #$66				; bottom/left
	beq jt_bottom_left
	cmp #$67				; bottom/right
	beq jt_bottom_right
	dex
	bpl jt_loop
	rts
	; --------
jt_top_right
	;inc $FF15
	lda zp02				; x=x-1
	sec
	sbc #$01
	sta zp02
	bcs *+4
	dec zp02+1
	jmp jt_calc_pos
	;
jt_bottom_left
	lda zp02				; y=y-1
	sec
	sbc #$80
	sta zp02
	bcs *+4
	dec zp02+1
	jmp jt_calc_pos
	;
jt_bottom_right
	lda zp02				; x=x-1, y=y-1
	sec
	sbc #$81
	sta zp02
	bcs *+4
	dec zp02+1
	;jmp jt_calc_pos
	;
jt_calc_pos
	tya					; for every 6 (in the 6x6 area)
	beq jt_add_x
	sec
	sbc #$06
	bcc jt_add_x
	tay
	lda zp02
	clc
	adc #$80				; subtract $80 - map width (move up one unit in the map)
	sta zp02
	bcc *+4
	inc zp02+1
	jmp jt_calc_pos
	;
jt_add_x
	tya					; add remainder as X
	clc
	adc zp02
	sta zp02
	bcc *+4
	inc zp02+1
	;
	lda VAR_SCORE_ADD
	clc
	adc #40					; score += 4000
	sta VAR_SCORE_ADD
	;
	lda #TXT_FOUND & 255
	sta ptr2
	lda #TXT_FOUND >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #TXT_SOME_JEWELS & 255
	sta ptr2
	lda #TXT_SOME_JEWELS >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #SND_ITEM_PICKUP
	sta VAR_SOUND_REQUESTED
	;
	;inc $FF19
;	lda #$00				; new code: erase from saved 6x6
;	sta SAVED_6X6+2				; to avoid "picking up jewels twice" bug
;	sta SAVED_6X6+3
;	sta SAVED_6X6+32
;	sta SAVED_6X6+33
;	sta SAVED_6X6+12
;	sta SAVED_6X6+18
;	sta SAVED_6X6+17
;	sta SAVED_6X6+23
	;
	jsr LAB_DRAW_EMPTY_CRATE
	;jsr SUB_UPDATE_SCREEN
	;jmp *
	rts
	;
	; ---
	;
SUB_INIT_PANEL
	lda #$45				; set up destination pointers
	sta panel_chr_dst + 1
	sta panel_col_dst + 1
	lda #CHAR_MAP >> 8
	sta panel_chr_dst + 2
	lda #COLOR_MAP >> 8
	sta panel_col_dst + 2
	;
	; ---
	;
	ldy #$15
panel_color_loop_1
	ldx #$00
	;
	lda #$09				; side character
panel_chr_dst
panel_color_loop_2
	sta $FFFF,x
	lda #$67				; panel color
panel_col_dst
	sta $FFFF,x
	;
	lda #$0B				; panel body character
	inx
	cpx #$0A
	bne panel_color_loop_2
	;
	lda panel_chr_dst + 1
	clc
	adc #$28				; +40
	sta panel_chr_dst + 1
	sta panel_col_dst + 1
	bcc pcl1
	inc panel_chr_dst + 2
	inc panel_col_dst + 2
pcl1	dey
	bne panel_color_loop_1
	;
	; ---
	;
	ldx #$0C
	stx CHAR_MAP + $0045			; corners
	inx
	stx CHAR_MAP + $004E
	inx
	stx CHAR_MAP + $0365
	inx
	stx CHAR_MAP + $036E
	;
	; ---
	;
	ldx #$07
	lda #$0A
pcl6	sta CHAR_MAP + $0366,x			; bottom shadow
	dex
	bpl pcl6
	;
	ldx #$2D
	stx CHAR_MAP + $00C1
	inx
	stx CHAR_MAP + $00BE			; write "KEYS"
	inx
	stx CHAR_MAP + $00BF
	lda #$74
	sta CHAR_MAP + $00C0
	;
	lda #$01
	sta CHAR_MAP + $010E
	ldx #$24
	stx CHAR_MAP + $010F
	stx CHAR_MAP + $0110
	inx
	stx CHAR_MAP + $0111
	inx
	stx CHAR_MAP + $015E
	inx
	stx CHAR_MAP + $015F
	ldx #$2C
	stx CHAR_MAP + $0160
	inx
	stx CHAR_MAP + $0161
	;
	ldx #$05
	lda #$0A				; "shadow"
pcl7	sta CHAR_MAP + $01AF,x			; above score
	dex
	bpl pcl7
	;
	sta CHAR_MAP + $009C
	sta CHAR_MAP + $009D
	;
	sta CHAR_MAP + $00EC
	sta CHAR_MAP + $00ED
	;
	sta CHAR_MAP + $013C
	sta CHAR_MAP + $013D
	;
	lda #$75
	sta CHAR_MAP + $009E			; shadow corners
	sta CHAR_MAP + $00EE
	sta CHAR_MAP + $013E
	sta CHAR_MAP + $01B5
	;
	lda #$09				; shadow sides
	sta CHAR_MAP + $00C6
	sta CHAR_MAP + $0116
	sta CHAR_MAP + $0166
	sta CHAR_MAP + $01DD
	;
	; --- draw the castle
	;
	ldx #$03
pcl2	lda panel_castle,x
	sta CHAR_MAP + 15 * 40 + 32,x
	lda panel_castle + 4,x
	sta CHAR_MAP + 16 * 40 + 32,x
	lda panel_castle + 8,x
	sta CHAR_MAP + 17 * 40 + 32,x
	lda panel_castle + 12,x
	sta CHAR_MAP + 18 * 40 + 30,x
	lda panel_castle + 16,x
	sta CHAR_MAP + 18 * 40 + 34,x
	dex
	bpl pcl2
	;
	; --- if cheats are enabled, color score differently
	;
	ldx #11
pcl4	lda cheat_flags,x
	bne pcl3
	dex
	bpl pcl4
	;
	rts
	;
pcl3	ldx #$05 
	lda #$21				; dark color for score (cheats enabled)
pcl5	sta COLOR_MAP + 11 * 40 + 31,x
	dex
	bpl pcl5
	;
	rts

panel_castle
	db $0B,$5C,$0B,$0B
	db $5C,$5D,$0B,$5C
	db $5D,$06,$B4,$5D
	db $0A,$B4,$40,$91
	db $40,$40,$B4,$0A
	;
	; --------
	;
SUB_SET_PLAYER_STARTING_POS
	lda VAR_PLAYER_INIT_FLAG		; player initial position
	bne player_at_castle_entrance
	;
	lda VAR_RESTORE_FLAG
	beq player_at_castle_entrance
	;
	lda #(GAME_MAP + 107 * $80 + 0) & 255	; top/left corner pos (elevator)
	sta ptr1
	lda #(GAME_MAP + 107 * $80 + 0) >> 8
	sta ptr1+1
	;
	lda #$00				; screen offset
	sta ptr3
	lda #$6B
	sta ptr3+1
	;
	lda #$A0				; player x/y
	sta VAR_PLAYER_SCREEN_X
	lda #$72
	sta VAR_PLAYER_SCREEN_Y
	;
	rts
	; --------
player_at_castle_entrance
	lda #(GAME_MAP + 107 * $80 + 56) & 255	; top/left corner pos (level 1 entrance)
	sta ptr1
	lda #(GAME_MAP + 107 * $80 + 56) >> 8
	sta ptr1+1
	;
	lda #56					; screen offset
	sta ptr3
	lda #107
	sta ptr3+1
	;
	lda #$80				; player x/y
	sta VAR_PLAYER_SCREEN_X
	lda #$A2
	sta VAR_PLAYER_SCREEN_Y
	;
	lda #$00
	sta VAR_PLAYER_INIT_FLAG		; clear init flag
	;
	rts
	;
	; --------
	;
SUB_CASTLE_EXIT_CHECK
	lda SAVED_6X6+32
	cmp #$55				; "laser door" char (changed from $E4 to $55)
	beq B_5F32
	sta VAR_DOOR_TEMP
	rts
	; --------
B_5F32	lda VAR_DOOR_TEMP
	cmp #$55				; "laser door" char (changed from $E4 to $55)
	bne B_5F3A
	rts
	; --------
B_5F3A	lda #$55				; "laser door" char (changed from $E4 to $55)
	sta VAR_DOOR_TEMP
	;
	lda VAR_PRISONER_FLOOR
	cmp #$FF
	beq B_5F75
	;
	cmp VAR_CURRENT_FLOOR
	beq B_5F5F
	;
mission_not_complete
	lda SCROLL_INDEX
	cmp SCROLL_MAX
	bne B_5F5E
	;
	lda #TXT_MISSION_NOT_COMPLETE & 255
	sta ptr2
	lda #TXT_MISSION_NOT_COMPLETE >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
B_5F5E	rts
	;
	; --------
	;
B_5F5F	lda VAR_TEMP_FLAG
	bne mission_not_complete
	lda #TXT_PRISONER_RESCUED & 255
	sta ptr2
	lda #TXT_PRISONER_RESCUED >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #$01				; win (prisoner rescued)
	sta VAR_WIN_STATE
	;
	rts
	; --------
B_5F75	lda VAR_DETONATOR_FLAGS
	beq mission_not_complete
	lda VAR_DETONATOR_FLAGS + 1
	beq mission_not_complete
	lda VAR_DETONATOR_FLAGS + 2
	beq mission_not_complete
	lda VAR_DETONATOR_FLAGS + 3
	beq mission_not_complete
	;
	lda #TXT_CASTLE_DESTROYED & 255
	sta ptr2
	lda #TXT_CASTLE_DESTROYED >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #$02				; win (castle destroyed)
	sta VAR_WIN_STATE
	;
	rts
	;
	; --------
	;
SUB_REMOVE_DETONATOR_IF_NEEDED
	lda VAR_MISSION				; if NOT mission 1 (blow up castle)
	cmp #$04				; then remove detonator from map
	bne rem1
	;
	ldx VAR_CURRENT_FLOOR
	lda VAR_DETONATOR_FLAGS,x		; check if detonator has been activated
	bne rem1				; if yes, then remove it
	rts
	; --------
rem1	lda VAR_DETONATOR_LO
	sta ptr2
	lda VAR_DETONATOR_HI
	sta ptr2+1
	;
	ldy #$81
	lda #$00				; space character
	sta (ptr2),y				; remove 2x2 area from map
	iny
	sta (ptr2),y
	inc ptr2+1
	ldy #$01
	sta (ptr2),y
	iny
	sta (ptr2),y
	rts
	;
	; --------
	;
SUB_PAUSE_CHECK
	lda #$7F				; check for Run/Stop (changed from space)
	sta $FD30
	lda #$FF
	sta $FF08
	lda $FF08
	and #$80
	eor #$80
	sta KEYBOARD
	rts
	; --------
SUB_PAUSE
	ldx #$FF
pau1	ldy #$FF
pau2	lda #$02
pau3	sec
	sbc #$01
	bne pau3
	dey
	bne pau2
	dex
	bne pau1
	rts
	;
	; ---
	;
SUB_MOVE_ENEMIES
	lda VAR_ENEMY_DELAY
cheat03b
	cmp #$03				; enemy speed, 3=default
	beq B_6054
	;
	inc VAR_ENEMY_DELAY
	;
	lda VAR_ENEMY_INDEX
	bne B_605C
	;
	rts
	; --------
B_6054	lda #$00
	sta VAR_ENEMY_INDEX
	sta VAR_ENEMY_DELAY
	;
B_605C	lda #$00
	sta VAR_RESURRECT_ENEMY_COUNT
	;
	ldx VAR_ENEMY_INDEX
B_6064	lda ENEMY_X,x
	clc
	adc #$0A
	sec
	sbc ptr3
	bcc B_60B3
	;
	sbc #$30
	bcs B_60B3
	;
	lda ENEMY_Y,x
	clc
	adc #$0A
	sec
	sbc ptr3+1
	bcc B_60B3
	;
	sbc #$28
	bcs B_60B3
	;
	jsr S_6129
	;
L_6085	lda VAR_RESURRECT_ENEMY_COUNT
	cmp #$08
	bne L_609B
	;
	inx
B_608D	stx VAR_ENEMY_INDEX
	cpx VAR_ENEMY_COUNT
	bne B_609A
	;
	lda #$00
	sta VAR_ENEMY_INDEX
B_609A	rts
	; --------
L_609B	inx
	cpx #$40
	beq B_608D
	cpx #$80
	beq B_608D
	cmp #$C0
	beq B_608D
	cpx VAR_ENEMY_COUNT
	bne B_6064
	lda #$00
	sta VAR_ENEMY_INDEX
	rts
	; --------
B_60B3	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$20				; check if enemy is dead
	bne B_60BD
	;
cant_place_enemy
	jmp L_609B
	;
B_60BD	jsr SUB_GET_RANDOM			; about 6% chance of enemy resurrecting ("wandering enemy")
	cmp #$F0
cheat02	nop					; with cheat, this will be a CLC
	bcc L_609B
	;
	lda ENEMY_LO,x
	sta ptr2
	lda ENEMY_HI,x
	sta ptr2+1
	;
	ldy #$03
B_60D3	lda (ptr2),y				; check if we're free to place an enemy here (line 1)
	bne cant_place_enemy
	dey
	bpl B_60D3
	;
	inc ptr2+1
	;
	ldy #$03
B_60DE	lda (ptr2),y				; check line 2
	bne cant_place_enemy
	dey
	bpl B_60DE
	;
	lda ptr2
	clc
	adc #$80
	sta ptr2
	bne B_60F0
	inc ptr2+1
	;
B_60F0	ldy #$03
B_60F2	lda (ptr2),y				; check line 3
	bne cant_place_enemy
	dey
	bpl B_60F2
	;
	dec ptr2+1
	;
	ldy #$03
B_60FD	lda (ptr2),y				; check line 4
	bne cant_place_enemy
	dey
	bpl B_60FD
	;
	; ---
	;
	lda ENEMY_DIRECTION_AND_FLAGS,x		; all good!
	and #$1F				; clear "shot" and "dead" flags
	sta ENEMY_DIRECTION_AND_FLAGS,x
	;
	lda ENEMY_LO,x
	sta ptr2
	sta zp08
	lda ENEMY_HI,x
	sta ptr2+1
	sta zp08+1
	;
	jsr LAB_UPDATE_ENEMY_SOLDIER
	;
	inc VAR_RESURRECT_ENEMY_COUNT
	;
	jmp L_6085
	;
S_6129	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$20
	beq B_6131
	rts
	; --------
B_6131	inc VAR_RESURRECT_ENEMY_COUNT
	;
	jsr S_630A
	;
	lda ENEMY_LO,x
	sta ptr2
	lda ENEMY_HI,x
	sta ptr2+1
	;
	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$07
	cmp #$01
	bne B_614D
	;
	jmp L_615E
	;
B_614D	cmp #$02
	bne B_6154
	;
	jmp L_61A2
	;
B_6154	cmp #$03
	bne B_615B
	;
	jmp L_61EB
	;
B_615B	jmp L_6237
	;
L_615E	dec ptr2+1
	;
	ldy #$80				; check from $80-$84
B_6162	lda (ptr2),y
	bne B_6192				; no space
	iny
	cpy #$84
	bne B_6162
	;
	inc ptr2+1
	lda ptr2
	sec
	sbc #$80
	sta ptr2
	bcs B_6178
	dec ptr2+1
B_6178	;
	dec ENEMY_Y,x				; enemy moves up
	;
	lda ENEMY_LO,x
	sta zp08
	lda ENEMY_HI,x
	sta zp08+1
	lda ptr2
	sta ENEMY_LO,x
	lda ptr2+1
	sta ENEMY_HI,x
	;
	jmp LAB_UPDATE_ENEMY_SOLDIER
	;
B_6192	cmp #$55				; check if player ($55 is special marker)
	bne B_6199
	;
	jsr SUB_PLAYER_IS_HIT
	;
B_6199	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$F7
	sta ENEMY_DIRECTION_AND_FLAGS,x
	rts
	; --------
L_61A2	inc ptr2+1
	inc ptr2+1
	;
	ldy #$00				; check from $00-$04
B_61A8	lda (ptr2),y
	bne B_61DB				; no space
	iny
	cpy #$04
	bne B_61A8
	;
	lda ENEMY_HI,x
	sta ptr2+1
	lda ptr2
	clc
	adc #$80
	sta ptr2
	bcc B_61C1
	inc ptr2+1
B_61C1	;
	inc ENEMY_Y,x				; enemy moves down
	;
	lda ENEMY_LO,x
	sta zp08
	lda ENEMY_HI,x
	sta zp08+1
	lda ptr2
	sta ENEMY_LO,x
	lda ptr2+1
	sta ENEMY_HI,x
	jmp LAB_UPDATE_ENEMY_SOLDIER
	;
B_61DB	cmp #$55				; check if player ($55 is special marker)
	bne B_61E2
	;
	jsr SUB_PLAYER_IS_HIT
	;
B_61E2	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$F7
	sta ENEMY_DIRECTION_AND_FLAGS,x
	rts
	; --------
L_61EB	dec ptr2
	bne B_61F1
	dec ptr2+1
B_61F1	ldy #$00
	lda (ptr2),y
	bne B_6227
	inc ptr2+1
	lda (ptr2),y
	bne B_6227
	dec ptr2+1
	ldy #$80
	lda (ptr2),y
	bne B_6227
	inc ptr2+1
	lda (ptr2),y
	bne B_6227
	;
	dec ptr2+1
	lda ENEMY_LO,x
	sta zp08
	lda ENEMY_HI,x
	sta zp08+1
	dec ENEMY_X,x
	lda ptr2
	sta ENEMY_LO,x
	lda ptr2+1
	sta ENEMY_HI,x
	jmp LAB_UPDATE_ENEMY_SOLDIER
	;
B_6227	cmp #$55				; check if player ($55 is special marker)
	bne B_622E
	;
	jsr SUB_PLAYER_IS_HIT
	;
B_622E	lda ENEMY_DIRECTION_AND_FLAGS,x
	ora #$08
	sta ENEMY_DIRECTION_AND_FLAGS,x
	rts
	; --------
L_6237	ldy #$04
	lda (ptr2),y
	bne B_6273
	inc ptr2+1
	lda (ptr2),y
	bne B_6273
	dec ptr2+1
	ldy #$84
	lda (ptr2),y
	bne B_6273
	inc ptr2+1
	lda (ptr2),y
	bne B_6273
	;
	dec ptr2+1
	inc ENEMY_X,x				; enemy moves right
	lda ENEMY_LO,x
	sta zp08
	lda ENEMY_HI,x
	sta zp08+1
	inc ptr2				; Increment 16bit value
	bne B_6266
	inc ptr2+1
B_6266	;
	lda ptr2
	sta ENEMY_LO,x
	lda ptr2+1
	sta ENEMY_HI,x
	;
	jmp LAB_UPDATE_ENEMY_SOLDIER
	;
B_6273	cmp #$55				; check if player ($55 is special marker)
	bne B_627A
	;
	jsr SUB_PLAYER_IS_HIT
	;
B_627A	lda ENEMY_DIRECTION_AND_FLAGS,x
	ora #$08
	sta ENEMY_DIRECTION_AND_FLAGS,x
	rts
	; --------

enemy_tile_lo
	db 0
	db (TILE_CHARS + $340) & 255
	db (TILE_CHARS + $300) & 255
	db (TILE_CHARS + $360) & 255
	db (TILE_CHARS + $320) & 255

enemy_tile_hi
	db 0
	db (TILE_CHARS + $340) >> 8
	db (TILE_CHARS + $300) >> 8
	db (TILE_CHARS + $360) >> 8
	db (TILE_CHARS + $320) >> 8

LAB_UPDATE_ENEMY_SOLDIER	;
	;jsr SUB_GET_ENEMY_TILE
;SUB_GET_ENEMY_TILE
	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$07
	tay
	;
;	cmp #$01				; 1 -> 7340
;	beq B_6383
;	cmp #$02				; 2 -> 7300
;	beq B_638C
;	cmp #$03				; 3 -> 7360
;	beq B_6395
;	;
;	lda #$20				; 4 -> 7320
;	sta zp0A
;	lda #$73
;	sta zp0A+1
;	rts
;;$6383  --------
;B_6383	lda #$40				; $7340 to $0A (16bit value)
;	sta zp0A
;	lda #$73
;	sta zp0A+1
;	rts
;;$638C  --------
;B_638C	lda #$00				; $7300 to $0A (16bit value)
;	sta zp0A
;	lda #$73
;	sta zp0A+1
;	rts
;;$6395  --------
;B_6395	lda #$60				; $7360 to $0A (16bit value)
;	sta zp0A
;	lda #$73
;	sta zp0A+1
;	;rts
	;
	lda ENEMY_DIRECTION_AND_FLAGS,x
	eor #$10				; enemy step
	sta ENEMY_DIRECTION_AND_FLAGS,x
	and #$10				; $10 is added to tile pointer
	;
	clc
	adc enemy_tile_lo,y
	sta zp0A
	lda enemy_tile_hi,y
	sta zp0A+1
;	beq B_6299
;	;
;	lda zp0A
;	clc
;	adc #$10
;	sta zp0A
;	;
;B_6299	
	;
	ldy #$00
B_629B	lda #$00				; erase enemy soldiers
	sta (zp08),y
	inc zp08+1
	sta (zp08),y
	tya
	ora #$80
	tay
	lda #$00				; erase enemy soldiers
	sta (zp08),y
	dec zp08+1
	sta (zp08),y
	tya
	and #$7F
	tay
	iny
	cpy #$04
	bne B_629B
	;
	lda zp0A+1
	sta ENEMY_SRC_PTR_1 + 2
	sta ENEMY_SRC_PTR_4 + 2
	sta ENEMY_SRC_PTR_2 + 2
	sta ENEMY_SRC_PTR_3 + 2
	clc
	lda zp0A
	sta ENEMY_SRC_PTR_1 + 1
	adc #$04
	sta ENEMY_SRC_PTR_4 + 1
	adc #$04
	sta ENEMY_SRC_PTR_2 + 1
	adc #$04
	sta ENEMY_SRC_PTR_3 + 1
	;
	txa
	pha
	ldy #$00
	ldx #$00
	;
draw_enemy_loop
ENEMY_SRC_PTR_1
	lda $FFFF,x				; draw enemy soldiers
	sta (ptr2),y
	;
	inc ptr2+1
ENEMY_SRC_PTR_2
	lda $FFFF,x
	sta (ptr2),y
	;
	tya
	ora #$80
	tay
ENEMY_SRC_PTR_3
	lda $FFFF,x
	sta (ptr2),y
	;
	dec ptr2+1
ENEMY_SRC_PTR_4
	lda $FFFF,x
	sta (ptr2),y
	tya
	and #$7F
	tay
	inx
	iny
	cpy #$04
	bne draw_enemy_loop
	pla
	tax
	rts
	; --------
S_630A	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$08
	bne L_6332
L_6311	lda ENEMY_X,x
	sec
	sbc VAR_PLAYER_CALC_X
	beq B_6353
	bcs B_6327
	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$F8
	ora #$04
	sta ENEMY_DIRECTION_AND_FLAGS,x
	rts
	; --------
B_6327	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$F8
	ora #$03
	sta ENEMY_DIRECTION_AND_FLAGS,x
	rts
	; --------
L_6332	lda ENEMY_Y,x
	sec
	sbc VAR_PLAYER_CALC_Y
	beq B_635E
	bcs B_6348
	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$F8
	ora #$02
	sta ENEMY_DIRECTION_AND_FLAGS,x
	rts
	; --------
B_6348	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$F8
	ora #$01
	sta ENEMY_DIRECTION_AND_FLAGS,x
	rts
	; --------
B_6353	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$F7
	sta ENEMY_DIRECTION_AND_FLAGS,x
	jmp L_6332
	;
B_635E	lda ENEMY_DIRECTION_AND_FLAGS,x
	ora #$08
	sta ENEMY_DIRECTION_AND_FLAGS,x
	jmp L_6311
	;
	; ---
	;
SUB_PLAYER_IS_HIT
	lda VAR_HITS
	cmp #$99
	beq ret3
	;
	sed
	lda VAR_HITS
	clc
cheat04	adc #$01				; *** no hits cheat
	sta VAR_HITS
	cld
	;
ret3	rts
	;
	; --------
	;
SUB_GET_RANDOM
	inc VAR_RANDOM_INDEX
	ldy VAR_RANDOM_INDEX
	lda DAT_RANDOM,y
	rts
	; --------
LAB_OFFICER_SHOT
	ldx zp02
	stx ptr2
	ldx zp02+1
	stx ptr2+1
	;
	cmp #$38
	bcc B_6817
	cmp #$3C
	bcs B_6817
	;
	lda ptr2
	clc
	adc #$80
	sta ptr2
	bcc B_6817
	inc ptr2+1
	;
B_6817	ldy #$00
	lda (ptr2),y
	cmp #$34
	beq B_6855
	cmp #$30
	beq B_682E
	cmp #$31
	beq B_683C
	cmp #$35
	beq B_684A
	jmp L_6888
	;
B_682E	lda ptr2
	sec
	sbc #$01
	sta ptr2
	bcs B_6839
	dec ptr2+1
B_6839	jmp B_6855
	;
B_683C	lda ptr2
	sec
	sbc #$02
	sta ptr2
	bcs B_6847
	dec ptr2+1
B_6847	jmp B_6855
	;
B_684A	lda ptr2
	sec
	sbc #$03
	sta ptr2
	bcs B_6855
	dec ptr2+1
	;
B_6855	ldy #$00
	lda #$03
	sta (ptr2),y
	lda #$FA;$E7				; CHANGED purple chair (left)
	iny
	sta (ptr2),y
	lda #$FB;$E8				; CHANGED purple chair (right)
	iny
	sta (ptr2),y
	lda #$00
	iny
	sta (ptr2),y
	ldy #$80
	lda #$4B				; table
	sta (ptr2),y				; 4x
	iny
	sta (ptr2),y
	iny
	sta (ptr2),y
	iny
	sta (ptr2),y
	;
	lda VAR_SCORE_ADD
	clc
	adc #25					; score += 2500
	sta VAR_SCORE_ADD
	;
	lda #SND_HIT
	sta VAR_SOUND_REQUESTED
	;
	rts
	; --------
L_6888	cmp #$C8
	bcc B_6895
	cmp #$CC
	bcs B_6895
	dec ptr2+1
	jmp L_68A0
	;
B_6895	lda ptr2
	sec
	sbc #$80
	sta ptr2
	bcs L_68A0
	dec ptr2+1
L_68A0	ldy #$80
	lda (ptr2),y
	cmp #$C2
	beq L_68DF
	cmp #$C0
	beq B_68B5
	cmp #$C3
	beq B_68C3
	cmp #$C1
	beq B_68D1
	rts
	; --------
B_68B5	lda ptr2
	sec
	sbc #$01
	sta ptr2
	bcs B_68C0
	dec ptr2+1
B_68C0	jmp L_68DF
	;
B_68C3	lda ptr2
	sec
	sbc #$02
	sta ptr2
	bcs B_68CE
	dec ptr2+1
B_68CE	jmp L_68DF
	;
B_68D1	lda ptr2
	sec
	sbc #$03
	sta ptr2
	bcs B_68DC
	dec ptr2+1
B_68DC	jmp L_68DF
	;
L_68DF	ldy #$00
	lda #$4B				; table char
	sta (ptr2),y				; 4x
	iny
	sta (ptr2),y
	iny
	sta (ptr2),y
	iny
	sta (ptr2),y
	ldy #$80
	lda #$06
	sta (ptr2),y
	iny
	lda #$3C				; purple chair
	sta (ptr2),y
	iny
	lda #$3D				; purple chair
	sta (ptr2),y
	iny
	lda #$05
	sta (ptr2),y
	;
	lda VAR_SCORE_ADD
	clc
	adc #25					; score += 2500
	sta VAR_SCORE_ADD
	;
	lda #SND_HIT
	sta VAR_SOUND_REQUESTED
	;
	rts
	; --------
SUB_SCREEN_TRANSITION_OUT
	lda #$0B
	sta VAR_TRANSITION_COUNT
	;
transition_loop
	jsr SUB_SCREEN_TRANSITION_COPY		; now synced to frame (no delay needed)
	;
	dec VAR_TRANSITION_COUNT
	bne transition_loop
	;
	rts
	; --------
SUB_SCREEN_TRANSITION_COPY
	lda #$CB
	cmp $FF1D
	bne *-3
	;
	ldx #$02
	ldy #$00
sstc1	iny
	lda CHAR_MAP + $0050,x
	sta CHAR_MAP + $0028,y
	lda COLOR_MAP + $0050,x
	sta COLOR_MAP + $0028,y
	lda CHAR_MAP + $0078,x
	sta CHAR_MAP + $0050,y
	lda COLOR_MAP + $0078,x
	sta COLOR_MAP + $0050,y
	lda CHAR_MAP + $00A0,x
	sta CHAR_MAP + $0078,y
	lda COLOR_MAP + $00A0,x
	sta COLOR_MAP + $0078,y
	lda CHAR_MAP + $00C8,x
	sta CHAR_MAP + $00A0,y
	lda COLOR_MAP + $00C8,x
	sta COLOR_MAP + $00A0,y
	lda CHAR_MAP + $00F0,x
	sta CHAR_MAP + $00C8,y
	lda COLOR_MAP + $00F0,x
	sta COLOR_MAP + $00C8,y
	lda CHAR_MAP + $0118,x
	sta CHAR_MAP + $00F0,y
	lda COLOR_MAP + $0118,x
	sta COLOR_MAP + $00F0,y
	lda CHAR_MAP + $0140,x
	sta CHAR_MAP + $0118,y
	lda COLOR_MAP + $0140,x
	sta COLOR_MAP + $0118,y
	lda CHAR_MAP + $0168,x
	sta CHAR_MAP + $0140,y
	lda COLOR_MAP + $0168,x
	sta COLOR_MAP + $0140,y
	lda CHAR_MAP + $0190,x
	sta CHAR_MAP + $0168,y
	lda COLOR_MAP + $0190,x
	sta COLOR_MAP + $0168,y
	lda CHAR_MAP + $01B8,x
	sta CHAR_MAP + $0190,y
	lda COLOR_MAP + $01B8,x
	sta COLOR_MAP + $0190,y
	inx
	cpx #$28
	bne sstc1
	;
	; ---
	;
	ldx #$27
	lda #$00
cl_01	sta COLOR_MAP + $01B8,x			; clear middle line
	dex
	bpl cl_01
	;
	; ---
	;
	lda #$CB
	cmp $FF1D
	bne *-3
	;
	ldx #$25
	ldy #$27
sstc2	dey
	lda CHAR_MAP + $0320,x
	sta CHAR_MAP + $0348,y
	lda COLOR_MAP + $0320,x
	sta COLOR_MAP + $0348,y
	lda CHAR_MAP + $02F8,x
	sta CHAR_MAP + $0320,y
	lda COLOR_MAP + $02F8,x
	sta COLOR_MAP + $0320,y
	lda CHAR_MAP + $02D0,x
	sta CHAR_MAP + $02F8,y
	lda COLOR_MAP + $02D0,x
	sta COLOR_MAP + $02F8,y
	lda CHAR_MAP + $02A8,x
	sta CHAR_MAP + $02D0,y
	lda COLOR_MAP + $02A8,x
	sta COLOR_MAP + $02D0,y
	lda CHAR_MAP + $0280,x
	sta CHAR_MAP + $02A8,y
	lda COLOR_MAP + $0280,x
	sta COLOR_MAP + $02A8,y
	lda CHAR_MAP + $0258,x
	sta CHAR_MAP + $0280,y
	lda COLOR_MAP + $0258,x
	sta COLOR_MAP + $0280,y
	lda CHAR_MAP + $0230,x
	sta CHAR_MAP + $0258,y
	lda COLOR_MAP + $0230,x
	sta COLOR_MAP + $0258,y
	lda CHAR_MAP + $0208,x
	sta CHAR_MAP + $0230,y
	lda COLOR_MAP + $0208,x
	sta COLOR_MAP + $0230,y
	lda CHAR_MAP + $01E0,x
	sta CHAR_MAP + $0208,y
	lda COLOR_MAP + $01E0,x
	sta COLOR_MAP + $0208,y
	lda CHAR_MAP + $01B8,x
	sta CHAR_MAP + $01E0,y
	lda COLOR_MAP + $01B8,x
	sta COLOR_MAP + $01E0,y
	dex
	bpl sstc2
	rts
	; --------
SUB_DELAY
	ldx #$FF
dly07	ldy #$10
dly08	dey					; Delay
	bne dly08
	dex
	bne dly07
	rts
	; --------
BUILD_MAP
	sei
	;
	lda #$00				; calculate map data address
	sta ptr2
	sta ptr2+1
	lda VAR_CURRENT_FLOOR
	and #$03
	asl a					; mul * 4, each map is $0400 bytes long
	asl a
	sta ptr2+1
	lda ptr2+1
	clc					; source
	adc #map_data >> 8			; high byte
	sta ptr2+1
	;
	lda #GAME_MAP & 255			; pointer to unpacked game map
	sta zp02
	lda #GAME_MAP >> 8
	sta zp02+1
	;
	lda #$00
	sta VAR_MAP_Y_COUNTER
	sta VAR_MAP_Y
	;
build_map_loop
	ldx #$00
	stx VAR_MAP_X
L_6A7D	lda #$00
	sta zp04
	sta zp04+1
	lda zp02
	pha
	lda zp02+1
	pha
	txa
	tay
	;
	lda (ptr2),y				; get byte from map data
	;
	cmp #$30				; enemy soldier facing down
	beq B_6AD2
	cmp #$31				; enemy soldier facing down
	beq B_6AD2
	;
	cmp #$32				; enemy soldier facing right
	beq B_6AD7
	cmp #$33				; enemy soldier facing right
	beq B_6AD7
	;
	cmp #$34				; enemy soldier facing up
	beq B_6ADC
	cmp #$35				; enemy soldier facing up
	beq B_6ADC
	;
	cmp #$36
	beq B_6AE1				; enemy soldier facing left
	cmp #$37
	beq B_6AE1				; enemy soldier facing left
	;
	cmp #$62				; detonator
	beq B_6AB8
	;
	cmp #$63				; prisoner
	beq B_6AC5
	;
	jmp L_6B02
	;
B_6AB8	lda zp02				; save position of detonator
	sta VAR_DETONATOR_LO
	lda zp02+1
	sta VAR_DETONATOR_HI
	;
	jmp L_6B02
	;
B_6AC5	lda zp02				; save position (address) of prisoner
	sta VAR_PRISONER_LO
	lda zp02+1
	sta VAR_PRISONER_HI
	;
	jmp L_6B02
	;
B_6AD2	lda #$02				; down
	bne place_enemy
	;
B_6AD7	lda #$04				; right
	bne place_enemy
	;
B_6ADC	lda #$01				; up
	bne place_enemy
	;
B_6AE1	lda #$03				; left
	;
place_enemy
	ldy VAR_ENEMY_COUNT			; --- set up enemy soldiers
	sta ENEMY_DIRECTION_AND_FLAGS,y
	;
	lda zp02
	;lda #$00				; for testing: enemies don't move
	sta ENEMY_LO,y				; enemy soldier pointer low
	lda zp02+1
	;lda #$00
	sta ENEMY_HI,y				; enemy soldier pointer hi
	;
	lda VAR_MAP_X
	sta ENEMY_X,y
	;
	lda VAR_MAP_Y
	sta ENEMY_Y,y
	;
	inc VAR_ENEMY_COUNT
	;
L_6B02	txa
	tay
	lda (ptr2),y
	sta zp04				; calculate tile pointer
	;
	asl zp04				; 16 bit *16
	rol zp04+1
	asl zp04
	rol zp04+1
	asl zp04
	rol zp04+1
	asl zp04
	rol zp04+1
	;
	lda zp04+1
	clc
	adc #TILE_CHARS >> 8
	sta zp04+1
	;
	ldy #$00
B_6B21	;					; 1st line	
	lda (zp04),y
	sta (zp02),y
	;
	iny
	cpy #$04
	bne B_6B21
	;
	lda zp02
	clc
	adc #$80
	sta zp02
	bcc B_6B35
	inc zp02+1
B_6B35	lda zp04
	clc
V_6B38	adc #$04
	sta zp04
	;
	ldy #$00
B_6B3E	;					; 2nd line	
	lda (zp04),y
	sta (zp02),y
	;
	iny
	cpy #$04
	bne B_6B3E
	lda zp02
	clc
	adc #$80
	sta zp02
	bcc B_6B52
	inc zp02+1
B_6B52	lda zp04
	clc
	adc #$04
	sta zp04
	ldy #$00
B_6B5B	;					; 3rd line
	lda (zp04),y
	sta (zp02),y
	;
	iny
	cpy #$04
	bne B_6B5B
	lda zp02
	clc
	adc #$80
	sta zp02
	bcc B_6B6F
	inc zp02+1
B_6B6F	lda zp04
	clc
	adc #$04
	sta zp04
	ldy #$00
B_6B78	;					; 4th line
	lda (zp04),y
	sta (zp02),y
	;
	iny
	cpy #$04
	bne B_6B78
	;
	pla
	sta zp02+1
	pla
	sta zp02
	;
	lda zp02
	clc
	adc #$04
	sta zp02
	bcc B_6B92
	inc zp02+1
B_6B92	;
	lda VAR_MAP_X
	clc
	adc #$04
	sta VAR_MAP_X
	inx
	cpx #$20
	beq B_6BA3
	jmp L_6A7D
	;
B_6BA3	inc zp02+1
	lda zp02
	clc
	adc #$80
	sta zp02
	bcc B_6BB0
	inc zp02+1
B_6BB0	lda ptr2
	clc
	adc #$20
	sta ptr2
	bcc B_6BBB
	inc ptr2+1
B_6BBB	inc VAR_MAP_Y_COUNTER			; are we done with the map?
	lda VAR_MAP_Y_COUNTER
	cmp #$20
	beq B_6BD1
	;
	lda VAR_MAP_Y
	clc
	adc #$04
	sta VAR_MAP_Y
	;
	jmp build_map_loop
	;
B_6BD1	cli
	rts
	; --------
L_6BD7	jsr SUB_UPDATE_PRISONER_SPRITE
	;
	lda VAR_PRISONER_LO_TEMP2
	sta ptr2
	lda VAR_PRISONER_HI_TEMP2
	sta ptr2+1
	;
	ldy #$03
B_6BE6	lda #$00				; erase prisoner
	sta (ptr2),y
	inc ptr2+1
	sta (ptr2),y
	tya
	ora #$80
	tay
	lda #$00
	sta (ptr2),y
	dec ptr2+1
	sta (ptr2),y
	tya
	and #$7F
	tay
	dey
	bpl B_6BE6
	;
	lda VAR_PRISONER_LO_TEMP1
	sta ptr2
	lda VAR_PRISONER_HI_TEMP1
	sta ptr2+1
	;
	ldy #$03				; 4x4 
	ldx #$03
B_6C0F	lda TILE_CHARS + $0630 + $00,x		; draw prisoner
	sta (ptr2),y
	inc ptr2+1
	lda TILE_CHARS + $0630 + $08,x
	sta (ptr2),y
	tya
	ora #$80
	tay
	lda TILE_CHARS + $0630 + $0C,x
	sta (ptr2),y
	dec ptr2+1
	lda TILE_CHARS + $0630 + $04,x
	sta (ptr2),y
	tya
	and #$7F
	tay
	dex
	dey
	bpl B_6C0F
	;
	rts
	; --------
B_6C34	lda #$01
	sta VAR_TEMP_FLAG
	rts
	; --------
SUB_MOVE_PRISONER
	lda VAR_PRISONER_FLAG
	beq B_6C40
	rts
	; --------
B_6C40	lda VAR_PRISONER_CNT
	cmp #$01
	beq B_6C4B
	inc VAR_PRISONER_CNT
	rts
	; --------
B_6C4B	lda #$00
	sta VAR_PRISONER_CNT
	;
	lda VAR_PRISONER_X
	clc
	adc #$0A
	sec
	sbc ptr3
	bcc B_6C34
	sbc #$30
	bcs B_6C34
	;
	lda VAR_PRISONER_Y
	clc
	adc #$0A
	sec
	sbc ptr3+1
	bcc B_6C34
	sbc #$28
	bcs B_6C34
	;
	lda #$00
	sta VAR_TEMP_FLAG
	;
	lda VAR_PRISONER_IS_CLOSE_FLAG
	bne B_6CC8
	;
	lda #TXT_PRISONER_CLOSE & 255
	sta ptr2
	lda #TXT_PRISONER_CLOSE >> 8
	sta ptr2+1
	jsr SUB_SET_SCROLLTEXT
	;
	lda #$01
	sta VAR_PRISONER_IS_CLOSE_FLAG
	;
B_6CC8	jsr S_6DB0
	;
	lda VAR_PRISONER_LO_TEMP1
	sta ptr2
	lda VAR_PRISONER_HI_TEMP1
	sta ptr2+1
	;
	lda VAR_PRISONER_FLAGS
	and #$03
	beq B_6CD2
	cmp #$01
	beq B_6CC9
	cmp #$02
	beq B_6CCC
	cmp #$03
	beq B_6CCF
	;
	rts
	;
	; --------
	;
B_6CC9	jmp L_6D09
	;
B_6CCC	jmp L_6D40
	;
B_6CCF	jmp L_6D79
	;
B_6CD2	dec ptr2+1
	ldy #$80
B_6CD6	lda (ptr2),y
	bne B_6D00
	iny
	cpy #$84
	bne B_6CD6
	;
	lda VAR_PRISONER_LO_TEMP1
	sta VAR_PRISONER_LO_TEMP2
	lda VAR_PRISONER_HI_TEMP1
	sta VAR_PRISONER_HI_TEMP2
	;
	dec VAR_PRISONER_Y
	lda VAR_PRISONER_LO_TEMP1
	sec
	sbc #$80
	sta VAR_PRISONER_LO_TEMP1
	bcs B_6CFC
	dec VAR_PRISONER_HI_TEMP1
B_6CFC	jsr L_6BD7
	rts
	; --------
B_6D00	lda VAR_PRISONER_FLAGS
	and #$F7
	sta VAR_PRISONER_FLAGS
	rts
	; --------
L_6D09	inc ptr2+1
	inc ptr2+1
	ldy #$03
B_6D0F	lda (ptr2),y
	bne B_6D37
	dey
	bpl B_6D0F
	lda VAR_PRISONER_LO_TEMP1
	sta VAR_PRISONER_LO_TEMP2
	lda VAR_PRISONER_HI_TEMP1
	sta VAR_PRISONER_HI_TEMP2
	inc VAR_PRISONER_Y
	lda VAR_PRISONER_LO_TEMP1
	clc
	adc #$80
	sta VAR_PRISONER_LO_TEMP1
	bcc B_6D33
	inc VAR_PRISONER_HI_TEMP1
B_6D33	jsr L_6BD7
	rts
	; --------
B_6D37	lda VAR_PRISONER_FLAGS
	and #$F7
	sta VAR_PRISONER_FLAGS
	rts
	; --------
L_6D40	dec ptr2
	ldy #$00
	lda (ptr2),y
	bne B_6D70
	ldy #$80
	lda (ptr2),y
	bne B_6D70
	inc ptr2+1
	lda (ptr2),y
	bne B_6D70
	ldy #$00
	lda (ptr2),y
	bne B_6D70
	lda VAR_PRISONER_LO_TEMP1
	sta VAR_PRISONER_LO_TEMP2
	lda VAR_PRISONER_HI_TEMP1
	sta VAR_PRISONER_HI_TEMP2
	dec VAR_PRISONER_X
	dec VAR_PRISONER_LO_TEMP1
	jsr L_6BD7
	rts
	; --------
B_6D70	lda VAR_PRISONER_FLAGS
	ora #$08
	sta VAR_PRISONER_FLAGS
	rts
	; --------
L_6D79	ldy #$04
	lda (ptr2),y
	bne B_6DA7
	ldy #$84
	lda (ptr2),y
	bne B_6DA7
	inc ptr2+1
	lda (ptr2),y
	bne B_6DA7
	ldy #$04
	lda (ptr2),y
	bne B_6DA7
	lda VAR_PRISONER_LO_TEMP1
	sta VAR_PRISONER_LO_TEMP2
	lda VAR_PRISONER_HI_TEMP1
	sta VAR_PRISONER_HI_TEMP2
	inc VAR_PRISONER_X
	inc VAR_PRISONER_LO_TEMP1
	jsr L_6BD7
	rts
	; --------
B_6DA7	lda VAR_PRISONER_FLAGS
	ora #$08
	sta VAR_PRISONER_FLAGS
	rts
	; --------
S_6DB0	lda VAR_PRISONER_FLAGS
	and #$F0
	beq B_6DC1
	lda VAR_PRISONER_FLAGS
	sec
	sbc #$10
	sta VAR_PRISONER_FLAGS
	rts
	; --------
B_6DC1	lda VAR_PRISONER_FLAGS
	and #$08
	bne L_6DE9
L_6DC8	lda VAR_PRISONER_X
	sec
	sbc VAR_PLAYER_CALC_X
	beq B_6E0A
	bcs B_6DDE
	lda VAR_PRISONER_FLAGS
	and #$F8
	ora #$03
	sta VAR_PRISONER_FLAGS
	rts
	; --------
B_6DDE	lda VAR_PRISONER_FLAGS
	and #$F8
	ora #$02
	sta VAR_PRISONER_FLAGS
	rts
	; --------
L_6DE9	lda VAR_PRISONER_Y
	sec
	sbc VAR_PLAYER_CALC_Y
	beq B_6E15
	bcs B_6DFF
	lda VAR_PRISONER_FLAGS
	and #$F8
	ora #$01
	sta VAR_PRISONER_FLAGS
	rts
	; --------
B_6DFF	lda VAR_PRISONER_FLAGS
	and #$F8
	ora #$00
	sta VAR_PRISONER_FLAGS
	rts
	; --------
B_6E0A	lda VAR_PRISONER_FLAGS
	and #$F7
	sta VAR_PRISONER_FLAGS
	jmp L_6DE9
	;
B_6E15	lda VAR_PRISONER_FLAGS
	ora #$08
	sta VAR_PRISONER_FLAGS
	jmp L_6DC8
	;
	; ---
	;
L_6E20	lda VAR_PRISONER_FLAGS
	and #$03
	beq B_6E39
	cmp #$01
	beq B_6E43
	cmp #$02
	beq B_6E4B
	lda VAR_PRISONER_FLAGS
	and #$F8
	ora #$02
	jmp L_6E52
	;
B_6E39	lda VAR_PRISONER_FLAGS
	and #$F8
	ora #$01
	jmp L_6E52
	;
B_6E43	lda VAR_PRISONER_FLAGS
	and #$F8
	jmp L_6E52
	;
B_6E4B	lda VAR_PRISONER_FLAGS
	and #$F8
	ora #$03
L_6E52	ora #$F0
	sta VAR_PRISONER_FLAGS
	rts
	;
	; --------
	;
SUB_UPDATE_PRISONER_SPRITE
	inc VAR_PRISONER_TICK			; update prisoner's gfx
	lda VAR_PRISONER_TICK
	and #$02
	beq B_6E68
	lda #$00
	jmp L_6E6A
	;
B_6E68	lda #$01
L_6E6A	sta VAR_PRISONER_STEP
	;
	lda VAR_PRISONER_FLAGS
	and #$03
	asl a
	adc VAR_PRISONER_STEP
	tax
	lda prisoner_gfx_lo,x
	sta prisoner_gfx_src + 1
	lda prisoner_gfx_hi,x
	sta prisoner_gfx_src + 2
	;
	ldx #$7F				; 128 bytes (4x4 chars)
prisoner_gfx_src
	lda $FFFF,x
	sta CHARSET_GAME + PRISONER_CHR * 8,x	; copy prisoner gfx
	dex
	bpl prisoner_gfx_src
	;
	; --------
	;
L_6EA9	lda zp02+1
	sec
	sbc #GAME_MAP >> 8
	asl a
	sta VAR_PLAYER_SCREEN_TEMP_Y
	;
	lda zp02
	and #$80
	beq B_6EBB
	inc VAR_PLAYER_SCREEN_TEMP_Y
B_6EBB	;
	lda zp02
	and #$7F
	sta VAR_PLAYER_SCREEN_TEMP_X
	ldx #$00
B_6EC4	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$20
	bne B_6F29
	;
	lda VAR_PLAYER_SCREEN_TEMP_X
	sec
	sbc ENEMY_X,x
	cmp #$05
	bcs B_6F29
	;
	lda VAR_PLAYER_SCREEN_TEMP_Y
	sec
	sbc ENEMY_Y,x
	cmp #$05
	bcs B_6F29
	;
	lda ENEMY_DIRECTION_AND_FLAGS,x		; shot once already?
cheat01	and #$40				; with cheat, this will be an LDA#$40, ensuring next jump
	bne enemy_killed
	;
	lda ENEMY_DIRECTION_AND_FLAGS,x
	ora #$40				; mark enemy as shot
	sta ENEMY_DIRECTION_AND_FLAGS,x
	;
	rts
	; --------
enemy_killed
	lda #SND_HIT
	sta VAR_SOUND_REQUESTED
	;
	inc VAR_SCORE_ADD			; score += 100
	;
	lda ENEMY_DIRECTION_AND_FLAGS,x
nuke03	ora #$20
	sta ENEMY_DIRECTION_AND_FLAGS,x
	;
	lda ENEMY_LO,x
	sta zp02
	lda ENEMY_HI,x
	sta zp02+1
	;
	ldx #$00
	ldy #$03
ek01	txa					; erase enemy soldier from map
	sta (zp02),y
	inc zp02+1
	sta (zp02),y
	tya
	ora #$80
	tay
	txa
	sta (zp02),y
	dec zp02+1
	sta (zp02),y
	tya
	and #$7F
	tay
	dey
	bpl ek01
	;
	rts
	; --------
B_6F29	inx
	cpx VAR_ENEMY_COUNT
	bne B_6EC4
	rts
	;
	; ---
	;
copy_one_byte macro addr
	lda (sprite_src_ptr),y
	sta addr
	iny
	endm
copy_8_bytes macro addr
	copy_one_byte addr+0
	copy_one_byte addr+1
	copy_one_byte addr+2
	copy_one_byte addr+3
	copy_one_byte addr+4
	copy_one_byte addr+5
	copy_one_byte addr+6
	copy_one_byte addr+7
	endm
	;
	; --- player sprite conversion
	;
SUB_CALC_SPRITE_SCREEN_POS
	lda VAR_PLAYER_SCREEN_Y
	sec
	sbc #$36
	lsr a
	lsr a
	lsr a
	tax
	;
	lda VAR_PLAYER_SCREEN_X
	lsr a
	lsr a
	lsr a
	sec
	sbc #$04
	clc
	adc screen_pos_lo,x
	sta sprite_chr_ptr
	lda screen_pos_hi,x
	adc #$00
	sta sprite_chr_ptr+1 
	;
	; ---
	;
	lda sprite_chr_ptr
	sta sprite_col_ptr
	lda sprite_chr_ptr+1
	and #$FB
	sta sprite_col_ptr+1 
	;
	rts
	;
	; ---
	;
SUB_SPRITE_CONVERSION
	ldx #$0F				; 4x4 chars
ssc1	ldy sprite_char_pos_index,x
	lda (sprite_chr_ptr),y			; only place player chars over 0 (floor)
	bne ssc2
	;beq ssc3
	;cmp #$0A
	;bcs ssc2
	;lda #$40
	;bne ssc4
	;
ssc3	lda player_sprite_chars,x 
ssc4	sta (sprite_chr_ptr),y			; place player sprite char
player_sprite_color
	lda #$3d				; *** player sprite color
	sta (sprite_col_ptr),y
	;
ssc2	dex
	bpl ssc1
	;
	ldy #$00
	copy_8_bytes CHARSET_GAME + $E0 * 8
	copy_8_bytes CHARSET_GAME + $E1 * 8
	copy_8_bytes CHARSET_GAME + $E2 * 8
	copy_8_bytes CHARSET_GAME + $E3 * 8
	copy_8_bytes CHARSET_GAME + $E4 * 8
	copy_8_bytes CHARSET_GAME + $E5 * 8
	copy_8_bytes CHARSET_GAME + $E6 * 8
	copy_8_bytes CHARSET_GAME + $E7 * 8
	copy_8_bytes CHARSET_GAME + $E8 * 8
	copy_8_bytes CHARSET_GAME + $E9 * 8
	copy_8_bytes CHARSET_GAME + $EA * 8
	copy_8_bytes CHARSET_GAME + $EB * 8
	copy_8_bytes CHARSET_GAME + $EC * 8
	copy_8_bytes CHARSET_GAME + $ED * 8
	copy_8_bytes CHARSET_GAME + $EE * 8
	copy_8_bytes CHARSET_GAME + $EF * 8
	;
	rts
	;
	; ---
	;
lift_hotkey
	lda #$DF
	sta $FD30
	lda #$FF
	sta $FF08
	lda $FF08
	and #$04				; Query keyboard for "L"
	bne lh_not_pressed
	;
	lda #$02				; entering lift
	sta VAR_LIFT_STATE
	;
lh_not_pressed
	rts
	
ghost_mode_hotkey
	lda #$00				; flag to check for key down or key up
	bne ghost_mode_check_release
	;
	lda #$F7
	sta $FD30
	lda #$FF
	sta $FF08
	lda $FF08
	and #$04				; Query keyboard for "G"
	bne gm_no_change
	;
	lda #$01
	bne set_gm_flag				; always jump
	;
ghost_mode_check_release
	lda #$F7
	sta $FD30
	lda #$FF
	sta $FF08
	lda $FF08
	and #$04				; Query keyboard for "G"
	beq gm_no_change
	;
	lda gm01+1
	eor #($2D ^ $75)
	sta gm01+1
	sta gm02+1
	sta gm03+1
	sta gm04+1
	sta gm11+1
	sta gm12+1
	sta gm13+1
	sta gm14+1
	sta gm21+1
	sta gm22+1
	sta gm23+1
	sta gm24+1
	sta gm31+1
	sta gm32+1
	sta gm33+1
	sta gm34+1
	;
	lda player_sprite_color+1
	eor #($3D ^ $39)
	sta player_sprite_color+1
	;
	lda check_door_vertical			; disable door checking in ghost mode
	eor #($EA ^ $60)			; so keys are not used 
	sta check_door_vertical			; and avoids a visual bug, when walking into "half" of a door
	;
	lda #$00
set_gm_flag
	sta ghost_mode_hotkey+1			; update flag
	;
gm_no_change
	rts
	;
	;
	;
nuke_hotkey
	lda #$EF
	sta $FD30
	lda #$FF
	sta $FF08
	lda $FF08
	and #$80				; Query keyboard for "N"
	bne nuke_not_pressed
	;
	lda #$01
	sta VAR_PAUSE
	;
	ldx #$00
nuke02	lda ENEMY_DIRECTION_AND_FLAGS,x
	and #$20
	bne nuke01
	;
	txa
	pha
	;
	lda ENEMY_DIRECTION_AND_FLAGS,x
	ora #$40
	jsr nuke03
	;
	pla
	tax
	;
nuke01	inx
	cpx VAR_ENEMY_COUNT
	bne nuke02
	;
	ldx #$01
	stx VAR_ENEMY_COUNT
	dex
	stx VAR_ENEMY_INDEX
	stx VAR_PAUSE
	;
nuke_not_pressed
	rts
	;
cheat_menu
	lda #$CB
	cmp $FF1D
	bne *-3
	;
	sei
	;
	lda #$00				; stop music
	sta $FF11
	;
	lda #$0B
	sta $FF06
	;
	lda #IRQ_NULL & 255
	sta $FFFE
	lda #IRQ_NULL >> 8
	sta $FFFF
	cli
	;
	ldx #$00
	lda #$80
cmc01	sta COLOR_MAP + $0000,x			; clear the screen
	sta COLOR_MAP + $0100,x
	sta COLOR_MAP + $0200,x
	sta COLOR_MAP + $0300,x
	sta CHAR_MAP + $0000,x
	sta CHAR_MAP + $0100,x
	sta CHAR_MAP + $0200,x
	sta CHAR_MAP + $0300,x
	inx
	bne cmc01
	;
	ldx #$17
cm04	lda charset_1x1,x			; copy the 1x1 set (in 3 chunks)
	sta CHARSET_2X2 + $75*8,x
	lda charset_1x1 + $18,x
	sta CHARSET_2X2 + $81*8,x
	dex
	bpl cm04
	;
	ldx #$C0
cm01	lda charset_1x1 + $30 - 1,x		; copy the 1x1 set to the end of the 2x2 charset
	sta CHARSET_2X2 + $740 - 1,x
	dex
	bne cm01
	; 
	; ---
	;
	lda #$4A				; set colors and print menu title
	sta PRINT_COLOR1
	lda #$5A
	sta PRINT_COLOR2
	print TXT_CHEATS_TITLE, 14,1
	;
	lda #TXT_CHEATS & 255			; set the data pointer
	sta ptr1
	lda #TXT_CHEATS >> 8
	sta ptr1+1
	;
	lda #$03				; starting line (y coordinate)
	sta cm_line_count+1
	;
	ldy #$FF
	;
cm_newline
	iny
	tya
	clc
	adc ptr1
	sta ptr1
	bcc *+4
	inc ptr1+1 
	;
	inc cm_line_count+1
	;
cm_line_count
	ldx #$03				; get screen position of current line
	lda screen_pos_lo,x
	ora #$04				; add 4 to x
	sta ptr2
	sta ptr3
	lda screen_pos_hi,x
	sta ptr2+1
	and #$FB
	sta ptr3+1
	;
	ldy #$00
cm02	lda (ptr1),y
	beq cm03				; $00 = end of text
	cmp #$0D				; $0D = newline
	beq cm_newline
	sta (ptr2),y
	lda #$31				; default color
	sta (ptr3),y
	iny
	bne cm02				; always jump
	;
cm03	; ---
	;
	ldx #4					; color the headers
	ldy #$35
	jsr cm_color_line_absolute
	;
	ldx #9
	ldy #$3C
	jsr cm_color_line_absolute
	;
	ldx #14
	ldy #$33
	jsr cm_color_line_absolute
	;
	ldx #19
	ldy #$3D
	jsr cm_color_line_absolute
	;
	; ---
	;
	ldx #(COLOR_MAP + 3 + 4 * 40) & 255	; draw borders
	lda #(COLOR_MAP + 3 + 4 * 40) >> 8
	jsr cm_draw_border
	;
	ldx #(COLOR_MAP + 3 + 9 * 40) & 255
	lda #(COLOR_MAP + 3 + 9 * 40) >> 8
	jsr cm_draw_border
	;
	ldx #(COLOR_MAP + 3 + 14 * 40) & 255
	lda #(COLOR_MAP + 3 + 14 * 40) >> 8
	jsr cm_draw_border
	;
	ldx #(COLOR_MAP + 3 + 19 * 40) & 255
	lda #(COLOR_MAP + 3 + 19 * 40) >> 8
	jsr cm_draw_border
	;
	lda $FF07
	and #$40
	ora #$88
	sta $FF07
	;
	lda #$1B
	sta $FF06
	;
	ldx #0
cm05	txa
	pha
	jsr cm_update_value
	pla
	tax
	pha
	lda cm_line_color,x
	tay
	jsr cm_color_line
	;
	pla
	tax
	inx
	cpx #12
	bne cm05
	;
	; ---
	;    
	lda #$00
	sta cheat_selection
	sta KEYBOARD
	;
	db $2C
cm_set_keyboard_delay
	lda #$08
	sta KEYBOARD+1
cm_loop
	ldx cheat_selection
	inc cm_tick+1
cm_tick
	lda #$00
	and #$08
	asl a
	eor cm_line_color,x
	eor #$30
	tay
	jsr cm_show_selection
	;
	lda #$CC
	cmp $FF1D
	bne *-3
	cmp $FF1D
	beq *-3
	;
	jsr SUB_JOYSTICK_CHECK
	cmp #$FF
	bne cm11
	;
	lda #$00				; clear delay
	sta KEYBOARD
	sta KEYBOARD+1
	beq cm_loop
	;
cm11	cmp KEYBOARD
	bne cm10
	;
	dec KEYBOARD+1
	bne cm_loop
	;
cm10	sta KEYBOARD
	tax
	and #$01
	beq cm_up
	txa
	and #$02
	beq cm_down
	txa
	and #$10
	beq cm_inc
	;
	jmp cm_loop

cm_up	jsr cm_clear_selection
	lda cheat_selection
	bne *+4
	lda #13
	sec
	sbc #$01
	sta cheat_selection
	;
	jmp cm_set_keyboard_delay

cm_down	jsr cm_clear_selection
	lda cheat_selection
	cmp #12
	bne *+4
	lda #0
	clc
	adc #$01
	sta cheat_selection
	;
	jmp cm_set_keyboard_delay

cm_inc	ldx cheat_selection
	cpx #12
	beq cm_set_cheats
	;
	lda cheat_flags,x			; increment cheat state
	cmp cm_max_values,x			; are we at the max? 
	bne cmi1
	;
	lda #$00
	beq cmi2
	;
cmi1	clc
	adc #$01
cmi2	sta cheat_flags,x
	;
	jsr cm_update_value
	;
	jmp cm_set_keyboard_delay
	;
	; ---
	;
cm_and_lda
	db $29,$A9
cm_nop_clc
	db $EA,$18
cm_enemy_speed_1
	db $20,$20,$2C,$20
cm_enemy_speed_2
	db 3,5,5,2
cm_10
	db 1,0
cm_player_speed
	db 3,2,1,4
cm_lda_lda
	db $A5,$A9
cm_beq_bit
	db $F0,$24
cm_cheat09_lo
	db LAB_SHOT_EXPLOSIVES & 255
	db LAB_DRAW_EMPTY_CRATE	& 255
cm_cheat09_hi
	db LAB_SHOT_EXPLOSIVES >> 8
	db LAB_DRAW_EMPTY_CRATE >> 8
cm_bit_jsr
	db $2C,$20
cm_set_cheats
	ldx cheat_flags+0			; cheat #1
	lda cm_and_lda,x			; one-shot
	sta cheat01
	;
	ldx cheat_flags+1			; cheat #2
	lda cm_nop_clc,x			; respawn
	sta cheat02
	;
	ldx cheat_flags+2			; cheat #3
	lda cm_enemy_speed_1,x			; enemy speed
	sta cheat03a
	lda cm_enemy_speed_2,x
	sta cheat03b+1
	;
	ldx cheat_flags+3			; cheat #4
	lda cm_10,x				; invulnerability
	sta cheat04+1
	;
	ldx cheat_flags+4			; cheat #5
	lda cm_10,x				; unlimited ammo
	sta cheat05+1
	;
	ldx cheat_flags+5			; cheat #6
	lda cm_player_speed,x			; player speed
	sta cheat06+1
	;
	ldx cheat_flags+6			; cheat #7
	lda cm_lda_lda,x			; switch between LDA $00 and LDA #$00
	sta cheat07a
	sta cheat07b
	;
	ldx cheat_flags+7			; cheat #8
	lda cm_beq_bit,x			; check lift state or not
	sta cheat08
	;
	ldx cheat_flags+8			; cheat #9
	lda cm_cheat09_lo,x			; overwrite JMP address
	sta cheat09+1
	lda cm_cheat09_hi,x
	sta cheat09+2
	;
	ldx cheat_flags+9
	lda cm_bit_jsr,x
	sta cheat10
	;
	ldx cheat_flags+10
	lda cm_bit_jsr,x
	sta cheat11
	;
	ldx cheat_flags+11
	lda cm_bit_jsr,x
	sta cheat12
	;
	jsr SUB_JOYSTICK_CHECK			; wait for release
	cmp #$FF
	bne *-5
	;
	jmp INTRO_COLD_START			; exit from cheat menu
	;jmp $FFF6
	;
	; ---
	;
cm_draw_border
	stx ptr1
	stx ptr2
	sta ptr1+1
	ora #$04
	sta ptr2+1
	;
	ldy #$00
	lda #$41				; corner
	sta (ptr1),y
	lda #$75
	sta (ptr2),y
	;
	ldy #33
	lda #$41
	sta (ptr1),y
	lda #$77
	sta (ptr2),y
	;
cmdb1	dey
	lda (ptr2),y
	cmp #$80
	bne cmdb2
	lda #$51				; top
	sta (ptr1),y
	lda #$76
	sta (ptr2),y
	bne cmdb1
	;
cmdb2	ldx #$02
	ldy #40
	;
cmdb3	lda #$51				; sides
	sta (ptr1),y
	lda #$81
	sta (ptr2),y
	;
	tya
	clc
	adc #33
	tay
	;
	lda #$51
	sta (ptr1),y
	lda #$81
	sta (ptr2),y
	;
	tya
	clc
	adc #(40-33)
	tay
	;
	dex
	bpl cmdb3
	;
	ldy #4*40+0
	lda #$41				; bottom corners
	sta (ptr1),y
	lda #$82
	sta (ptr2),y
	;
	ldy #4*40+33
	lda #$41
	sta (ptr1),y
	lda #$83
	sta (ptr2),y
	;
cmdb4	dey
	cpy #4*40+0
	beq cmdb5
	lda #$51				; bottom
	sta (ptr1),y
	lda #$76
	sta (ptr2),y
	bne cmdb4
	;
cmdb5	rts
	;
	
	; x = line index
cm_update_value
	lda cheat_flags,x			; print the cheat value on screen
	asl a
	asl a
	asl a
	clc
	adc cm_value_lo,x
	sta ptr1
	lda cm_value_hi,x
	adc #$00
	sta ptr1+1
	; 
	lda cm_line_index,x
	tax
	lda screen_pos_lo,x
	clc
	adc #28
	sta ptr3
	lda screen_pos_hi,x
	adc #$00
	sta ptr3+1
	;
	ldy #$07
cmuv1	lda (ptr1),y
	sta (ptr3),y
	dey
	bpl cmuv1
	;
	rts

cm_show_selection
	jsr cm_color_line
	;
;	lda ptr3+1
;	ora #$04
;	sta ptr3+1
;	;
;	ldy #2					; left side
;	jsr cmss1
;	;
;	ldy #36					; right side
;	;
;cmss1	lda #$B9				; draw the "selection dot"
;	sta (ptr3),y
;	iny
;	lda #$BB
;	sta (ptr3),y
	rts
 
cm_clear_selection
	ldx cheat_selection
	lda cm_line_color,x
	tay
	jsr cm_color_line
	;
;	lda ptr3+1
;	ora #$04
;	sta ptr3+1
;	;
;	lda #$80				; space character
;	ldy #2					; left side (position 2)
;	jsr cmcs1
;	;
;	ldy #36					; and right side (position 36)
;	;
;cmcs1	sta (ptr3),y				; erase two chars
;	iny
;	sta (ptr3),y
	rts
	
	; x = line index
	; y = color
cm_color_line
	lda cm_line_index,x
	tax
cm_color_line_absolute
	lda screen_pos_lo,x
	clc
	adc #$04
	sta ptr3
	lda screen_pos_hi,x
	and #$FB
	sta ptr3+1
	tya
	ldy #31
cmcl1	sta (ptr3),y
	dey
	bpl cmcl1
	rts

cm_line_index
	db 5,6,7
	db 10,11,12
	db 15,16,17
	db 20,21,22
	db 24
cm_line_color
	db $55,$55,$55 
	db $5C,$5C,$5C
	db $53,$53,$53
	db $5D,$5D,$5D
	db $31
cm_max_values
	db 1,1,3
	db 1,1,3
	db 1,1,1
	db 1,1,1
cm_value_lo
	db TXT_CHEAT_NO_YES & 255
	db TXT_CHEAT_YES_NO & 255
	db TXT_CHEAT_ENEMY_SPEED & 255
	;
	db TXT_CHEAT_NO_YES & 255
	db TXT_CHEAT_NO_YES & 255
	db TXT_CHEAT_PLAYER_SPEED & 255
	;
	db TXT_CHEAT_NO_YES & 255
	db TXT_CHEAT_YES_NO & 255
	db TXT_CHEAT_NO_YES & 255
	;
	db TXT_CHEAT_DISABLED_ENABLED & 255
	db TXT_CHEAT_DISABLED_ENABLED & 255
	db TXT_CHEAT_DISABLED_ENABLED & 255
cm_value_hi
	db TXT_CHEAT_NO_YES >> 8
	db TXT_CHEAT_YES_NO >> 8
	db TXT_CHEAT_ENEMY_SPEED >> 8
	;
	db TXT_CHEAT_NO_YES >> 8
	db TXT_CHEAT_NO_YES >> 8
	db TXT_CHEAT_PLAYER_SPEED >> 8
	;
	db TXT_CHEAT_NO_YES >> 8
	db TXT_CHEAT_YES_NO >> 8
	db TXT_CHEAT_NO_YES >> 8
	;
	db TXT_CHEAT_DISABLED_ENABLED >> 8
	db TXT_CHEAT_DISABLED_ENABLED >> 8
	db TXT_CHEAT_DISABLED_ENABLED >> 8
	;
	; === GAME DATA
	;
inner_check_table
	db $07,$08,$09,$0A
	db $0D,$0E,$0F,$10
	db $13,$14,$15,$16
	db $19,$1A,$1B,$1C
	;
	; --------
	;
player_sprite_lo
	db 0
	db player_up_1 & 255
	db player_down_1 & 255
	db player_left_1 & 255
	db player_right_1 & 255
	;
	db player_up_2 & 255
	db player_down_2 & 255
	db player_left_2 & 255
	db player_right_2 & 255
player_sprite_hi
	db 0
	db player_up_1 >> 8
	db player_down_1 >> 8
	db player_left_1 >> 8
	db player_right_1 >> 8
	;
	db player_up_2 >> 8
	db player_down_2 >> 8
	db player_left_2 >> 8
	db player_right_2 >> 8
	;
player_shoot_lo
	db 0
	db player_up_shoot & 255
	db player_down_shoot & 255
	db player_left_shoot & 255
	db player_right_shoot & 255
player_shoot_hi
	db 0
	db player_up_shoot >> 8
	db player_down_shoot >> 8
	db player_left_shoot >> 8
	db player_right_shoot >> 8
	;
player_flash_lo
	db 0
	db player_up_flash & 255
	db player_down_flash & 255
	db player_left_flash & 255
	db player_right_flash & 255
player_flash_hi
	db 0
	db player_up_flash >> 8
	db player_down_flash >> 8
	db player_left_flash >> 8
	db player_right_flash >> 8
	;
bullet_table_1	db $10,$08,$00,$18
bullet_table_2	db $00,$18,$08,$10
	;
	; "todays' real heroes" text fade in colors
	;
HS_FADE_COLORS_1
	db $02,$02,$02,$12,$12,$22,$22,$32,$32,$32
HS_FADE_COLORS_2
	db $02,$02,$12,$12,$22,$22,$32,$32,$42,$42
	;
	; high scores fade in color
	;
HS_FADE_COLORS_3
	db $01,$01,$01,$11,$11,$21,$21,$31,$31,$41
HS_FADE_COLORS_4
	db $01,$11,$21,$31,$41,$51,$61,$61,$71,$71
	;
HS_SCORE
	db "000000"
HS_NAME
	db "........."
	db $12
screen_pos_lo
	db $00,$28,$50,$78,$A0
	db $C8,$F0,$18,$40,$68
	db $90,$B8,$E0,$08,$30
	db $58,$80,$A8,$D0,$F8
	db $20,$48,$70,$98,$C0
screen_pos_hi
	db (CHAR_MAP >> 8)+0,(CHAR_MAP >> 8)+0,(CHAR_MAP >> 8)+0,(CHAR_MAP >> 8)+0,(CHAR_MAP >> 8)+0
	db (CHAR_MAP >> 8)+0,(CHAR_MAP >> 8)+0,(CHAR_MAP >> 8)+1,(CHAR_MAP >> 8)+1,(CHAR_MAP >> 8)+1
	db (CHAR_MAP >> 8)+1,(CHAR_MAP >> 8)+1,(CHAR_MAP >> 8)+1,(CHAR_MAP >> 8)+2,(CHAR_MAP >> 8)+2
	db (CHAR_MAP >> 8)+2,(CHAR_MAP >> 8)+2,(CHAR_MAP >> 8)+2,(CHAR_MAP >> 8)+2,(CHAR_MAP >> 8)+2
	db (CHAR_MAP >> 8)+3,(CHAR_MAP >> 8)+3,(CHAR_MAP >> 8)+3,(CHAR_MAP >> 8)+3,(CHAR_MAP >> 8)+3
	;
	; === TEXT DATA ===
	;
TXT_WELL_DONE
	db "WELL DONE",0
TXT_REAL_HERO
	db "YOU ARE A REAL HERO",0
TXT_ENTER_YOUR_NAME
	db "ENTER YOUR NAME",0

TXT_LETTERS
	db $40,"ABCDEFGHIJKLMNOPQRSTUVWXYZ 0123456789!",$22,$23,$24,$25,$26,$27,$28,$29,$5F
TXT_DOTS
	db "    ",$29,"..........",$28,$00

TXT_TODAYS_REAL_HEROES
	db " TODAYS REAL HEROES",$00
TXT_HIGHSCORES
	db " PANDORA     010000 "
	db " PANDORA     009000 "
	db " PANDORA     008000 "
	db " PANDORA     007000 "
	db " PANDORA     006000 "
	db " PANDORA     005000 "
	db " PANDORA     004000 "
	db " PANDORA     003000 "
	db " PANDORA     002000 "
	db " PANDORA     001000 "
	db "                    "

TXT_PROGRAMMING
	db "PROGRAMMING BY",$00
TXT_ANDY
	db "ANDREW CHALLIS",$00
TXT_GRAPHICS
	db "GRAPHICS BY",$00
TXT_ROB
	db "ROBIN CHAPMAN",$00
TXT_VALUABLES
	db "VALUABLES",$00
TXT_PAINTING
	db "PAINTING",$00
TXT_VASE
	db "VASE",$00
TXT_PENDANT
	db "PENDANT",$00
TXT_JEWELS
	db "JEWELS",$00
TXT_SUPPLIES
	db " SUPPLIES ",$00			; space needed to clear prev. word
TXT_COLD_FOOD
	db "COLD FOOD",$00
TXT_DOOR_KEY
	db "DOOR KEY",$00
TXT_AMMUNITION
	db "AMMUNITION",$00
TXT_FIRST_AID
	db "FIRST AID",$00

TXT_DEAD
	db "YOU ARE DEAD",$00

TXT_SELECT_FLOOR
	db "SELECT FLOOR",$00
TXT_FL0	db "BASEMENT     ",$00
TXT_FL1	db "GROUND FLOOR ",$00
TXT_FL2	db "FIRST FLOOR  ",$00
TXT_FL3	db "SECOND FLOOR ",$00

TXT_SELECT_MISSION
	db "SELECT MISSION",$00

TXT_MISSION
	db "MISSION",$00
TXT_UNDERLINE
	db "---------",$00
TXT_MISSION_2_3_4
	db "RESCUE THE PRISONER",$00
TXT_MISSION_1
	db "BLOW UP THE CASTLE",$00

TXT_WIN_1
	db "WELL DONE!",$00
TXT_WIN_2
	db "YOUR MISSION WAS A",$00
TXT_WIN_3
	db "SUCCESS!",$00

TXT_FOUND
	db "YOU HAVE FOUND",$40
TXT_A_PAINTING
	db "A PA/NT/NG.",$40
TXT_A_VASE
	db "A VASE.",$40
TXT_SOME_COLD_FOOD
	db "SOME COLD FOOD.",$40
TXT_FIRST_AID_PERIOD
	db "F/RST A/D.",$40
TXT_A_PENDANT
	db "A PENDANT.",$40
TXT_LIFT_PASS
	db "A L/FT PASS.",$40
TXT_A_DOOR_KEY
	db "A DOOR KEY.",$40
TXT_DETONATOR_ACTIVATED
	db "DETONATOR ACT/VATED.",$40
TXT_YOU_HAVE_SHOT
	db "YOU HAVE SHOT",$40                
TXT_WELCOME
	db "         WELCOME TO THE EAGLE",$1D,"S NEST.",$40	; added apostrophe
TXT_AMMO_WASTED
	db "ANY MORE AMMO WOULD BE WASTED!",$40
TXT_OUT_OF_AMMO
	db "OUT OF AMMO! ",$40
TXT_DOOR_LOCKED
	db "TH/S DOOR /S LOCKED! ",$40
TXT_COME_ON
	db "COME ON! GET ON WITH /T.....",$40
TXT_LETS_GO
	db "LETS GET DOWN TO SOME SER/OUS DESTRUCT/ON!",$40
TXT_SHOT_LIFT_PASS
	db "SHOOT/NG A L/FT PASS /S A B/T S/LLY!",$40
TXT_NEED_LIFT_PASS
	db "YOU NEED A L/FT PASS! ",$40
TXT_ENTERING_LIFT
	db "YOU ARE ENTER/NG THE L/FT.        ",$40
TXT_BASEMENT
	db "/N THE BASEMENT.",$40
TXT_GROUND_FLOOR
	db "ON THE GROUND FLOOR.",$40
TXT_FIRST_FLOOR
	db "ON THE F/RST FLOOR.",$40
TXT_SECOND_FLOOR
	db "ON THE SECOND FLOOR.",$40
TXT_YOU_ARE
	db "YOU ARE",$40
TXT_WASTING_AMMO
	db "YOU ARE WAST/NG AMMO. ",$40	; typo fixed, it says "WASTING" in the c64 version
TXT_EMPTY
	db "EMPTY!",$40
TXT_FOOL_EXPLOSIVES
	db "FOOL!....EXPLOS/VES.        ",$40
TXT_SOME_JEWELS
	db "SOME JEWELS.",$40
TXT_MISSION_NOT_COMPLETE
	db "YOUR M/SS/ON /S NOT YET COMPLETE!",$40
TXT_PRISONER_RESCUED
	db "WELL DONE.....YOU HAVE RESCUED A PR/SONER.",$40
TXT_CASTLE_DESTROYED
	db "THE CASTLE /S NOW DESTROYED.",$40	; *** fixed, it says "DESTROYED" in the c64 version
TXT_PAUSING
	db "...PAUS/NG...",$40
TXT_PRISONER_CLOSE
	db "THE PR/SONER /S VERY CLOSE.",$40
	
	; --- cheats
TXT_CHEATS_TITLE
	db "CHEATS",0
	cmap " ",$80
	cmap "-",$7F
	cmap "A",$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF,$F0
	cmap "K",$F1,$F2,$F3,$F4,$F5,$F6
	cmap "R",$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE
	cmap ".",$FF
TXT_CREDITS_1
	db "ORIGINAL CONVERSION BY MUCSI"
TXT_CREDITS_2
	db "DELUXE VERSION BY CSABO OF LOD"
TXT_CREDITS_3
	db "PUSH LEFT FOR CHEAT MENU"
	
TXT_CHEATS
	;  |0123456789012345678901234567890123456789|
	; unused letters: J,Q,Z
	; $B9,$BB
	
	db "ENEMY SOLDIERS",$0D
	db "ONE-SHOT ENEMIES........",$0D ; no/yes
	db "RESPAWN ENEMIES.........",$0D ; yes/no
	db "MOVEMENT SPEED..........",$0D ; normal/slow/stopped/fast
	db $0D
	
	db "PLAYER",$0D
	db "INVULNERABILITY.........",$0D ; no/yes
	db "UNLIMITED AMMO..........",$0D ; no/yes
	db "MOVEMENT SPEED..........",$0D ; normal/faster/turbo/slow
	db $0D
	
	db "ENVIRONMENT",$0D
	db "ALL DOORS ARE UNLOCKED..",$0D ; no/yes
	db "LIFTS NEED PASS.........",$0D ; yes/no
	db "SAFE EXPLOSIVES.........",$0D ; no/yes
	db $0D
	
	db "HOTKEYS",$0D
	db "LIFT HOTKEY -L-.........",$0D ; disabled/enabled
	db "NUKE HOTKEY -N-.........",$0D ; disabled/enabled 
	db "GHOST MODE HOTKEY -G-...",$0D ; disabled/enabled
	db $0D
	db "              BACK"
	db 0

TXT_CHEAT_NO_YES
	db "......NO"
TXT_CHEAT_YES_NO
	db ".....YES"
	db "......NO"
TXT_CHEAT_ENEMY_SPEED
	db "..NORMAL"
	db "....SLOW"
	db ".STOPPED"
	db "....FAST"
TXT_CHEAT_PLAYER_SPEED
	db "..NORMAL"
	db "..FASTER"
	db "...TURBO"
	db "....SLOW"
TXT_CHEAT_DISABLED_ENABLED
	db "DISABLED"
	db ".ENABLED"
	cmap
	;
	; === GFX DATA ===
	;
y_apostrophe	; 32*8 h none
	db $00,$00,$00,$00,$00,$03,$80,$C0
	db $00,$00,$00,$00,$00,$C0,$C0,$61
	db $00,$00,$00,$00,$03,$F3,$C6,$80
	db $31,$18,$0C,$0C,$0C,$1E,$00,$00

charset_1x1
	include "charset_1x1.asm"

sprite_char_pos_index
	db $29,$2A,$2B,$2C			; position indexes
	db $51,$52,$53,$54
	db $79,$7A,$7B,$7C
	db $A1,$A2,$A3,$A4
	
player_sprite_chars
	db $E0,$E1,$E2,$E3, $E4,$E5,$E6,$E7, $E8,$E9,$EA,$EB, $EC,$ED,$EE,$EF
	
prisoner_gfx_lo
	db prisoner_gfx_5 & 255
	db prisoner_gfx_6 & 255
	db prisoner_gfx_1 & 255
	db prisoner_gfx_2 & 255
	db prisoner_gfx_7 & 255
	db prisoner_gfx_8 & 255
	db prisoner_gfx_3 & 255
	db prisoner_gfx_4 & 255
prisoner_gfx_hi
	db prisoner_gfx_5 >> 8
	db prisoner_gfx_6 >> 8
	db prisoner_gfx_1 >> 8
	db prisoner_gfx_2 >> 8
	db prisoner_gfx_7 >> 8
	db prisoner_gfx_8 >> 8
	db prisoner_gfx_3 >> 8
	db prisoner_gfx_4 >> 8

prisoner_gfx_1	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$44
	db $16,$15,$15,$84,$61,$51,$51,$44
	db $16,$15,$15,$84,$61,$51,$51,$44
	db $16,$15,$15,$84,$61,$51,$51,$44

	db $01,$05,$1F,$1E,$1E,$1F,$1F,$15
	db $05,$17,$5F,$5F,$5F,$5F,$1F,$1F
	db $51,$F5,$FD,$FD,$FD,$FD,$FD,$FD
	db $15,$51,$F5,$B4,$B4,$F4,$F4,$54

	db $1D,$07,$01,$40,$50,$50,$51,$44
	db $17,$45,$55,$74,$1E,$00,$00,$10
	db $F5,$54,$51,$0F,$AA,$00,$15,$56
	db $14,$75,$B4,$11,$AE,$00,$40,$C0

	db $15,$15,$15,$84,$61,$51,$51,$48
	db $00,$10,$14,$44,$51,$51,$51,$48
	db $5F,$15,$00,$44,$51,$51,$51,$48
	db $46,$05,$15,$44,$61,$51,$51,$48
prisoner_gfx_2	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$44
	db $16,$15,$15,$84,$61,$51,$51,$44
	db $16,$15,$15,$84,$61,$51,$51,$44
	db $16,$15,$15,$84,$61,$51,$51,$44

	db $01,$05,$1F,$1E,$1E,$1F,$1F,$15
	db $05,$17,$5F,$5F,$5F,$5F,$1F,$1F
	db $51,$F5,$FD,$FD,$FD,$FD,$FD,$FD
	db $15,$51,$F5,$B4,$B4,$F4,$F4,$54

	db $1D,$07,$01,$40,$00,$41,$51,$41
	db $17,$C5,$55,$74,$1D,$40,$55,$FD
	db $F5,$54,$51,$0F,$AA,$00,$00,$00
	db $14,$75,$B4,$11,$AE,$00,$00,$00

	db $11,$10,$14,$84,$61,$51,$51,$48
	db $7D,$54,$00,$44,$51,$51,$51,$48
	db $01,$15,$15,$44,$51,$51,$51,$48
	db $16,$15,$15,$44,$61,$51,$51,$48
prisoner_gfx_3	; 32*32 m none
	db $15,$15,$15,$40,$45,$1E,$1E,$1F
	db $15,$15,$15,$04,$41,$D1,$D4,$DD
	db $16,$15,$15,$84,$91,$91,$91,$54
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $15,$10,$05,$1F,$1F,$1F,$1F,$1F
	db $56,$01,$50,$F4,$FD,$FD,$FD,$FD
	db $DD,$DF,$D7,$95,$95,$95,$94,$80
	db $16,$15,$15,$04,$21,$11,$11,$48

	db $1F,$1F,$1F,$17,$05,$10,$15,$1E
	db $FD,$FD,$F5,$D4,$50,$01,$C5,$C7
	db $80,$81,$95,$C4,$11,$85,$41,$44
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $1E,$1F,$05,$00,$40,$51,$51,$48
	db $DD,$D4,$51,$04,$11,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
prisoner_gfx_4	; 32*32 m none
	db $15,$15,$15,$40,$45,$1E,$1E,$1F
	db $15,$15,$15,$04,$41,$D1,$D4,$DD
	db $16,$15,$15,$84,$91,$91,$91,$D8
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $15,$10,$05,$1F,$1F,$1F,$1F,$1F
	db $57,$01,$50,$F4,$FD,$FD,$FD,$FD
	db $D6,$D5,$D5,$84,$B1,$B1,$B1,$80
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $1F,$1F,$1F,$17,$05,$10,$15,$1E
	db $FD,$FD,$F5,$D4,$50,$01,$C5,$C7
	db $94,$9D,$95,$D5,$15,$95,$D4,$50
	db $16,$15,$15,$04,$21,$11,$51,$48

	db $1E,$1F,$05,$00,$40,$51,$51,$48
	db $DD,$D4,$51,$04,$11,$51,$51,$48
	db $02,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
prisoner_gfx_5	; 32*32 m none
	db $16,$15,$15,$84,$60,$51,$56,$45
	db $16,$15,$15,$84,$01,$51,$F5,$F4
	db $16,$15,$15,$84,$61,$51,$51,$44
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $05,$00,$AA,$01,$07,$1F,$15,$15
	db $54,$00,$AA,$40,$D1,$47,$1F,$1F
	db $16,$00,$9D,$17,$51,$F4,$FD,$FD
	db $16,$15,$15,$44,$D1,$74,$15,$54

	db $1F,$1E,$1E,$1F,$1F,$05,$00,$40
	db $5F,$5F,$1F,$1F,$17,$15,$05,$00
	db $FD,$FD,$FD,$FD,$F5,$55,$54,$00
	db $F5,$B4,$B5,$F4,$F5,$51,$01,$04

	db $16,$15,$15,$84,$61,$51,$51,$48
	db $00,$15,$15,$84,$61,$51,$51,$48
	db $00,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
prisoner_gfx_6	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$60,$45,$1F,$17
	db $16,$15,$15,$84,$21,$41,$D1,$D0

	db $15,$00,$AA,$01,$07,$1F,$15,$15
	db $15,$00,$AA,$40,$D1,$47,$1F,$1F
	db $15,$01,$9C,$17,$51,$F4,$FD,$FD
	db $D1,$51,$11,$44,$C1,$74,$15,$54

	db $1F,$1E,$1E,$1F,$1F,$05,$00,$40
	db $5F,$5F,$1F,$1F,$17,$15,$05,$00
	db $FD,$FD,$FD,$FD,$F5,$55,$54,$00
	db $F5,$B4,$B5,$F4,$F5,$51,$01,$04

	db $16,$15,$15,$84,$61,$51,$51,$48
	db $00,$15,$15,$84,$61,$51,$51,$48
	db $00,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
prisoner_gfx_7	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$40,$11
	db $16,$15,$15,$80,$45,$17,$57,$97
	db $16,$15,$15,$04,$51,$F4,$B5,$B4

	db $16,$15,$15,$84,$61,$51,$51,$48
	db $03,$11,$02,$42,$02,$02,$02,$02
	db $45,$40,$05,$1F,$7F,$7F,$7F,$7F
	db $55,$04,$51,$D4,$F5,$F4,$F5,$F4

	db $15,$14,$14,$84,$60,$50,$50,$48
	db $02,$12,$52,$D2,$D2,$52,$52,$52
	db $7F,$7F,$5F,$15,$05,$40,$D5,$77
	db $F5,$F4,$D5,$54,$51,$04,$55,$B4

	db $16,$15,$15,$84,$61,$51,$51,$48
	db $12,$02,$02,$42,$50,$51,$51,$44
	db $17,$07,$01,$00,$00,$40,$51,$44
	db $B5,$F4,$51,$04,$11,$51,$51,$44
prisoner_gfx_8	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$40,$11
	db $16,$15,$15,$80,$45,$17,$57,$97
	db $16,$15,$15,$04,$51,$F4,$B5,$B4

	db $14,$14,$14,$84,$60,$50,$50,$48
	db $53,$71,$72,$52,$52,$52,$12,$02
	db $45,$40,$05,$1F,$7F,$7F,$7F,$7F
	db $55,$04,$51,$D4,$F5,$F4,$F5,$F4

	db $16,$15,$15,$84,$61,$51,$51,$48
	db $02,$02,$02,$02,$02,$42,$42,$42
	db $7F,$7F,$5F,$15,$05,$40,$D5,$77
	db $F5,$F4,$D5,$54,$51,$04,$55,$B4

	db $16,$15,$15,$84,$61,$51,$51,$48
	db $02,$02,$02,$42,$50,$51,$51,$44
	db $17,$07,$01,$00,$00,$40,$51,$44
	db $B5,$F4,$51,$04,$11,$51,$51,$44
	;
player_up_1	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$40
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $16,$15,$AA,$00,$60,$53,$4F,$3C
	db $16,$15,$AA,$AA,$00,$C3,$CF,$3F
	db $0F,$3F,$03,$8C,$3E,$CF,$F0,$FC
	db $16,$C5,$C5,$C4,$01,$81,$E1,$3C

	db $3F,$3B,$3B,$3F,$3F,$3F,$4F,$40
	db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$0F
	db $EC,$EC,$FC,$FC,$FC,$FC,$FC,$F0
	db $FC,$EC,$EC,$FC,$FC,$FC,$F1,$08

	db $14,$13,$13,$83,$63,$50,$51,$48
	db $00,$8F,$CF,$CF,$CF,$00,$51,$48
	db $00,$E3,$F3,$F3,$F3,$00,$51,$48
	db $16,$85,$C5,$C4,$C1,$11,$51,$48
player_up_2	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$08
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $14,$13,$AA,$00,$60,$53,$4F,$3C
	db $F2,$FD,$AA,$AA,$00,$C3,$CF,$3F
	db $16,$15,$15,$80,$3E,$CF,$F0,$FC
	db $16,$15,$15,$84,$21,$81,$E1,$3C

	db $3F,$3B,$3B,$3F,$3F,$3F,$4F,$40
	db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$0F
	db $EC,$EC,$FC,$FC,$FC,$FC,$FC,$F0
	db $FC,$EC,$EC,$FC,$FC,$FC,$F1,$08

	db $14,$13,$13,$83,$63,$50,$51,$48
	db $00,$8F,$CF,$CF,$CF,$00,$51,$48
	db $00,$E3,$F3,$F3,$F3,$00,$51,$48
	db $16,$85,$C5,$C4,$C1,$11,$51,$48
player_up_shoot	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$08
	db $16,$15,$15,$84,$61,$51,$53,$42
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $16,$15,$14,$83,$63,$53,$53,$43
	db $10,$0F,$F3,$EC,$FC,$F3,$CF,$3F
	db $02,$E2,$F2,$C2,$32,$C2,$F2,$FE
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $30,$F0,$C0,$0C,$3C,$7C,$4C,$4F
	db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$0F
	db $EE,$EE,$FE,$FE,$FE,$FF,$FC,$F3
	db $16,$15,$15,$84,$21,$91,$91,$08

	db $13,$13,$14,$84,$61,$51,$51,$48
	db $00,$F0,$C0,$3C,$39,$31,$51,$48
	db $0F,$FB,$FB,$FF,$3F,$3F,$4C,$40
	db $16,$15,$15,$04,$21,$11,$51,$48
player_up_flash	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$08
	db $17,$12,$0A,$8A,$4A,$52,$53,$42
	db $16,$15,$85,$84,$81,$11,$51,$48

	db $16,$15,$14,$83,$63,$53,$53,$43
	db $10,$0F,$F3,$EC,$FC,$F3,$CF,$3F
	db $02,$E2,$F2,$C2,$32,$C2,$F2,$FE
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $30,$F0,$C0,$0C,$3C,$7C,$4C,$4F
	db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$0F
	db $EE,$EE,$FE,$FE,$FE,$FF,$FC,$F3
	db $16,$15,$15,$84,$21,$91,$91,$08

	db $13,$13,$14,$84,$61,$51,$51,$48
	db $00,$F0,$C0,$3C,$39,$31,$51,$48
	db $0F,$FB,$FB,$FF,$3F,$3F,$4C,$40
	db $16,$15,$15,$04,$21,$11,$51,$48
player_down_1	; 32*32 m none
	db $16,$15,$15,$84,$63,$53,$53,$43
	db $16,$15,$15,$00,$8F,$CF,$CF,$CF
	db $16,$15,$15,$00,$E3,$F3,$F3,$F3
	db $16,$15,$15,$04,$81,$C1,$C1,$C8

	db $14,$10,$0F,$3B,$3B,$3F,$3F,$3F
	db $00,$0F,$3F,$3F,$3F,$3F,$3F,$3F
	db $00,$F3,$AC,$EC,$FC,$FC,$FC,$FC
	db $16,$C5,$F1,$EC,$EC,$FC,$FC,$FC

	db $30,$3F,$0E,$83,$60,$50,$53,$43
	db $3F,$3F,$CF,$B3,$C0,$3E,$23,$C0
	db $FC,$FC,$F3,$CE,$00,$AA,$AA,$00
	db $0C,$FC,$B1,$C4,$31,$11,$AA,$00

	db $13,$14,$15,$84,$61,$51,$51,$48
	db $FC,$F1,$05,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
player_down_2	; 32*32 m none
	db $16,$15,$15,$84,$63,$53,$53,$43
	db $16,$15,$15,$00,$8F,$CF,$CF,$CF
	db $16,$15,$15,$00,$E3,$F3,$F3,$F3
	db $16,$15,$15,$04,$81,$C1,$C1,$C8

	db $14,$10,$0F,$3B,$3B,$3F,$3F,$3F
	db $00,$0F,$3F,$3F,$3F,$3F,$3F,$3F
	db $00,$F3,$AC,$EC,$FC,$FC,$FC,$FC
	db $16,$C5,$F1,$EC,$EC,$FC,$FC,$FC

	db $30,$3F,$0E,$83,$60,$51,$51,$48
	db $3F,$3F,$CF,$B3,$C0,$3E,$43,$40
	db $FC,$FC,$F3,$CE,$00,$AA,$AA,$00
	db $0C,$FC,$B1,$C4,$31,$11,$AA,$00

	db $16,$15,$15,$84,$61,$51,$51,$48
	db $14,$15,$15,$84,$61,$51,$51,$48
	db $3F,$0F,$10,$84,$61,$51,$51,$48
	db $C6,$15,$15,$84,$61,$51,$51,$48
player_down_shoot	; 32*32 m none
	db $16,$15,$14,$83,$63,$53,$53,$43
	db $16,$15,$C4,$B0,$EC,$EB,$FB,$FF
	db $16,$35,$E5,$CC,$0B,$3E,$3E,$0F
	db $16,$15,$15,$84,$61,$51,$D1,$88

	db $13,$13,$0C,$8F,$62,$52,$52,$42
	db $30,$0F,$3F,$3F,$3F,$3F,$3F,$3F
	db $03,$F0,$AC,$EC,$FC,$FC,$FC,$FC
	db $D6,$35,$E5,$F4,$C1,$11,$51,$08

	db $12,$12,$12,$82,$62,$52,$52,$42
	db $3F,$3F,$0F,$03,$30,$C0,$3F,$FF
	db $FC,$FC,$F3,$CF,$0F,$0F,$C3,$F0
	db $C6,$85,$85,$C4,$C1,$C1,$11,$48

	db $12,$12,$11,$84,$61,$51,$51,$48
	db $00,$15,$15,$84,$61,$51,$51,$48
	db $06,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
player_down_flash	; 32*32 m none
	db $16,$15,$14,$83,$63,$53,$53,$43
	db $16,$15,$C4,$B0,$EC,$EB,$FB,$FF
	db $16,$35,$E5,$CC,$0B,$3E,$3E,$0F
	db $16,$15,$15,$84,$61,$51,$D1,$88

	db $13,$13,$0C,$8F,$62,$52,$52,$42
	db $30,$0F,$3F,$3F,$3F,$3F,$3F,$3F
	db $03,$F0,$AC,$EC,$FC,$FC,$FC,$FC
	db $D6,$35,$E5,$F4,$C1,$11,$51,$08

	db $12,$12,$12,$82,$62,$52,$52,$42
	db $3F,$3F,$0F,$03,$30,$C0,$3F,$FF
	db $FC,$FC,$F3,$CF,$0F,$0F,$C3,$F0
	db $C6,$85,$85,$C4,$C1,$C1,$11,$48

	db $12,$12,$11,$82,$4A,$4A,$4A,$42
	db $00,$15,$15,$04,$81,$81,$81,$08
	db $06,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
player_left_1	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$60,$4F,$3F,$EF
	db $16,$15,$15,$84,$01,$F1,$FD,$EC
	db $16,$15,$15,$84,$61,$51,$51,$08

	db $13,$13,$12,$80,$62,$52,$52,$42
	db $BF,$CF,$C0,$03,$0F,$3F,$3F,$3F
	db $EC,$FC,$0C,$F0,$FC,$BC,$BC,$FC
	db $E2,$F1,$F1,$00,$E1,$F1,$F1,$F0

	db $02,$32,$F2,$F2,$F2,$F2,$F2,$32
	db $3F,$3F,$3F,$3F,$0F,$03,$C0,$F3
	db $FC,$FC,$FC,$FC,$FC,$F0,$0C,$FC
	db $F2,$F1,$F1,$F0,$F1,$01,$E1,$F0

	db $02,$12,$12,$82,$62,$51,$51,$48
	db $EF,$3F,$0F,$83,$60,$51,$51,$48
	db $AC,$EC,$FC,$F0,$01,$51,$51,$48
	db $F2,$05,$15,$84,$61,$51,$51,$48
player_left_2	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$60,$4F,$3F,$EF
	db $16,$15,$15,$84,$01,$F1,$FD,$EC
	db $16,$15,$15,$84,$61,$51,$51,$08

	db $13,$03,$32,$F0,$F2,$F2,$F2,$F2
	db $BF,$CF,$C0,$03,$0F,$3F,$3F,$3F
	db $EC,$FC,$0C,$F0,$FC,$BC,$BC,$FC
	db $E2,$F1,$F1,$00,$E1,$F1,$F1,$F0

	db $32,$02,$12,$82,$62,$52,$52,$42
	db $3F,$3F,$3F,$3F,$0F,$03,$C0,$F3
	db $FC,$FC,$FC,$FC,$FC,$F0,$0C,$FC
	db $F2,$F1,$F1,$F0,$F1,$01,$E1,$F0

	db $12,$12,$12,$82,$62,$51,$51,$48
	db $EF,$3F,$0F,$83,$60,$51,$51,$48
	db $AC,$EC,$FC,$F0,$01,$51,$51,$48
	db $F2,$05,$15,$84,$61,$51,$51,$48
player_left_shoot	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$40
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $00,$2A,$03,$8C,$4D,$4C,$4C,$4C
	db $00,$AA,$00,$C3,$0F,$3F,$3F,$3F
	db $3C,$8F,$0C,$F0,$FC,$BC,$BC,$FC
	db $02,$FC,$FF,$3B,$3F,$EF,$EC,$FC

	db $0C,$0C,$0C,$8C,$4F,$4F,$4F,$43
	db $3F,$3F,$3F,$3F,$0F,$C3,$F0,$EC
	db $FC,$FC,$FC,$FC,$FC,$F0,$03,$03
	db $FC,$31,$01,$08,$0F,$CE,$CF,$F0

	db $14,$15,$15,$84,$61,$51,$51,$48
	db $F0,$00,$14,$84,$61,$51,$51,$48
	db $0F,$33,$FC,$F8,$3C,$41,$51,$48
	db $F2,$C5,$15,$84,$61,$51,$51,$48
player_left_flash	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$40
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$51,$51,$40
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $28,$AA,$2B,$80,$4D,$4C,$4C,$4C
	db $00,$AA,$00,$C3,$0F,$3F,$3F,$3F
	db $3C,$8F,$0C,$F0,$FC,$BC,$BC,$FC
	db $02,$FC,$FF,$3B,$3F,$EF,$EC,$FC

	db $0C,$0C,$0C,$8C,$4F,$4F,$4F,$43
	db $3F,$3F,$3F,$3F,$0F,$C3,$F0,$EC
	db $FC,$FC,$FC,$FC,$FC,$F0,$03,$03
	db $FC,$31,$01,$08,$0F,$CE,$CF,$F0

	db $14,$15,$15,$84,$61,$51,$51,$48
	db $F0,$00,$14,$84,$61,$51,$51,$48
	db $0F,$33,$FC,$F8,$3C,$41,$51,$48
	db $F2,$C5,$15,$84,$61,$51,$51,$48
player_right_1	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$50,$4E
	db $16,$15,$15,$84,$60,$4F,$3E,$3E
	db $16,$15,$15,$84,$21,$C0,$F0,$FC
	db $16,$15,$15,$84,$A1,$91,$91,$88

	db $0F,$0F,$00,$8E,$4F,$4F,$4F,$4F
	db $3F,$3F,$30,$0F,$3F,$3F,$3F,$3F
	db $CB,$CE,$0F,$C0,$F0,$BC,$BC,$FC
	db $96,$95,$95,$84,$A1,$91,$91,$88

	db $0F,$0F,$0F,$8F,$4F,$40,$4E,$4F
	db $3F,$3F,$3F,$3F,$3F,$0F,$30,$3F
	db $FC,$FC,$FC,$FC,$F0,$C0,$00,$CC
	db $B2,$BD,$BD,$BC,$BD,$BD,$31,$88

	db $0F,$10,$15,$84,$61,$51,$51,$48
	db $3E,$3E,$3F,$8F,$60,$51,$51,$48
	db $CB,$FF,$FC,$F0,$01,$51,$51,$48
	db $D6,$15,$15,$84,$61,$51,$51,$48
player_right_2	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$50,$4E
	db $16,$15,$15,$84,$60,$4F,$3E,$3E
	db $16,$15,$15,$84,$21,$C0,$F0,$FC
	db $16,$15,$15,$84,$A1,$91,$91,$88

	db $0F,$0F,$00,$8E,$4F,$4F,$4F,$4F
	db $3F,$3F,$30,$0F,$3F,$3F,$3F,$3F
	db $CB,$CE,$0F,$C0,$F0,$BC,$BC,$FC
	db $86,$B1,$BD,$BC,$BD,$BD,$BD,$BC

	db $0F,$0F,$0F,$8F,$4F,$40,$4E,$4F
	db $3F,$3F,$3F,$3F,$3F,$0F,$30,$3F
	db $FC,$FC,$FC,$FC,$F0,$C0,$00,$CC
	db $B2,$85,$95,$84,$A1,$91,$11,$88

	db $0F,$10,$15,$84,$61,$51,$51,$48
	db $3E,$3E,$3F,$8F,$60,$51,$51,$48
	db $CB,$FF,$FC,$F0,$01,$51,$51,$48
	db $D6,$15,$15,$84,$61,$51,$51,$48
player_right_shoot	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$50,$4F,$3E
	db $16,$15,$15,$04,$F1,$E1,$2D,$3C
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $14,$12,$0F,$8F,$43,$50,$50,$4F
	db $FF,$3F,$30,$CF,$3F,$3F,$3F,$3F
	db $00,$00,$3F,$CF,$F3,$BC,$BC,$FC
	db $16,$05,$C1,$B0,$B1,$F1,$F1,$F0

	db $3F,$3F,$3F,$FF,$FF,$EC,$F0,$FF
	db $3F,$3F,$3F,$3F,$3F,$0F,$40,$3C
	db $FC,$FC,$FC,$FC,$F1,$C1,$11,$4C
	db $32,$31,$31,$30,$31,$31,$31,$F0

	db $3F,$00,$15,$84,$61,$51,$51,$48
	db $F2,$3C,$01,$84,$61,$51,$51,$48
	db $AA,$00,$15,$84,$61,$51,$51,$48
	db $A8,$05,$15,$84,$61,$51,$51,$48
player_right_flash	; 32*32 m none
	db $16,$15,$15,$84,$61,$51,$51,$48
	db $16,$15,$15,$84,$61,$50,$4F,$3E
	db $16,$15,$15,$04,$F1,$E1,$2D,$3C
	db $16,$15,$15,$84,$61,$51,$51,$48

	db $14,$12,$0F,$8F,$43,$50,$50,$4F
	db $FF,$3F,$30,$CF,$3F,$3F,$3F,$3F
	db $00,$00,$3F,$CF,$F3,$BC,$BC,$FC
	db $16,$05,$C1,$B0,$B1,$F1,$F1,$F0

	db $3F,$3F,$3F,$FF,$FF,$EC,$F0,$FF
	db $3F,$3F,$3F,$3F,$3F,$0F,$40,$3C
	db $FC,$FC,$FC,$FC,$F1,$C1,$11,$4C
	db $32,$31,$31,$30,$31,$31,$31,$E8

	db $3F,$00,$15,$84,$61,$51,$51,$48
	db $F2,$3C,$01,$84,$61,$51,$51,$48
	db $AA,$00,$15,$84,$61,$51,$51,$48
	db $AA,$28,$01,$84,$61,$51,$51,$48
; EOF